<?php
if ( isset( $_GET['_phpinfo'] ) ) {
	phpinfo();
	exit();
}

if ( ! isset( $time_start ) ) {
	$time_start = microtime( true );
}

@header( 'Cache-Control: no-store, no-cache, must-revalidate' );
@header( 'Cache-Control: post-check=0, pre-check=0', false );
@header( 'Pragma: no-cache' );
@header( "Expires: Mon, 26 Jul 1997 05:00:00 GMT" );
@header( "Expires: 0" );
@header( "Cache-Control: must-revalidate, post-check=0, pre-check=0" );

defined( 'SESSION_TIMEOUT' ) OR define( 'SESSION_TIMEOUT', 1440 );

@ini_set( 'session.gc_maxlifetime', SESSION_TIMEOUT );
@ini_set( 'session.gc_probability', 1 );
@ini_set( 'session.gc_divisor', 1 );

$profiler_array = [];
$sum_time       = 0;

if ( isset( $_GET['_version'] ) ) {
	header( 'Content-Type: text/plain' );
	echo VERSION;
	exit();
}

include_once( __DIR__ . DIRECTORY_SEPARATOR . "_constants.php" );

if ( is_file( __DIR__ . DIRECTORY_SEPARATOR . "config.php" ) ) {
	include_once( __DIR__ . DIRECTORY_SEPARATOR . "config.php" );
}

$require = require __DIR__ . DIRECTORY_SEPARATOR . "vendor" . DIRECTORY_SEPARATOR . "autoload.php";
if ( is_file( __DIR__ . DIRECTORY_SEPARATOR . 'library' . DIRECTORY_SEPARATOR . "classes" . DIRECTORY_SEPARATOR . "Student.php" ) ) {
	$require->addPsr4( '', __DIR__ . DIRECTORY_SEPARATOR . 'library' . DIRECTORY_SEPARATOR . 'classes' );
} else {
	$require->addPsr4( '', __DIR__ . DIRECTORY_SEPARATOR . 'source' . DIRECTORY_SEPARATOR . 'library' . DIRECTORY_SEPARATOR . 'classes' );
}


if ( ! is_developer_machine() ) {
	## Removing config.php file if Database name is fekara_client and if it is developer machine
	if ( defined( DBNAME ) && DBNAME == 'fekara_client' && DBSERVER == '127.0.0.1' ) {
		@unlink( __DIR__ . DIRECTORY_SEPARATOR . 'config.php' );
	}

	if ( is_file( __DIR__ . DIRECTORY_SEPARATOR . 'composer.json' ) ) {
		@unlink( __DIR__ . DIRECTORY_SEPARATOR . 'composer.json' );
	}

	if ( is_dir( __DIR__ . DIRECTORY_SEPARATOR . '.idea' ) ) {
		\File::rrmdir( __DIR__ . DIRECTORY_SEPARATOR . '.idea' );
	}
}

//}

/**
 * @return bool
 *
 * Check if code is running on .fekara.com server.
 */
if ( ! function_exists( "is_fekara_server" ) ) {
	function is_fekara_server() {
		if ( ! isset( $_SERVER["HTTP_HOST"] ) ) {
			return false;
		}

		$server = $_SERVER["HTTP_HOST"];

		return substr( $server, - 11 ) == ".fekara.com";
	}
}

if ( is_fekara_server() ) {
	$path = __DIR__ . DIRECTORY_SEPARATOR . "source";

	if ( ! is_link( $path ) ) {
		$target = realpath( __DIR__ . DIRECTORY_SEPARATOR . ".." . DIRECTORY_SEPARATOR . "__files" . DIRECTORY_SEPARATOR );
		@symlink( $target, $path );
	}
}

if ( ! isset( $API_CALL ) ) {
	$API_CALL = false;
}

if ( ( ! $API_CALL ) && ( ! is_fekara_server() ) && ( ! defined( 'ALLOK' ) ) ) {
	header( "location: requirements.php" );
	exit;
}

if ( ! $API_CALL && ( ! is_fekara_server() ) && version_compare( PHP_VERSION, REQUIRED_PHP, "<" ) ) {
	//header( "location: requirements.php" );
	//exit;
	echo "<h1>PHP version required " . REQUIRED_PHP . " or later</h1>";
	exit;
}

//// Initialize Application
@session_start();


//if ( is_fekara_server() ) {
//	define( "ASSETS_PATH", ROOT_PATH . "source/assets/" );
//	define( "ASSETS_URL", DOMAIN_URL . "source/assets/" );
//	define( "LIBRARY_PATH", ROOT_PATH . "source/library/" );
//} else {
define( "ASSETS_PATH", ROOT_PATH . "assets" . DIRECTORY_SEPARATOR );
define( "ASSETS_URL", DOMAIN_URL . "assets/" );
define( "LIBRARY_PATH", ROOT_PATH . "library" . DIRECTORY_SEPARATOR );
//}


if ( ! $API_CALL && ( ! is_fekara_server() ) && ! defined( "ALLOK" ) ) {
	header( "location: requirements.php" );
	exit;
}

if ( $_SERVER["HTTP_HOST"] <> "fekara.pk" && ( ! $API_CALL ) && ( ! defined( "ALLOK" ) ) ) {
	header( "location: install.php" );
	exit;
}

if ( $_SERVER["HTTP_HOST"] <> "fekara.pk" && ( ! $API_CALL ) && ( ! defined( "DBSERVER" ) ) ) {
	header( "location: install.php" );
	exit;
}

if ( ! defined( 'DBSERVER' ) && basename( $_SERVER["SCRIPT_FILENAME"] ) <> "install.php" ) {
	header( "location: install.php" );
	exit;
}

if ( defined( 'DISPLAY_ERRORS' ) && ( DISPLAY_ERRORS || DISPLAY_ERRORS == 1 ) ) {
	error_reporting( E_ALL & ~E_STRICT );
	@ini_set( 'display_errors', 1 );
	@ini_set( 'log_errors', 1 );
}
//@ini_set( 'error_log', ROOT_PATH . 'D:/errors.txt' );

date_default_timezone_set( TIME_ZONE );

if ( ! function_exists( "curl_init" ) ) {
	die( "<h1>This application requires \"curl_init\" function." );
}

if ( ! function_exists( "curl_exec" ) ) {
	die( "<h1>This application requires \"curl_exec()\" function." );
}

if ( ! class_exists( "ZipArchive" ) ) {
	die( "<h1>This application requires \"ZipArchive\" extension." );
}

if ( ! is_dir( ROOT_PATH . "contents/" ) ) {
	@mkdir( ROOT_PATH . "contents/" );
}

if ( ! is_dir( ROOT_PATH . "contents/" ) ) {
	die( "Folder [" . ROOT_PATH . "contents] doesn't exists!" );
}

if ( ! file_exists( ROOT_PATH . "contents/index.html" ) ) {
	@file_put_contents( ROOT_PATH . "contents/index.html", "" );
}

## Create if temporary folder doesn't exists
if ( ! is_dir( TMP_PATH ) ) {
	@mkdir( rtrim( TMP_PATH, DIRECTORY_SEPARATOR ), 0777, true );
}

if ( ! is_dir( STUDENT_FILES ) ) {
	@mkdir( rtrim( STUDENT_FILES, DIRECTORY_SEPARATOR ), 0777, true );
}

if ( ! $API_CALL && ! is_dir( STUDENT_FILES ) ) {
	die( "Folder [" . STUDENT_FILES . "] doesn't exists, Please create this folder." );
}

if ( ! file_exists( STUDENT_FILES . "index.html" ) ) {
	@file_put_contents( STUDENT_FILES . "index.html", "" );
}

if ( ! is_dir( TEACHER_FILES ) ) {
	@mkdir( rtrim( TEACHER_FILES, DIRECTORY_SEPARATOR ), 0777, true );
}
if ( ! $API_CALL && ! is_dir( TEACHER_FILES ) ) {
	die( "Folder [" . TEACHER_FILES . "] doesn't exists, Please create this folder." );
}
if ( ! file_exists( TEACHER_FILES . "index.html" ) ) {
	@file_put_contents( TEACHER_FILES . "index.html", "" );
}

if ( ! is_dir( ADMIN_FILES ) ) {
	@mkdir( rtrim( ADMIN_FILES, DIRECTORY_SEPARATOR ), 0777, true );
}
if ( ! $API_CALL && ! is_dir( ADMIN_FILES ) ) {
	die( "Folder [" . ADMIN_FILES . "] doesn't exists, Please create this folder." );
}
if ( ! file_exists( ADMIN_FILES . "index.html" ) ) {
	@file_put_contents( ADMIN_FILES . "index.html", "" );
}


//function my_autoloader( $class ) {
//	$class   = trim( $class, "/" );
//	$class   = trim( $class, '\\' );
//	$replace = [
//		"()" => "",
//		'\\' => "/",
//	];
//	$class   = strtr( $class, $replace );
//	//str_replace( array_keys( $replace ), array_values( $replace ), $class );
//	//die(LIBRARY_PATH . 'classes/' . $class . '.php');
//	if ( is_file( LIBRARY_PATH . 'classes' . DIRECTORY_SEPARATOR . $class . '.php' ) ) {
//		include_once( LIBRARY_PATH . 'classes' . DIRECTORY_SEPARATOR . $class . '.php' );
//	}
//}
//spl_autoload_register( 'my_autoloader' );

## Connecting DB
## https://github.com/ThingEngineer/PHP-MySQLi-Database-Class
$db2 = new \MysqliDb ( [
	'host'     => DBSERVER,
	'username' => DBUSER,
	'password' => DBPASS,
	'db'       => DBNAME,
	'port'     => 3306,
	'charset'  => 'utf8',
] );

try {
	$db2->rawQuery( "SET sql_mode=''" );
} catch ( Exception $e ) {
}

$db = new \DB\Select( DBUSER, DBPASS, DBNAME, DBSERVER );
//$db->query( "SET sql_mode=''" );

register_shutdown_function( function () {
	if ( \General::localhost() ) {
		$errorData = error_get_last();
		if ( is_array( $errorData ) ) {
			@ob_end_clean();
			file_put_contents( __DIR__ . '/errors.txt', 'Time: ' . date( 'd-M-Y H:i' ) . "\n" . print_r( $errorData, true ), FILE_APPEND );
		}
	}
} );

//$translator->addResource('array', array(
//	'Hello World!' => 'Bonjour',
//), 'fr_FR');

//echo $translator->trans( 'Let’s make the web multilingual.' );

//$options_table      = $db->options_table_array();
// $translations_array = \Translation::translations_table_array();

## Initializing MySQL DB;
new MySQL();

$Student      = new \Student();
$BatchClass   = new \BatchClass();
$Active_Batch = $Student->get_active_batch();
$All_Batches  = null;

defined( 'CURRENCY' ) OR define( 'CURRENCY', 'Rs.' );
defined( 'CURRENCY_DEC' ) OR define( 'CURRENCY_DEC', 2 );

include_once( LIBRARY_PATH . "__functions.php" );
include_once( __DIR__ . "/_init.php" );
/**
 * Initializing Translations
 */

include_once( ASSETS_PATH . 'translations/autoload.php' );

//use Symfony\Component\Translation\Translator;
//use Symfony\Component\Translation\Loader\PoFileLoader;

$Trans  = new \Translation();
$locale = $Trans->get_language();
if ( version_compare( phpversion(), '7.1', '>=' ) ) {


	//use Symfony\Component\Translation\Loader\ArrayLoader;
	$translator = new \Symfony\Component\Translation\Translator( $locale );
	//$locale = Locale::acceptFromHttp($_SERVER['HTTP_ACCEPT_LANGUAGE']);
	//$translator->setFallbackLocale('UR');
	$translator->addLoader( 'po', new \Symfony\Component\Translation\Loader\PoFileLoader() );
	$translator->setFallbackLocales( [ $locale ] );
	$translator->addResource( 'po', $Trans->get_translation_file( $locale ), $locale );
	$translator->getCatalogue( $locale );
	//$translator->addResource( 'pofile', ASSETS_PATH . 'translations/source_files/UR.po', 'fr_FR' );
}

function is_json( $string ) {
	if ( \General::is_really_int( $string ) || $string == '' || empty( $string ) ) {
		return false;
	}
	@json_decode( $string );

	return ( json_last_error() == JSON_ERROR_NONE );
}

defined( 'System' ) OR define( 'System', $db->get_option( "system", "feKara by creativeON" ), true );


/**
 * @param $errno
 * @param $errstr
 * @param $errfile
 * @param $errline
 *
 * @return bool|void
 * @throws ErrorException
 */
//function myErrorHandler( $errno, $errstr, $errfile, $errline ) {
//	if ( ! ( error_reporting() & $errno ) ) {
//		// This error code is not included in error_reporting
//	} else {
//		$u = new \Urls();
//
//		$ErrorServers = [
//			'demo10.fekara.com',
//			'demo17.fekara.com',
//			'ems.ecom.pk',
//			'faunus.fekara.com',
//			'ashlawnschool.fekara.com',
//			'ilearn.org.pk',
//		];
//
//		if ( \General::localhost() ) {
//			$f   = ROOT_PATH . "errors.txt";
//			$str = "Error #: $errno - Error: $errstr - File: $errfile - Line: $errline - Time: " . date( "d-M-Y h:i" ) . " - URL: " . $u->this_url() . ".\n";
//			@file_put_contents( $f, $str, FILE_APPEND );
//			throw new \ErrorException( $errstr, 0, $errno, $errfile, $errline );
//		} else if ( in_array( @$_SERVER['HTTP_HOST'], $ErrorServers ) ) {
//			//$error_code = md5( $errstr . $errfile );
//			$subject = System . ": Fekara-Line # $errline";
//			$body    = "Dear Developer,<br>
//		<br />Error Number: $errno<br>
//		Error: $errstr<br>
//		<br>Error occured on line number $errline of file \"$errfile\"<br>
//		<br>
//		URL: " . $u->this_url() . "
//		From IP: " . \General::myip();
//
//			send_mail( 'shakeel@shakeel.pk', $subject, $body );
//		}
//	}
//}
//$old_error_handler = set_error_handler( 'myErrorHandler' );


//define('DEFAULT_LOCALE', 'ur_PK');

//require_once( ROOT_PATH.'gettext/gettext.inc');
// Implement language settings

//require_once( LIBRARY_PATH . 'modules/gettext/gettext.inc' );
//T_setlocale( LC_ALL, 'en_US' );

if ( ( ! $API_CALL ) && ( $db->get_var( "SELECT COUNT(*) FROM `admins`" ) == 0 ) ) {
	header( "Location: install.php?add_admin_user" );
	exit;
}

if ( is_dir( __DIR__ . '/source' ) ) {
	\File::rrmdir( __DIR__ . '/source' );
}

//if ( is_fekara_server() && is_dir( ROOT_PATH . "assets" ) ) {
//	\File::rrmdir( ROOT_PATH . "assets" );
//}

//if ( is_fekara_server() && is_dir( ROOT_PATH . "library" ) ) {
//	\File::rrmdir( ROOT_PATH . "library" );
//}


if ( \Admins\UserInfo::is_administrator() ) {
	$Permissions = 'all';
} else {
	$Permissions = $db->get_var( 'SELECT GROUP_CONCAT(`permission`) FROM `' . \MySQL\Tables::table_name( 'users_permissions' ) . '` WHERE `admin_id`=' . \Admins\UserInfo::get_my_id() );
	$Permissions = explode( ',', $Permissions );
}