<?php

/**
 * Class Accounts
 *
 * Handle accounts
 */
class Accounts {

	var $table = "account_transactions";
	var $DB;

	function __construct() {
		global $db;
		$this->DB = $db;
	}

	/**
	 * @param       $id
	 * @param array $data
	 *
	 * @return mixed
	 *
	 * Save entry in MySQL table
	 */
	function save_entry( $id, $data = array() ) {
		if ( isset( $data["date"] ) ) {
			$data["date"] = \Dates::mysql_date( $data["date"] );
		}
		$response = $this->DB->update( $this->table, $data, [ "id" => $id ] );

		if ( $response === false ) {
			return $this->DB->last_error;
		} else {
			return "OK";
		}
	}

	/**
	 * @param string $params
	 *
	 * @return string
	 *
	 * Add new entry
	 */
	function add_entry( $params = "" ) {
		$default = [
			"description"    => "",
			"student_id"     => "0",
			"teacher_id"     => "0",
			"employee_id"    => "0",
			"amount"         => "0",
			"date"           => "",
			"added_by"       => "",
			"added_time"     => "",
			"added_ip"       => "",
			"fee_paid_id"    => 0,
			"receipt_number" => "",
			"daily_expense"  => 0,
			"amount_out"     => 0,
			"head"           => "",
		];
		$params  = set_args( $params, $default );
		extract( $params );

		$amount = (float) $params['amount'];
		if ( $amount <= 0 && $params['amount_out'] <= 0 ) {
			return "Amount missing";
		}

		$data["description"]    = $params['description'];
		$student_id             = (int) $params['student_id'];
		$data["student_id"]     = $student_id;
		$teacher_id             = (int) $params['teacher_id'];
		$data["teacher_id"]     = $teacher_id;
		$employee_id            = (int) $params['employee_id'];
		$data["employee_id"]    = $employee_id;
		$data["amount"]         = $amount;
		$data["date"]           = ! \Dates::is_date( $params['date'] ) ? \Dates::mysql_date() : \Dates::mysql_date( $params['date'] );
		$data["added_by"]       = empty( $added_by ) ? ( \Admins\UserInfo::get_my_id() ) : "0";
		$data["added_time"]     = empty( $added_time ) ? \Dates::mysql_datetime() : \Dates::mysql_datetime( $added_time );
		$data["added_ip"]       = empty( $added_ip ) ? \General::myip() : $added_ip;
		$data["fee_paid_id"]    = $params['fee_paid_id'];
		$data["receipt_number"] = $params['receipt_number'];
		$data["daily_expense"]  = $params['daily_expense'];
		$data["amount_out"]     = $params['amount_out'];
		$data["head"]           = $params['head'];

		$response = $this->DB->insert( $this->table, $data );
		if ( $response === false ) {
			return $this->DB->last_error;
		} else {
			return "OK" . $this->DB->insert_id;
		}
	}

	/* Get list of all transactions.
	* Parameters Detail
	* orderby = id is default
	* order = DESC is default
	* student_id = -1 is default and not restriction on stduents
	* fee_paid_id = -1 is default
	* teacher_id = -1 is default
	* employee_id = -1 is default
	* page = 0 is for all pages and default
	* pagsize = 10 is default
	*/
	/**
	 * @param string $params
	 *
	 * @return array
	 *
	 * Get array of transactions
	 */
	function transactions_list( $params = "" ) {
		$default = array(
			"orderby"       => "id",
			"order"         => "DESC",
			"student_id"    => - 1,           # -1 = all students
			"fee_paid_id"   => - 1,          # -1 = any fee paid_id
			"teacher_id"    => - 1,           # -1 = any teacher
			"employee_id"   => - 1,          # -1 = any employee
			"page"          => 0,                  # 0 = all pages
			"pagesize"      => 10,
			"search"        => "",
			"amount"        => - 1,               # -1 for no restriction, specific amount 1500 or 1000,2000 for range
			"date"          => - 1,                 # -1 for no restriction, or range here with comma and single date
			"daily_expense" => - 1,        # -1 for all transactions, 0 for false, 1 for true
			"head"          => "",                 # search in heads
		);
		$params  = set_args( $params, $default );
		extract( $params );
		global $db;

		$Q = "SELECT * FROM `{$this->table}` WHERE 1";
		if ( $params['student_id'] >= 0 ) {
			$Q .= " AND `student_id`={$params['student_id']}";
		}
		if ( $params['fee_paid_id'] >= 0 ) {
			$Q .= " AND `fee_paid_id`={$params['fee_paid_id']}";
		}
		if ( $params['teacher_id'] >= 0 ) {
			$Q .= " AND `teacher_id`={$params['teacher_id']}";
		}
		if ( $params['employee_id'] >= 0 ) {
			$Q .= " AND `employee_id`={$params['employee_id']}";
		}

		if ( $params['amount'] <> "-1" ) {
			$amount = trim( $params['amount'] );
			$amt    = explode( ",", $amount );
			if ( ! isset( $amt[1] ) ) {
				$Q .= " AND `amount`=$amount";
			} else {
				$Q .= " AND (`amount` BETWEEN $amt[0] AND $amt[1])";
			}
		}

		if ( $params['head'] <> "" ) {
			$head = explode( ",", $params['head'] );
			if ( sizeof( $head ) > 0 ) {
				$Q .= " AND (";
				foreach ( $head as $h ) {
					$h = $db->set_string( $h );
					$Q .= "`head`='{$h}' OR ";
				}
				$Q = substr( $Q, 0, - 4 );
				$Q .= ")";
			}
		}

		if ( $params['date'] <> "-1" ) {
			$date = trim( $params['date'] );
			$amt  = explode( ",", $date );
			if ( ! isset( $amt[1] ) ) {
				$Q .= " AND `date`='$date'";
			} else {
				$Q .= " AND (`date` BETWEEN '" . \Dates::mysql_date( $amt[0] ) . "' AND '" . \Dates::mysql_date( $amt[1] ) . "')";
			}
		}

		if ( $params['daily_expense'] <> "-1" ) {
			$Q .= " AND `daily_expense`={$params['daily_expense']}";
		}

		$search = trim( $params['search'] );
		if ( $search <> "" ) {
			$search = $db->set_string( $search );
			$Q      .= " AND (`description` LIKE '%$search%' OR `receipt_number` LIKE '%$search%'";
			if ( \Dates::is_date( $search ) ) {
				$Q .= " OR `date`='" . \Dates::mysql_date( $search ) . "'";
			}
			if ( is_numeric( $search ) ) {
				$Q .= " OR `student_id`=$search OR `teacher_id`=$search OR `employee_id`=$search OR `id`=$search OR `amount`=$search";
				$Q .= " OR YEAR(`date`)=$search";
				$Q .= " OR MONTH(`date`)=$search";
				$Q .= " OR DAY(`date`)=$search";
			}
			$Q .= ")";
		}

		$Q .= " ORDER BY `{$params['orderby']}` {$params['order']}";

		if ( $params['page'] == 0 ) {
			$rows["rows"] = $db->get_rows( $Q );
		} else {
			$rows = $db->get_rows2( $Q, $params['page'], $params['pagesize'] );
		}

		return $rows;
	}

	/**
	 * @param $id
	 *
	 * @return string
	 *
	 * Delete transaction
	 */
	function delete( $id ) {
		if ( ! \Admins\UserInfo::is_allowed( "delete_edit_daily_expenses" ) ) {
			return "You'r not allowed to perform this action";
		}
		$Q = "DELETE FROM `{$this->table}` WHERE `id`=$id";
		global $db;
		$response = $db->query( $Q );
		if ( $response === false ) {
			return "Can't delete record";
		} else {
			return "OK";
		}
	}

	/**
	 * @param $id
	 *
	 * @return mixed
	 *
	 * Get one transaction
	 */
	function get_transaction( $id ) {
		$Q = "SELECT * FROM `{$this->table}` WHERE `id`='$id'";
		global $db;
		$row = $db->get_row( $Q );

		return $row;
	}

	/**
	 * @param string $params
	 *
	 * @return array
	 *
	 * Get account headers array
	 */
	function get_account_heads( $params = "" ) {
		$default = array(
			"orderby"  => "id",
			"order"    => "DESC",
			"search"   => "",
			"pagesize" => 10,
			"page"     => 1,
		);
		$params  = set_args( $params, $default );
		extract( $params );
		global $db;

		$Q = "SELECT * FROM `account_heads` WHERE 1";

		if ( $params['search'] <> "" ) {
			$search = $db->set_string( $params['search'] );
			$Q      .= " AND `head` LIKE '%{$search}%'";
		}

		$Q    .= " ORDER BY `{$params['orderby']}` {$params['order']}";
		$rows = $db->get_rows2( $Q, $params['page'], $params['pagesize'] );

		return $rows;
	}

	/**
	 * @param $data
	 *
	 * @return string
	 *
	 * Save account head in database.
	 */
	function save_account_head( $data ) {
		if ( ! isset( $data["head"] ) ) {
			return "Head name missing.";
		}
		global $db;
		$head       = $db->set_string( $data["head"] );
		$data["id"] = isset( $data["id"] ) ? $data["id"] : 0;
		$Q          = "SELECT COUNT(*) FROM `account_heads` WHERE `head`='{$head}' AND `id`<>" . $data["id"];
		if ( $db->get_var( $Q ) > 0 ) {
			return "Head name already exists.";
		}
		if ( empty( $data["id"] ) ) {
			unset( $data["id"] );
			$insert_data = [ 'head' => $data['head'] ];
			$response    = $this->DB->insert( "account_heads", $insert_data );
		} else {
			$update_data = [ 'head' => $data['head'] ];
			$response = $this->DB->update( "account_heads", $update_data, [ "id" => $data["id"] ] );
		}

		return $this->DB->Status;
	}

	/**
	 * @param $id
	 *
	 * @return mixed
	 *
	 * Delete account head.
	 */
	function delete_account_head( $id ) {
		$response = $this->DB->delete( "account_heads", [ "id" => $id ] );

		return $this->DB->Status;
	}
}