<?php

namespace Admins;

class UserInfo {

	const TableAdmins = 'admins';

	const Permissions = [
		"time_table"                 => "Time Table",
		"datesheet"                  => "Date Sheet/Schedule",
		"allow_schedule"             => "Schedule Module",
		"new_admission"              => "Can Add New Admission",
		"manage_student_attendance"  => "Manage Student Attendance",
		"allow_students"             => "Students Module",
		"allow_accounts"             => "Accounts Module",
		"view_account_transactions"  => "View Account Transactions",
		"allow_examination"          => "Examinations Module",
		"allow_reports"              => "Reports Module",
		"allow_library"              => "Library Module",
		"manage_library"             => "Library >> Manage Library",
		"library_assign_books"       => "Library >> Assign Books",
		"allow_users"                => "Users Module",
		"allow_parents"              => "Parents Module",
		"fee_module"                 => "Allow Fee Module",
		"pay_fee"                    => "Can Pay Fee",
		"allow_cource_material"      => "Allow Course Material Module",
		"change_app_settings"        => "Can Change Application Settings",
		"sms_section"                => "SMS Allowed",
		"change_sms_settings"        => "Can Change SMS Settings",
		"change_fee_settings"        => "Can Change Fee Settings",
		"change_calendar_settings"   => "Can Change Calendar Settings",
		"manage_daily_expenses"      => "Can Manage Daily Expenses",
		"delete_edit_daily_expenses" => "Can Delete/Edit Daily Expenses",
		"manage_teachers"            => "Manage Teachers",
		"manage_employees"           => "Manage Employees",
		"manage_teacher_salary"      => "Manage Teacher Salaries",
	];

	/**
	 * @param int $id
	 *
	 * @return null|string
	 *
	 * Check if given admin user is administrator or not
	 */
	public static function is_administrator( $id = 0 ) {
		$id = (int) $id;
		if ( empty( $id ) ) {
			$id = self::get_my_id();

			$session_data = self::get_session_data();
			if ( isset( $session_data['is_admin'] ) ) {
				return $session_data['is_admin'] == 1;
			}

			if ( empty( $id ) ) {
				return false;
			}
		}

		global $db;

		return $db->where( 'id', $id )
		          ->select_cols( 'is_admin' )
		          ->get_var( \MySQL\Tables::table_name( "admins" ) ) == 1;
	}

	public static function get_session_data() {
		if ( isset( $_SESSION[ self::get_system_var() ] ) ) {
			return $_SESSION[ self::get_system_var() ];
		}

		return [];
	}

	public static function is_loggedin() {
		$table_name = \MySQL\Tables::table_name( self::TableAdmins );
		global $db;
		if ( ! isset( $_SESSION[ self::get_system_var() ] ) && isset( $_COOKIE[ self::get_system_var() ] ) ) {
			$id = (int) $_COOKIE[ self::get_system_var() ];
			if ( ! empty( $id ) ) {
				$row = $db->where( 'id', $id )->get_row( $table_name );
				if ( isset( $row['id'] ) && $row['id'] == $id ) {
					$_SESSION[ self::get_system_var() ] = $row;
					set_cookie( self::get_system_var(), $id );

					global $options_table;
					$options_table = $db->options_table_array();
				}
			}
		}

		if ( ! isset( $_SESSION[ self::get_system_var() ] ) ) {
			return false;
		} else {
			$row = isset($_SESSION[ self::get_system_var() ])?$_SESSION[ self::get_system_var() ]:'';

			if ( method_exists( $db, 'where' ) ) {
				$rowlive = $db->where( 'id', $row["id"] )->select_cols( 'password,active' )->get_row( $table_name );
			} else {
				$rowlive = $db->get_row( 'SELECT `password`,`active` FROM `' . $table_name . '` WHERE `id`=' . $row['id'] );
			}

			if ( ! is_array( $rowlive ) || ! isset( $rowlive["password"] ) ) {
				return false;
			} else if ( $rowlive["password"] <> $row["password"] ) {
				return false;
			} else if ( $rowlive["active"] <> "1" ) {
				return false;
			} else {
				return true;
			}
		}
	}

	public static function get_system_var() {
		//$str = str_replace(" ", "", get_option("system", "feKara by creativeON"));
		global $db;
		$str   = $db->get_option( "system", "feKara by creativeON" );
		$str   = strtr( $str, [" " => ""]  );
		$Array = [
			"&"  => "",
			"/"  => "",
			"\\" => "",
			"*"  => "",
			","  => "",
			"%"  => "",
			"$"  => "",
			"#"  => "",
			"@"  => "",
			"!"  => "",
			"~"  => "",
			" `" => "",
			"("  => "",
			")"  => "",
			"="  => "",
			"+"  => "",
		];
		$str   = strtr( $str, $Array );
		if ( strlen( $str ) > 10 ) {
			$str = substr( $str, 0, 10 );
		}

		return $str . "admin";
	}

	/**
	 * @return string
	 *
	 * Get ID of logged in user.
	 */
	public static function get_my_id() {
		if ( ! isset( $_SESSION[ self::get_system_var() ] ) ) {
			return 0;
		}
		$id = $_SESSION[ self::get_system_var() ];

		if ( ! isset( $id["id"] ) ) {
			return 0;
		}

		return $id["id"];
	}

	public static function get_admin( $col = "*", $id = "" ) {
		if ( empty( $col ) ) {
			$col = "*";
		}
		$table_name = \MySQL\Tables::table_name( self::TableAdmins );
		global $db;

		if ( empty( $id ) ) {
			$id = self::get_my_id();
		}

		$row = $db->where( 'id', $id )->select_cols( $col )->get_row( $table_name );

		if ( $col == "*" && is_array( $row ) ) {
			return $row;
		} else if ( isset( $row[ $col ] ) ) {
			return $row[ $col ];
		} else {
			return false;
		}
	}

	public static function get_photo_path( $id = "" ) {
		if ( empty( $id ) ) {
			$id = self::get_my_id();
		}
		$path = ADMIN_FILES . $id . "/photo/";
		if ( ! is_dir( $path ) ) {
			$url = \Themes\Info::path() . "images/no-image.png";

			return $url;
		}
		$files = glob( $path . "*" );

		foreach ( $files as $file ) {
			if ( substr( basename( $file ), 0, 3 ) == "FG_" ) {
				continue;
			}
			if ( \Files\File::is_image_file( $file ) ) {
				return $file;
			}
		}

		return \Theme::path() . "images/no-image.png";
	}

	public static function get_photo_url( $id = 0, $width = 0 ) {
		if ( empty( $id ) ) {
			$id = self::get_my_id();
		}
		$path = self::get_photo_path( $id );

		if ( ! empty( $width ) ) {
			$pathinfo = pathinfo( $path );
			if ( isset( $pathinfo["filename"] ) ) {
				$path2 = dirname( $path ) . "/FG_" . $pathinfo["filename"] . "-" . $width . "." . $pathinfo["extension"];
				if ( is_file( $path2 ) ) {
					return str_replace( ROOT_PATH, DOMAIN_URL, $path2 );
				} else if ( is_file( $path ) && ! is_file( $path2 ) ) {
					$image = new \Image();
					$i     = $image->load( $path );
					$i2    = $image->resizeToWidth( $i, $width );
					$image->save( $i2, $path2 );

					if ( is_file( $path2 ) ) {
						return str_replace( ROOT_PATH, DOMAIN_URL, $path2 );
					}
				}
			}
		}

		if ( is_file( $path ) ) {
			return str_replace( ROOT_PATH, DOMAIN_URL, $path );
		}

		return DOMAIN_URL . 'admin_photo.php?id=' . $id . '&width=' . $width;
	}

	public static function is_allowed( $permission, $admin_id = 0 ) {
		$admin_id = (int) $admin_id;
		if ( empty( $admin_id ) ) {
			global $Permissions;
			if ( $Permissions === 'all' ) {
				return true;
			}

			if ( is_array( $Permissions ) ) {
				return in_array( $permission, $Permissions );
			}
		}

		$row = self::get_admin( 'active,is_admin', $admin_id );
		if ( ! isset( $row['active'] ) ) {
			return false;
		}
		if ( $row['active'] == 0 ) {
			return false;
		}
		if ( $row['is_admin'] == 1 ) {
			return true;
		}

		if ( ! array_key_exists( $permission, self::Permissions ) ) {
			return false;
		}
		global $db;

		return $db->where( 'admin_id', $admin_id )
		          ->where( 'permission', $permission )
		          ->count_result( \MySQL\Tables::table_name( 'users_permissions' ) ) > 0;
	}

	public static function get_admin_row( $params = "" ) {
		$default = [
			"id" => 0,
		];
		$params  = set_args( $params, $default );

		$params['id'] = (int) $params['id'];
		if ( $params['id'] < 1 ) {
			$params['id'] = self::get_my_id();
		}
		$table_name = \MySQL\Tables::table_name( 'admins' );
		global $db;
		$row = $db->where( 'id', $params['id'] )->get_row( $table_name );

		if ( is_array( $row ) ) {
			$row["permissions"] = self::get_permissions( $params['id'] );
		}

		return $row;
	}

	public static function get_permissions( $id = 0 ) {
		$id = (int) $id;
		if ( empty( $id ) ) {
			$id = self::get_my_id();
		}
		$table_name = \MySQL\Tables::table_name( 'users_permissions' );

		global $db;
		$permissions = $db->group_concat( 'permission' )->where( 'admin_id', $id )->get_var( $table_name );

		return array_filter( explode( ',', $permissions ) );
	}

	public static function get_list( $params ) {
		$default = [
			"orderby"  => "id",
			"order"    => "DESC",
			"page"     => 0,
			"pagesize" => 0,
			"filter"   => "",
			"active"   => - 1,       # -1 for all or 1 or 0
		];
		$params  = set_args( $params, $default );
		extract( $params);

		global $db;
		$table_name = \MySQL\Tables::table_name( 'admins' );
		$db->select_from( $table_name );
		$params['active'] = (int) $params['active'];
		if ( $params['active'] <> - 1 ) {
			$db->where( 'active', $params['active'] );
		}

		if ( ! empty( $filter ) ) {
			$filter = urldecode( $filter );
			parse_str( $filter, $filter );
			foreach ( $filter as $key => $val ) {
				if ( $key == "admission_date" ) {
					#$d = explode(",",$val);
					#if (empty($d[1])) $d[1]=$d[0];
					#$Q .= " AND (`admission_date` BETWEEN '".$_dates->mysql_date($d[0])."' AND '".$_dates->mysql_date($d[1])."')";
				} else {
					$db->where( $key, $val );
				}
			}
		}

		//$Q .= " ORDER BY `{$params['orderby']}` {$params['order']}";
		$db->order( $params['orderby'], $params['order']);
		if ( $params['page'] > 0 && $params['pagesize'] > 0 ) {
			$rows = $db->get_rows2( $params['page'], $params['pagesize'] );
		} else {
			$rows = $db->get_rows();
		}

		return $rows;
	}
}