<?php
/**
 * @noinspection SqlResolve, SpellCheckingInspection, Sql
 *
 */

/**
 * @since 0.71
 */
define( 'OBJECT', 'OBJECT' );
define( 'object', 'OBJECT' ); // Back compat.

/**
 * @since 2.5.0
 */
define( 'OBJECT_K', 'OBJECT_K' );

/**
 * @since 0.71
 */
define( 'ARRAY_A', 'ARRAY_A' );

/**
 * @since 0.71
 */
define( 'ARRAY_N', 'ARRAY_N' );

class DB {

	const TableOptions = 'options';
	const Collation = 'utf8mb4_unicode_ci';
	const Charset = "utf8mb4";
	const Engine = "MyISAM";

	protected $Global = [];
	var $Status = '';

	/**
	 * Whether to show SQL/DB errors.
	 *
	 * Default behavior is to show errors if both WP_DEBUG and WP_DEBUG_DISPLAY
	 * evaluated to true.
	 *
	 * @since  0.71
	 * @access private
	 * @var bool
	 */
	var $show_errors = false;
	var $connection_error_msg;

	/**
	 * Whether to suppress errors during the DB bootstrapping.
	 *
	 * @access private
	 * @since  2.5.0
	 * @var bool
	 */
	var $suppress_errors = false;

	/**
	 * The last error during query.
	 *
	 * @since 2.5.0
	 * @var string
	 */
	public $last_error = '';

	/**
	 * Amount of queries made
	 *
	 * @since  1.2.0
	 * @access private
	 * @var int
	 */
	var $num_queries = 0;

	/**
	 * Count of rows returned by previous query
	 *
	 * @since  0.71
	 * @access private
	 * @var int
	 */
	var $num_rows = 0;

	/**
	 * Count of affected rows by previous query
	 *
	 * @since  0.71
	 * @access private
	 * @var int
	 */
	var $rows_affected = 0;

	/**
	 * The ID generated for an AUTO_INCREMENT column by the previous query (usually INSERT).
	 *
	 * @since  0.71
	 * @access public
	 * @var int
	 */
	var $insert_id = 0;

	/**
	 * Last query made
	 *
	 * @since  0.71
	 * @access private
	 * @var array
	 */
	var $last_query;

	var $exection_time;

	/**
	 * Results of the last query made
	 *
	 * @since  0.71
	 * @access private
	 * @var array|null
	 */
	var $last_result;

	/**
	 * MySQL result, which is either a resource or boolean.
	 *
	 * @since  0.71
	 * @access protected
	 * @var mixed
	 */
	protected $result;

	/**
	 * Cached column info, for sanity checking data before inserting
	 *
	 * @since  4.2.0
	 * @access protected
	 * @var array
	 */
	protected $col_meta = [];

	/**
	 * Calculated character sets on tables
	 *
	 * @since  4.2.0
	 * @access protected
	 * @var array
	 */
	protected $table_charset = [];

	/**
	 * Whether text fields in the current query need to be sanity checked.
	 *
	 * @since  4.2.0
	 * @access protected
	 * @var bool
	 */
	protected $check_current_query = true;

	/**
	 * Flag to ensure we don't run into recursion problems when checking the collation.
	 *
	 * @since  4.2.0
	 * @access private
	 * @see    check_safe_collation()
	 * @var boolean
	 */
	private $checking_collation = false;

	/**
	 * Saved info on the table column
	 *
	 * @since  0.71
	 * @access protected
	 * @var array
	 */
	protected $col_info;

	/**
	 * Saved queries that were executed
	 *
	 * @since  1.5.0
	 * @access private
	 * @var array
	 */
	var $queries;

	/**
	 * The number of times to retry reconnecting before dying.
	 *
	 * @since  3.9.0
	 * @access protected
	 * @see    check_connection()
	 * @var int
	 */
	protected $reconnect_retries = 5;


	var $prefix = '';
	public $base_prefix;

	var $ready = false;

	public $blogid = 0;

	/**
	 * {@internal Missing Description}}
	 *
	 * @since  3.0.0
	 * @access public
	 * @var int
	 */
	public $siteid = 0;

	/**
	 * List of WordPress per-blog tables
	 *
	 * @since  2.5.0
	 * @access private
	 * @see    tables()
	 * @var array
	 */
	var $tables = [/*'posts',
		'comments',
		'links',
		'options',
		'postmeta',
		'terms',
		'term_taxonomy',
		'term_relationships',
		'commentmeta',*/
	];

	/**
	 * List of deprecated WordPress tables
	 *
	 * categories, post2cat, and link2cat were deprecated in 2.3.0, db version 5539
	 *
	 * @since  2.9.0
	 * @access private
	 * @see    tables()
	 * @var array
	 */
	var $old_tables = [//'categories', 'post2cat', 'link2cat'
	];

	/**
	 * List of WordPress global tables
	 *
	 * @since  3.0.0
	 * @access private
	 * @see    tables()
	 * @var array
	 */
	var $global_tables = [//'users', 'usermeta'
	];

	/**
	 * List of Multisite global tables
	 *
	 * @since  3.0.0
	 * @access private
	 * @see    tables()
	 * @var array
	 */
	var $ms_global_tables = [/*'blogs',
		'signups',
		'site',
		'sitemeta',
		'sitecategories',
		'registration_log',
		'blog_versions',*/
	];

	/**
	 * WordPress Comments table
	 *
	 * @since  1.5.0
	 * @access public
	 * @var string
	 */
	public $comments;

	/**
	 * WordPress Comment Metadata table
	 *
	 * @since  2.9.0
	 * @access public
	 * @var string
	 */
	public $commentmeta;

	/**
	 * WordPress Links table
	 *
	 * @since  1.5.0
	 * @access public
	 * @var string
	 */
	public $links;

	/**
	 * WordPress Options table
	 *
	 * @since  1.5.0
	 * @access public
	 * @var string
	 */
	public $options;

	/**
	 * WordPress Post Metadata table
	 *
	 * @since  1.5.0
	 * @access public
	 * @var string
	 */
	public $postmeta;

	/**
	 * WordPress Posts table
	 *
	 * @since  1.5.0
	 * @access public
	 * @var string
	 */
	public $posts;

	/**
	 * WordPress Terms table
	 *
	 * @since  2.3.0
	 * @access public
	 * @var string
	 */
	public $terms;

	/**
	 * WordPress Term Relationships table
	 *
	 * @since  2.3.0
	 * @access public
	 * @var string
	 */
	public $term_relationships;

	/**
	 * WordPress Term Taxonomy table
	 *
	 * @since  2.3.0
	 * @access public
	 * @var string
	 */
	public $term_taxonomy;

	/*
	 * Global and Multisite tables
	 */

	/**
	 * WordPress User Metadata table
	 *
	 * @since  2.3.0
	 * @access public
	 * @var string
	 */
	public $usermeta;

	/**
	 * WordPress Users table
	 *
	 * @since  1.5.0
	 * @access public
	 * @var string
	 */
	public $users;

	/**
	 * Multisite Blogs table
	 *
	 * @since  3.0.0
	 * @access public
	 * @var string
	 */
	public $blogs;

	/**
	 * Multisite Blog Versions table
	 *
	 * @since  3.0.0
	 * @access public
	 * @var string
	 */
	public $blog_versions;

	/**
	 * Multisite Registration Log table
	 *
	 * @since  3.0.0
	 * @access public
	 * @var string
	 */
	public $registration_log;

	/**
	 * Multisite Signups table
	 *
	 * @since  3.0.0
	 * @access public
	 * @var string
	 */
	public $signups;

	/**
	 * Multisite Sites table
	 *
	 * @since  3.0.0
	 * @access public
	 * @var string
	 */
	public $site;

	/**
	 * Multisite Sitewide Terms table
	 *
	 * @since  3.0.0
	 * @access public
	 * @var string
	 */
	public $sitecategories;

	/**
	 * Multisite Site Metadata table
	 *
	 * @since  3.0.0
	 * @access public
	 * @var string
	 */
	public $sitemeta;

	/**
	 * Format specifiers for DB columns. Columns not listed here default to %s.
	 *
	 * Keys are column names, values are format types: 'ID' => '%d'
	 *
	 * @since  2.8.0
	 * @see    prepare()
	 * @see    insert()
	 * @see    update()
	 * @see    delete()
	 * @see    wp_set_wpdb_vars()
	 * @access public
	 * @var array
	 */
	public $field_types = [];

	/**
	 * Database table columns charset
	 *
	 * @since  2.2.0
	 * @access public
	 * @var string
	 */
	public $charset;

	/**
	 * Database table columns collate
	 *
	 * @since  2.2.0
	 * @access public
	 * @var string
	 */
	public $collate;

	/**
	 * Database Username
	 *
	 * @since  2.9.0
	 * @access protected
	 * @var string
	 */
	protected $dbuser;

	/**
	 * Database Password
	 *
	 * @since  3.1.0
	 * @access protected
	 * @var string
	 */
	protected $dbpassword;

	/**
	 * Database Name
	 *
	 * @since  3.1.0
	 * @access protected
	 * @var string
	 */
	protected $dbname;

	/**
	 * Database Host
	 *
	 * @since  3.1.0
	 * @access protected
	 * @var string
	 */
	protected $dbhost;

	/**
	 * Database Handle
	 *
	 * @since  0.71
	 * @access protected
	 * @var string
	 */
	protected $dbh;

	/**
	 * A textual description of the last query/get_row/get_var call
	 *
	 * @since  3.0.0
	 * @access public
	 * @var string
	 */
	public $func_call;

	/**
	 * Whether MySQL is used as the database engine.
	 *
	 * Set in db_connect() to true, by default. This is used when checking
	 * against the required MySQL version for WordPress. Normally, a replacement
	 * database drop-in (db.php) will skip these checks, but setting this to true
	 * will force the checks to occur.
	 *
	 * @since  3.3.0
	 * @access public
	 * @var bool
	 */
	public $is_mysql = null;

	/**
	 * A list of incompatible SQL modes.
	 *
	 * @since  3.9.0
	 * @access protected
	 * @var array
	 */
	protected $incompatible_modes = [
		'NO_ZERO_DATE',
		'ONLY_FULL_GROUP_BY',
		'STRICT_TRANS_TABLES',
		'STRICT_ALL_TABLES',
		'TRADITIONAL',
	];

	/**
	 * Whether to use mysqli over mysql.
	 *
	 * @since  3.9.0
	 * @access private
	 * @var bool
	 */
	private $use_mysqli = true;

	/**
	 * Whether we've managed to successfully connect at some point
	 *
	 * @since  3.9.0
	 * @access private
	 * @var bool
	 */
	private $has_connected = false;

	private $options_ini_file;

	/**
	 * Connects to the database server and selects a database
	 *
	 * PHP5 style constructor for compatibility with PHP5. Does
	 * the actual setting up of the class properties and connection
	 * to the database.
	 *
	 * @link  https://core.trac.wordpress.org/ticket/3354
	 * @since 2.0.8
	 *
	 * @param string $dbuser     MySQL database user
	 * @param string $dbpassword MySQL database password
	 * @param string $dbname     MySQL database name
	 * @param string $dbhost     MySQL database host
	 */
	public function __construct( $dbuser, $dbpassword, $dbname, $dbhost ) {
		register_shutdown_function( [ $this, '__destruct' ] );

		$this->show_errors();

		//		$this->options_ini_file = ROOT_PATH . 'contents/options.ini';

		$this->init_charset();

		$this->dbuser     = $dbuser;
		$this->dbpassword = $dbpassword;
		$this->dbname     = $dbname;
		$this->dbhost     = $dbhost;

		$this->db_connect();

		if ( ! $this->db_connect() ) {
			echo "<h1>DB Connection Error</h1>
			Please check DB information in file <b>\"" . __DIR__ . "/config.php\"</b>";
			echo "<br><br>Please review this information.<br>";
			echo "=================";
			echo "<br>DB User: " . $this->dbuser;
			echo "<br>DB Password: " . str_repeat( "*", strlen( $this->dbpassword ) );
			echo "<br>DB Name: " . $this->dbname;
			echo "<br>DB Server: " . $this->dbhost;
			echo "<br>Error Message: " . $this->connection_error_msg;

			die;
		}
		if ( ! $this->ready ) {
			echo $this->print_error();
			die;
		}

		if ( is_file( LIBRARY_PATH . 'classes/DB/Updates/options.php' ) ) {
			include_once LIBRARY_PATH . 'classes/DB/Updates/options.php';
		}
		//if ( version_compare( VERSION, ) )
	}

	function is_mysql_view( $table_name ) {
		$Q   = "SHOW FULL TABLES LIKE '{$table_name}'";
		$row = $this->get_row( $Q );
		if ( ! isset( $row['Table_type'] ) ) {
			return false;
		}

		return $row['Table_type'] == "VIEW";
	}

	function is_mysql_table( $table_name ) {
		$Q   = "SHOW FULL TABLES LIKE '{$table_name}'";
		$row = $this->get_row( $Q );
		if ( ! isset( $row['Table_type'] ) ) {
			return false;
		}

		return $row['Table_type'] == "BASE TABLE";
	}

	function fix_tables() {
		$Q    = "SHOW FULL TABLES WHERE `Table_type`='BASE TABLE'";
		$rows = $this->get_rows( $Q, ARRAY_N );
		foreach ( $rows as $row ) {
			$Q = "ALTER TABLE `{$row[0]}` ROW_FORMAT=Fixed;";
			$this->query( $Q );
		}
	}

	function convert_all_tables_into_myisam() {
		$Q = "SELECT CONCAT('ALTER TABLE ', TABLE_SCHEMA, '.', TABLE_NAME,' ENGINE=MyISAM;') 
		FROM Information_schema.TABLES WHERE TABLE_SCHEMA = '" . $this->dbname . "' AND ENGINE = 'InnoDB' AND TABLE_TYPE = 'BASE TABLE'";

		$rows = $this->get_rows( $Q, ARRAY_N );
		foreach ( $rows as $row ) {
			$this->query( $row[0] );
		}
	}

	/**
	 * PHP5 style destructor and will run when database object is destroyed.
	 *
	 * @see   __construct()
	 * @since 2.0.8
	 * @return bool true
	 */
	public function __destruct() {
		$this->db_disconnect();

		return true;
	}

	/**
	 * PHP5 style magic getter, used to lazy-load expensive data.
	 *
	 * @since 3.5.0
	 *
	 * @param string $name The private member to get, and optionally process
	 *
	 * @return mixed The private member
	 */
	public function __get( $name ) {
		if ( 'col_info' === $name ) {
			$this->load_col_info();
		}

		return $this->$name;
	}

	/**
	 * Magic function, for backwards compatibility.
	 *
	 * @since 3.5.0
	 *
	 * @param string $name  The private member to set
	 * @param mixed  $value The value to set
	 */
	public function __set( $name, $value ) {
		$protected_members = [
			'col_meta',
			'table_charset',
			'check_current_query',
		];
		if ( in_array( $name, $protected_members, true ) ) {
			return;
		}
		$this->$name = $value;
	}

	/**
	 * Magic function, for backwards compatibility.
	 *
	 * @since 3.5.0
	 *
	 * @param string $name The private member to check
	 *
	 * @return bool If the member is set or not
	 */
	public function __isset( $name ) {
		return isset( $this->$name );
	}

	/**
	 * Magic function, for backwards compatibility.
	 *
	 * @since 3.5.0
	 *
	 * @param string $name The private member to unset
	 */
	public function __unset( $name ) {
		unset( $this->$name );
	}

	/**
	 * Set $this->charset and $this->collate
	 *
	 * @since 3.1.0
	 */
	public function init_charset() {
		$this->charset = 'utf8mb4';
	}

	/**
	 * @param mysqli $dbh
	 * @param null   $charset
	 * @param null   $collate
	 */
	public function set_charset( mysqli $dbh, $charset = null, $collate = null ) {
		if ( ! isset( $charset ) ) {
			$charset = $this->charset;
		}
		if ( ! isset( $collate ) ) {
			$collate = $this->collate;
		}
		if ( $this->has_cap( 'collation' ) && ! empty( $charset ) ) {
			if ( function_exists( 'mysqli_set_charset' ) && $this->has_cap( 'set_charset' ) ) {
				mysqli_set_charset( $dbh, $charset );
			} else {
				$query = $this->prepare( 'SET NAMES %s', $charset );
				if ( ! empty( $collate ) ) {
					$query .= $this->prepare( ' COLLATE %s', $collate );
				}
				mysqli_query( $dbh, $query );
			}

		}
	}

	/**
	 * Sets the table prefix for the WordPress tables.
	 *
	 * @since 2.5.0
	 *
	 * @param string $prefix          Alphanumeric name for the new prefix.
	 * @param bool   $set_table_names Optional. Whether the table names, e.g. $posts, should be updated or not.
	 *
	 * @return string|WP_Error Old prefix or WP_Error on error
	 */
	public function set_prefix( $prefix, $set_table_names = true ) {

		if ( preg_match( '|[^a-z0-9_]|i', $prefix ) ) {
			return 'Invalid database prefix';
		}

		$old_prefix = $prefix;

		if ( isset( $this->base_prefix ) ) {
			$old_prefix = $this->base_prefix;
		}

		$this->base_prefix = $prefix;

		if ( $set_table_names ) {
			foreach ( $this->tables( 'global' ) as $table => $prefixed_table ) {
				$this->$table = $prefixed_table;
			}

			if ( empty( $this->blogid ) ) {
				return $old_prefix;
			}

			foreach ( $this->tables( 'blog' ) as $table => $prefixed_table ) {
				$this->$table = $prefixed_table;
			}

			foreach ( $this->tables( 'old' ) as $table => $prefixed_table ) {
				$this->$table = $prefixed_table;
			}
		}

		return $old_prefix;
	}

	/**
	 * Sets blog id.
	 *
	 * @since  3.0.0
	 * @access public
	 *
	 * @param int $blog_id
	 * @param int $site_id Optional.
	 *
	 * @return int previous blog id
	 */
	public function set_blog_id( $blog_id, $site_id = 0 ) {
		if ( ! empty( $site_id ) ) {
			$this->siteid = $site_id;
		}

		$old_blog_id  = $this->blogid;
		$this->blogid = $blog_id;

		foreach ( $this->tables( 'blog' ) as $table => $prefixed_table ) {
			$this->$table = $prefixed_table;
		}

		foreach ( $this->tables( 'old' ) as $table => $prefixed_table ) {
			$this->$table = $prefixed_table;
		}

		return $old_blog_id;
	}

	/**
	 * Gets blog prefix.
	 *
	 * @since 3.0.0
	 *
	 * @param int $blog_id Optional.
	 *
	 * @return string Blog prefix.
	 */
	public function get_blog_prefix( $blog_id = null ) {
		return $this->base_prefix;
	}

	/**
	 * Returns an array of WordPress tables.
	 *
	 * Also allows for the CUSTOM_USER_TABLE and CUSTOM_USER_META_TABLE to
	 * override the WordPress users and usermeta tables that would otherwise
	 * be determined by the prefix.
	 *
	 * The scope argument can take one of the following:
	 *
	 * 'all' - returns 'all' and 'global' tables. No old tables are returned.
	 * 'blog' - returns the blog-level tables for the queried blog.
	 * 'global' - returns the global tables for the installation, returning multisite tables only if running multisite.
	 * 'ms_global' - returns the multisite global tables, regardless if current installation is multisite.
	 * 'old' - returns tables which are deprecated.
	 *
	 * @since 3.0.0
	 * @uses  $tables
	 * @uses  $old_tables
	 * @uses  $global_tables
	 * @uses  $ms_global_tables
	 *
	 * @param string $scope   Optional. Can be all, global, ms_global, blog, or old tables. Defaults to all.
	 * @param bool   $prefix  Optional. Whether to include table prefixes. Default true. If blog
	 *                        prefix is requested, then the custom users and usermeta tables will be mapped.
	 * @param int    $blog_id Optional. The blog_id to prefix. Defaults to $blogid. Used only when prefix is
	 *                        requested.
	 *
	 * @return array Table names. When a prefix is requested, the key is the unprefixed table name.
	 */
	public function tables( $scope = 'all', $prefix = true, $blog_id = 0 ) {
		switch ( $scope ) {
			case 'all' :
				$tables = array_merge( $this->global_tables, $this->tables );
				break;
			case 'blog' :
				$tables = $this->tables;
				break;
			case 'global' :
				$tables = $this->global_tables;
				break;
			case 'ms_global' :
				$tables = $this->ms_global_tables;
				break;
			case 'old' :
				$tables = $this->old_tables;
				break;
			default :
				return [];
		}

		if ( $prefix ) {
			if ( ! $blog_id ) {
				$blog_id = $this->blogid;
			}
			$base_prefix   = $this->base_prefix;
			$global_tables = array_merge( $this->global_tables, $this->ms_global_tables );
			foreach ( $tables as $k => $table ) {
				if ( in_array( $table, $global_tables ) ) {
					$tables[ $table ] = $base_prefix . $table;
				} else {
					$tables[ $table ] = $table;
				}
				unset( $tables[ $k ] );
			}

			if ( isset( $tables['users'] ) && defined( 'CUSTOM_USER_TABLE' ) ) {
				$tables['users'] = CUSTOM_USER_TABLE;
			}

			if ( isset( $tables['usermeta'] ) && defined( 'CUSTOM_USER_META_TABLE' ) ) {
				$tables['usermeta'] = CUSTOM_USER_META_TABLE;
			}
		}

		return $tables;
	}

	/**
	 * Selects a database using the current database connection.
	 *
	 * The database name will be changed based on the current database
	 * connection. On failure, the execution will bail and display an DB error.
	 *
	 * @since 0.71
	 *
	 * @param string   $db  MySQL database name
	 * @param resource $dbh Optional link identifier.
	 *
	 * @return null Always null.
	 */
	public function select( $db, $dbh = null ) {
		if ( is_null( $dbh ) ) {
			$dbh = $this->dbh;
		}

		$success = @mysqli_select_db( $dbh, $db );

		if ( ! $success ) {
			$this->ready = false;

			return false;
		}

		return true;
	}

	/**
	 * Do not use, deprecated.
	 *
	 * Use esc_sql() or prepare() instead.
	 *
	 * @since      2.8.0
	 * @deprecated 3.6.0
	 * @see        prepare
	 * @see        esc_sql()
	 * @access     private
	 *
	 * @param string $string
	 *
	 * @return string
	 */
	function _weak_escape( $string ) {
		if ( func_num_args() === 1 && function_exists( '_deprecated_function' ) ) {
			_deprecated_function( __METHOD__, '3.6', 'prepare() or esc_sql()' );
		}

		return addslashes( $string );
	}

	/**
	 * Real escape, using mysqli_real_escape_string() or mysql_real_escape_string()
	 *
	 * @see    mysqli_real_escape_string()
	 * @see    mysql_real_escape_string()
	 * @since  2.8.0
	 * @access private
	 *
	 * @param  string $string to escape
	 *
	 * @return string escaped
	 */
	function _real_escape( $string ) {
		if ( is_array( $string ) ) {
			return $string;
		}
		if ( $this->dbh ) {
			return mysqli_real_escape_string( $this->dbh, $string );

		}

		$class = get_class( $this );

		return addslashes( $string );
	}

	/**
	 * Escape data. Works on arrays.
	 *
	 * @uses   _real_escape()
	 * @since  2.8.0
	 * @access private
	 *
	 * @param  string|array $data
	 *
	 * @return string|array escaped
	 */
	public function _escape( $data ) {
		if ( is_array( $data ) ) {
			foreach ( $data as $k => $v ) {
				if ( is_array( $v ) ) {
					$data[ $k ] = $this->_escape( $v );
				} else {
					$data[ $k ] = $this->_real_escape( $v );
				}
			}
		} else {
			$data = $this->_real_escape( $data );
		}

		return $data;
	}

	/**
	 * Do not use, deprecated.
	 *
	 * Use esc_sql() or prepare() instead.
	 *
	 * @since      0.71
	 * @deprecated 3.6.0
	 * @see        prepare()
	 * @see        esc_sql()
	 *
	 * @param mixed $data
	 *
	 * @return mixed
	 */
	//	public function escape( $data ) {
	//		if ( func_num_args() === 1 && function_exists( '_deprecated_function' ) ) {
	//			_deprecated_function( __METHOD__, '3.6', 'prepare() or esc_sql()' );
	//		}
	//		if ( is_array( $data ) ) {
	//			foreach ( $data as $k => $v ) {
	//				if ( is_array( $v ) ) {
	//					$data[ $k ] = $this->escape( $v, 'recursive' );
	//				} else {
	//					$data[ $k ] = $this->_weak_escape( $v, 'internal' );
	//				}
	//			}
	//		} else {
	//			$data = $this->_weak_escape( $data, 'internal' );
	//		}
	//
	//		return $data;
	//	}

	/**
	 * Escapes content by reference for insertion into the database, for security
	 *
	 * @uses  _real_escape()
	 * @since 2.3.0
	 *
	 * @param string $string to escape
	 *
	 * @return void
	 */
	public function escape_by_ref( &$string ) {
		if ( ! is_float( $string ) ) {
			$string = $this->_real_escape( $string );
		}
	}

	/**
	 * Prepares a SQL query for safe execution. Uses sprintf()-like syntax.
	 *
	 * The following directives can be used in the query format string:
	 *   %d (integer)
	 *   %f (float)
	 *   %s (string)
	 *   %% (literal percentage sign - no argument needed)
	 *
	 * All of %d, %f, and %s are to be left unquoted in the query string and they need an argument passed for them.
	 * Literals (%) as parts of the query must be properly written as %%.
	 *
	 * This function only supports a small subset of the sprintf syntax; it only supports %d (integer), %f (float), and
	 * %s (string). Does not support sign, padding, alignment, width or precision specifiers. Does not support argument
	 * numbering/swapping.
	 *
	 * May be called like {@link http://php.net/sprintf sprintf()} or like {@link http://php.net/vsprintf vsprintf()}.
	 *
	 * Both %d and %s should be left unquoted in the query string.
	 *
	 *     prepare( "SELECT * FROM `table` WHERE `column` = %s AND `field` = %d", 'foo', 1337 )
	 *     prepare( "SELECT DATE_FORMAT(`field`, '%%c') FROM `table` WHERE `column` = %s", 'foo' );
	 *
	 * @link  http://php.net/sprintf Description of syntax.
	 * @since 2.3.0
	 *
	 * @param string      $query    Query statement with sprintf()-like placeholders
	 * @param array|mixed $args     The array of variables to substitute into the query's placeholders if being called
	 *                              like
	 *                              {@link http://php.net/vsprintf vsprintf()}, or the first variable to substitute
	 *                              into the query's placeholders if being called like {@link http://php.net/sprintf
	 *                              sprintf()}.
	 *
	 *
	 * @return null|false|string Sanitized query string, null if there is no query, false if there is an error and
	 *                           string if there was something to prepare
	 */
	public function prepare( $query, $args ) {
		if ( is_null( $query ) ) {
			return;
		}


		$args = func_get_args();
		array_shift( $args );
		// If args were passed as an array (as in vsprintf), move them up
		if ( isset( $args[0] ) && is_array( $args[0] ) ) {
			$args = $args[0];
		}
		$query = str_replace( "'%s'", '%s', $query ); // in case someone mistakenly already singlequoted it
		$query = str_replace( '"%s"', '%s', $query ); // doublequote unquoting
		$query = preg_replace( '|(?<!%)%f|', '%F', $query ); // Force floats to be locale unaware
		$query = preg_replace( '|(?<!%)%s|', "'%s'", $query ); // quote the strings, avoiding escaped strings like %%s
		$query = preg_replace( '|(?<!%)%t|', "`%s`", $query ); // %t for table or column names
		$query = preg_replace( '|(?<!%)%w|', "%s", $query ); // String without quotes
		array_walk( $args, [ $this, 'escape_by_ref' ] );

		return @vsprintf( $query, $args );
	}

	/**
	 * First half of escaping for LIKE special characters % and _ before preparing for MySQL.
	 *
	 * Use this only before prepare() or esc_sql().  Reversing the order is very bad for security.
	 *
	 * Example Prepared Statement:
	 *  $wild = '%';
	 *  $find = 'only 43% of planets';
	 *  $like = $wild . $wpdb->esc_like( $find ) . $wild;
	 *  $sql  = $wpdb->prepare( "SELECT * FROM $wpdb->posts WHERE post_content LIKE %s", $like );
	 *
	 * Example Escape Chain:
	 *  $sql  = esc_sql( $wpdb->esc_like( $input ) );
	 *
	 * @since  4.0.0
	 * @access public
	 *
	 * @param string $text The raw text to be escaped. The input typed by the user should have no
	 *                     extra or deleted slashes.
	 *
	 * @return string Text in the form of a LIKE phrase. The output is not SQL safe. Call prepare()
	 *                or real_escape next.
	 */
	public function esc_like( $text ) {
		return addcslashes( $text, '_%\\' );
	}

	/**
	 * Print SQL/DB error.
	 *
	 * @since 0.71
	 * @global array $EZSQL_ERROR Stores error information of query and error string
	 *
	 * @param string $str         The error to display
	 *
	 * @return false|null False if the showing of errors is disabled.
	 */
	public function print_error( $str = '' ) {
		global $EZSQL_ERROR;

		if ( ! $str ) {
			$str = @mysqli_error( $this->dbh );

		}

		return $str;

		$EZSQL_ERROR[] = [ 'query' => $this->last_query, 'error_str' => $str ];

		if ( $this->suppress_errors ) {
			return false;
		}

		#wp_load_translations_early();

		#if ( $caller = $this->get_caller() )
		#	$error_str = sprintf( __( 'WordPress database error %1$s for query %2$s made by %3$s' ), $str, $this->last_query, $caller );
		#else
		#	$error_str = sprintf( __( 'WordPress database error %1$s for query %2$s' ), $str, $this->last_query );

		#error_log( $error_str );

		// Are we showing errors?
		if ( ! $this->show_errors ) {
			return false;
		}

		// If there is an error then take note of it
		$str   = htmlspecialchars( $str, ENT_QUOTES );
		$query = htmlspecialchars( $this->last_query, ENT_QUOTES );

		/*print "<div id='error'>
		<p class='wpdberror'><strong>WordPress database error:</strong> [$str]<br />
		<code>$query</code></p>
		</div>";*/

		return $str;
	}

	/**
	 * Enables showing of database errors.
	 *
	 * This function should be used only to enable showing of errors.
	 * hide_errors() should be used instead for hiding of errors. However,
	 * this function can be used to enable and disable showing of database
	 * errors.
	 *
	 * @since 0.71
	 * @see   hide_errors()
	 *
	 * @param bool $show Whether to show or hide errors
	 *
	 * @return bool Old value for showing errors.
	 */
	public function show_errors( $show = true ) {
		$errors            = $this->show_errors;
		$this->show_errors = $show;

		return $errors;
	}

	/**
	 * Disables showing of database errors.
	 *
	 * By default database errors are not shown.
	 *
	 * @since 0.71
	 * @see   show_errors()
	 *
	 * @return bool Whether showing of errors was active
	 */
	public function hide_errors() {
		$show              = $this->show_errors;
		$this->show_errors = false;

		return $show;
	}

	/**
	 * Whether to suppress database errors.
	 *
	 * By default database errors are suppressed, with a simple
	 * call to this function they can be enabled.
	 *
	 * @since 2.5.0
	 * @see   hide_errors()
	 *
	 * @param bool $suppress Optional. New value. Defaults to true.
	 *
	 * @return bool Old value
	 */
	public function suppress_errors( $suppress = true ) {
		$errors                = $this->suppress_errors;
		$this->suppress_errors = (bool) $suppress;

		return $errors;
	}

	/**
	 * Kill cached query results.
	 *
	 * @since 0.71
	 * @return void
	 */
	public function flush() {
		$this->last_result   = [];
		$this->col_info      = null;
		$this->last_query    = null;
		$this->exection_time = null;
		$this->rows_affected = $this->num_rows = 0;
		$this->last_error    = '';
		$this->Status        = '';

		if ( $this->result instanceof mysqli_result ) {
			mysqli_free_result( $this->result );
			$this->result = null;

			// Sanity check before using the handle
			if ( empty( $this->dbh ) || ! ( $this->dbh instanceof mysqli ) ) {
				return;
			}

			// Clear out any results from a multi-query
			while ( mysqli_more_results( $this->dbh ) ) {
				mysqli_next_result( $this->dbh );
			}
		}
	}

	/**
	 * Connect to and select database.
	 *
	 * If $allow_bail is false, the lack of database connection will need
	 * to be handled manually.
	 *
	 * @since 3.0.0
	 * @since 3.9.0 $allow_bail parameter added.
	 *
	 * @param bool $allow_bail Optional. Allows the function to bail. Default true.
	 *
	 * @return null|bool True with a successful connection, false on failure.
	 */
	public function db_connect( $allow_bail = true ) {

		$this->is_mysql = true;

		/*
		 * Deprecated in 3.9+ when using MySQLi. No equivalent
		 * $new_link parameter exists for mysqli_* functions.
		 */
		$new_link     = defined( 'MYSQL_NEW_LINK' ) ? MYSQL_NEW_LINK : true;
		$client_flags = defined( 'MYSQL_CLIENT_FLAGS' ) ? MYSQL_CLIENT_FLAGS : 0;

		$this->dbh = mysqli_init();

		// mysqli_real_connect doesn't support the host param including a port or socket
		// like mysql_connect does. This duplicates how mysql_connect detects a port and/or socket file.
		$port           = null;
		$socket         = null;
		$host           = $this->dbhost;
		$port_or_socket = strstr( $host, ':' );
		if ( ! empty( $port_or_socket ) ) {
			$host           = substr( $host, 0, strpos( $host, ':' ) );
			$port_or_socket = substr( $port_or_socket, 1 );
			if ( 0 !== strpos( $port_or_socket, '/' ) ) {
				$port         = intval( $port_or_socket );
				$maybe_socket = strstr( $port_or_socket, ':' );
				if ( ! empty( $maybe_socket ) ) {
					$socket = substr( $maybe_socket, 1 );
				}
			} else {
				$socket = $port_or_socket;
			}
		}

		if ( defined( 'DISPLAY_ERRORS' ) && ( DISPLAY_ERRORS || DISPLAY_ERRORS == 1 ) ) {
			mysqli_real_connect( $this->dbh, $host, $this->dbuser, $this->dbpassword, null, $port, $socket, $client_flags );
		} else {
			@mysqli_real_connect( $this->dbh, $host, $this->dbuser, $this->dbpassword, null, $port, $socket, $client_flags );
		}

		if ( $this->dbh->connect_errno ) {
			echo '<h1>DB Connection Error: ' . mysqli_connect_error() . '</h1>';
			die;
			$this->dbh = null;

			/* It's possible ext/mysqli is misconfigured. Fall back to ext/mysql if:
			  *  - We haven't previously connected, and
			  */
			$attempt_fallback = true;

			if ( $this->has_connected ) {
				$attempt_fallback = false;
			} else if ( ! function_exists( 'mysql_connect' ) ) {
				$attempt_fallback = false;
			}

			if ( $attempt_fallback ) {
				$this->use_mysqli = false;
				$this->db_connect();
			}
		}

		if ( ! $this->dbh && $allow_bail ) {
			$this->bail( sprintf( ( "
			<h1>Error establishing a database connection</h1>
			" ), htmlspecialchars( $this->dbhost, ENT_QUOTES ) ), 'db_connect_fail' );

			return false;
		} else if ( $this->dbh ) {
			$this->query( "set character_set_results='utf8'" );
			$this->has_connected = true;
			$this->set_charset( $this->dbh );
			//$this->set_sql_mode();
			$this->ready = true;
			$this->select( $this->dbname, $this->dbh );

			return true;
		}

		return false;
	}

	/**
	 * Check that the connection to the database is still up. If not, try to reconnect.
	 *
	 * If this function is unable to reconnect, it will forcibly die, or if after the
	 * the template_redirect hook has been fired, return false instead.
	 *
	 * If $allow_bail is false, the lack of database connection will need
	 * to be handled manually.
	 *
	 * @since 3.9.0
	 *
	 * @param bool $allow_bail Optional. Allows the function to bail. Default true.
	 *
	 * @return bool|null True if the connection is up.
	 */
	public function check_connection( $allow_bail = true ) {
		if ( @mysqli_ping( $this->dbh ) ) {
			return true;
		}


		$error_reporting = false;

		// Disable warnings, as we don't want to see a multitude of "unable to connect" messages

		for ( $tries = 1; $tries <= $this->reconnect_retries; $tries ++ ) {
			// On the last try, re-enable warnings. We want to see a single instance of the
			// "unable to connect" message on the bail() screen, if it appears.
			if ( $this->reconnect_retries === $tries ) {
				error_reporting( $error_reporting );
			}

			if ( $this->db_connect( false ) ) {
				if ( $error_reporting ) {
					error_reporting( $error_reporting );
				}

				return true;
			}

			sleep( 1 );
		}

		// If template_redirect has already happened, it's too late for wp_die()/dead_db().
		// Let's just return and hope for the best.
		if ( did_action( 'template_redirect' ) ) {
			return false;
		}

		if ( ! $allow_bail ) {
			return false;
		}

		// We weren't able to reconnect, so we better bail.
		/*$this->bail( sprintf( ( "
<h1>Error reconnecting to the database</h1>
<p>This means that we lost contact with the database server at <code>%s</code>. This could mean your host's database server is down.</p>
<ul>
	<li>Are you sure that the database server is running?</li>
	<li>Are you sure that the database server is not under particularly heavy load?</li>
</ul>
<p>If you're unsure what these terms mean you should probably contact your host. If you still need help you can always visit the <a href='https://wordpress.org/support/'>WordPress Support Forums</a>.</p>
" ), htmlspecialchars( $this->dbhost, ENT_QUOTES ) ), 'db_connect_fail' );*/

		$this->Status = last_error;
		echo $this->last_error;

		// Call dead_db() if bail didn't die, because this database is no more. It has ceased to be (at least temporarily).
		dead_db();
	}

	/**
	 * Perform a MySQL database query, using current database connection.
	 *
	 * More information can be found on the codex page.
	 *
	 * @since 0.71
	 *
	 * @param string $query Database query
	 *
	 * @return int|false Number of rows affected/selected or false on error
	 */
	public function query( $query ) {
		if ( ! $this->ready ) {
			$this->check_current_query = true;
			$this->Status              = $this->last_error;

			return false;
		}

		/**
		 * Filter the database query.
		 *
		 * Some queries are made before the plugins have been loaded,
		 * and thus cannot be filtered with this method.
		 *
		 * @since 2.1.0
		 *
		 * @param string $query Database query.
		 */

		$this->flush();

		$this->exection_time = microtime( true );

		// Log how the function was called
		$this->func_call = "\$db->query(\"$query\")";

		// If we're writing to the database, make sure the query will write safely.
		if ( $this->check_current_query && ! $this->check_ascii( $query ) ) {
			$stripped_query = $this->strip_invalid_text_from_query( $query );
			// strip_invalid_text_from_query() can perform queries, so we need
			// to flush again, just to make sure everything is clear.
			$this->flush();
			if ( $stripped_query !== $query ) {
				$this->insert_id = 0;
				$this->Status    = $this->last_error;

				return false;
			}
		}

		$this->check_current_query = true;
		// Keep track of the last query for debug..
		$this->last_query = $query;

		$this->_do_query( $query );

		// MySQL server has gone away, try to reconnect
		$mysql_errno = 0;
		if ( ! empty( $this->dbh ) ) {
			$mysql_errno = mysqli_errno( $this->dbh );
		}

		if ( empty( $this->dbh ) || 2006 == $mysql_errno ) {
			if ( $this->check_connection() ) {
				$this->_do_query( $query );
			} else {
				$this->insert_id = 0;
				$this->Status    = $this->last_error;

				return false;
			}
		}

		// If there is an error then take note of it..
		$this->last_error = mysqli_error( $this->dbh );


		if ( $this->last_error ) {
			$this->Status = $this->last_error;

			// Clear insert_id on a subsequent failed insert.
			if ( $this->insert_id && preg_match( '/^\s*(insert|replace)\s/i', $query ) ) {
				$this->insert_id = 0;
			}

			$this->print_error();
			$this->Status = $this->last_error;

			return false;
		}

		if ( preg_match( '/^\s*(create|alter|truncate|drop)\s/i', $query ) ) {
			$return_val = $this->result;
		} elseif ( preg_match( '/^\s*(insert|delete|update|replace)\s/i', $query ) ) {
			$this->rows_affected = mysqli_affected_rows( $this->dbh );

			// Take note of the insert_id
			if ( preg_match( '/^\s*(insert|replace)\s/i', $query ) ) {
				$this->insert_id = mysqli_insert_id( $this->dbh );

			}
			// Return number of rows affected
			$return_val = $this->rows_affected;
		} else {
			$num_rows = 0;
			if ( $this->result instanceof mysqli_result ) {
				while ( $row = @mysqli_fetch_object( $this->result ) ) {
					$this->last_result[ $num_rows ] = $row;
					$num_rows ++;
				}
			}

			// Log number of rows the query returned
			// and return number of rows selected
			$this->num_rows = $num_rows;
			$return_val     = $num_rows;
		}

		$this->exection_time = microtime( true ) - $this->exection_time;

		if ( \General::localhost() ) {
			global $profiler_array, $sum_time;

			$bt    = debug_backtrace();
			$lines = [];
			foreach ( $bt as $btline ) {
				if ( ! isset( $btline['file'] ) ) {
					debug( $btline );
				}
				$btline['file'] = isset( $btline['file'] ) ? $btline['file'] : '';
				$btline['line'] = isset( $btline['line'] ) ? $btline['line'] : '';
				$lines[]        = [
					'file'     => $btline['file'],
					'line'     => $btline['line'],
					'function' => $btline['function'],
					'args'     => $btline['args'],
				];
			}
			$lines = array_reverse( $lines );

			$profiler_array[] = [
				//'last_result'    => $this->last_result,
				'col_info'       => $this->col_info,
				'last_query'     => $this->last_query,
				'execution_time' => $this->exection_time,
				'rows_affected'  => $this->rows_affected,
				'num_rows'       => $this->num_rows,
				'last_error'     => $this->last_error,
				'url'            => \Urls::current_url_static(),
				'lines'          => $lines,
			];
			$sum_time         += $this->exection_time;
		}
		$this->Status = 'OK';

		return $return_val;
	}

	/**
	 * Internal function to perform the mysql_query() call.
	 *
	 * @since  3.9.0
	 *
	 * @access private
	 * @see    query()
	 *
	 * @param string $query The query to run.
	 */
	private function _do_query( $query ) {
		if ( ! empty( $this->dbh ) ) {
			$this->result = @mysqli_query( $this->dbh, $query );
		}

		$this->num_queries ++;
	}

	/**
	 * Insert a row into a table.
	 *
	 *     insert( 'table', array( 'column' => 'foo', 'field' => 'bar' ) )
	 *     insert( 'table', array( 'column' => 'foo', 'field' => 1337 ), array( '%s', '%d' ) )
	 *
	 * @since 2.5.0
	 * @see   prepare()
	 * @see   $field_types
	 * @see   wp_set_wpdb_vars()
	 *
	 * @param string       $table  table name
	 * @param array        $data   Data to insert (in column => value pairs). Both $data columns and $data values
	 *                             should be "raw" (neither should be SQL escaped).
	 * @param array|string $format Optional. An array of formats to be mapped to each of the value in $data. If string,
	 *                             that format will be used for all of the values in $data. A format is one of '%d',
	 *                             '%f', '%s' (integer, float, string). If omitted, all values in $data will be treated
	 *                             as strings unless otherwise specified in $field_types.
	 *
	 * @return int|false The number of rows inserted, or false on error.
	 */
	public function insert( $table, $data, $format = null ) {
		return $this->_insert_replace_helper( $table, $data, $format, 'INSERT' );
	}

	/**
	 * Replace a row into a table.
	 *
	 *     replace( 'table', array( 'column' => 'foo', 'field' => 'bar' ) )
	 *     replace( 'table', array( 'column' => 'foo', 'field' => 1337 ), array( '%s', '%d' ) )
	 *
	 * @since 3.0.0
	 * @see   prepare()
	 * @see   $field_types
	 * @see   wp_set_wpdb_vars()
	 *
	 * @param string       $table  table name
	 * @param array        $data   Data to insert (in column => value pairs). Both $data columns and $data values
	 *                             should be "raw" (neither should be SQL escaped).
	 * @param array|string $format Optional. An array of formats to be mapped to each of the value in $data. If string,
	 *                             that format will be used for all of the values in $data. A format is one of '%d',
	 *                             '%f', '%s' (integer, float, string). If omitted, all values in $data will be treated
	 *                             as strings unless otherwise specified in $field_types.
	 *
	 * @return int|false The number of rows affected, or false on error.
	 */
	public function replace( $table, $data, $format = null ) {
		return $this->_insert_replace_helper( $table, $data, $format, 'REPLACE' );
	}

	/**
	 * Helper function for insert and replace.
	 *
	 * Runs an insert or replace query based on $type argument.
	 *
	 * @access private
	 * @since  3.0.0
	 * @see    prepare()
	 * @see    $field_types
	 * @see    wp_set_wpdb_vars()
	 *
	 * @param string       $table  table name
	 * @param array        $data   Data to insert (in column => value pairs). Both $data columns and $data values
	 *                             should be "raw" (neither should be SQL escaped).
	 * @param array|string $format Optional. An array of formats to be mapped to each of the value in $data. If string,
	 *                             that format will be used for all of the values in $data. A format is one of '%d',
	 *                             '%f', '%s' (integer, float, string). If omitted, all values in $data will be treated
	 *                             as strings unless otherwise specified in $field_types.
	 * @param string       $type   Optional. What type of operation is this? INSERT or REPLACE. Defaults to INSERT.
	 *
	 * @return int|false The number of rows affected, or false on error.
	 */
	function _insert_replace_helper( $table, $data, $format = null, $type = 'INSERT' ) {
		if ( ! in_array( strtoupper( $type ), [ 'REPLACE', 'INSERT' ] ) ) {
			return false;
		}

		$data = $this->process_fields( $table, $data, $format );
		if ( false === $data ) {
			return false;
		}

		$formats = $values = [];
		foreach ( $data as $value ) {
			$formats[] = $value['format'];
			$values[]  = $value['value'];
		}

		$fields  = '`' . implode( '`, `', array_keys( $data ) ) . '`';
		$formats = implode( ', ', $formats );

		$sql = "$type INTO `$table` ($fields) VALUES ($formats)";

		$this->insert_id           = 0;
		$this->check_current_query = false;

		return $this->query( $this->prepare( $sql, $values ) );
	}

	/**
	 * Update a row in the table
	 *
	 *     update( 'table', array( 'column' => 'foo', 'field' => 'bar' ), array( 'ID' => 1 ) )
	 *     update( 'table', array( 'column' => 'foo', 'field' => 1337 ), array( 'ID' => 1 ), array( '%s', '%d' ),
	 *     array( '%d' ) )
	 *
	 * @since 2.5.0
	 * @see   prepare()
	 * @see   $field_types
	 * @see   wp_set_wpdb_vars()
	 *
	 * @param string       $table        table name
	 * @param array        $data         Data to update (in column => value pairs). Both $data columns and $data values
	 *                                   should be "raw" (neither should be SQL escaped).
	 * @param array        $where        A named array of WHERE clauses (in column => value pairs). Multiple clauses
	 *                                   will be joined with ANDs. Both $where columns and $where values should be
	 *                                   "raw".
	 * @param array|string $format       Optional. An array of formats to be mapped to each of the values in $data. If
	 *                                   string, that format will be used for all of the values in $data. A format is
	 *                                   one of '%d', '%f', '%s' (integer, float, string). If omitted, all values in
	 *                                   $data will be treated as strings unless otherwise specified in
	 *                                   $field_types.
	 * @param array|string $where_format Optional. An array of formats to be mapped to each of the values in $where. If
	 *                                   string, that format will be used for all of the items in $where. A format is
	 *                                   one of '%d', '%f', '%s' (integer, float, string). If omitted, all values in
	 *                                   $where will be treated as strings.
	 *
	 * @return int|false The number of rows updated, or false on error.
	 */
	public function update( $table, $data, $where, $format = null, $where_format = null ) {
		if ( ! is_array( $data ) || ! is_array( $where ) ) {
			return false;
		}

		$data = $this->process_fields( $table, $data, $format );
		if ( false === $data ) {
			return false;
		}
		$where = $this->process_fields( $table, $where, $where_format );
		if ( false === $where ) {
			return false;
		}

		$fields = $conditions = $values = [];
		foreach ( $data as $field => $value ) {
			$fields[] = "`$field` = " . $value['format'];
			$values[] = $value['value'];
		}
		foreach ( $where as $field => $value ) {
			$conditions[] = "`$field` = " . $value['format'];
			$values[]     = $value['value'];
		}

		$fields     = implode( ', ', $fields );
		$conditions = implode( ' AND ', $conditions );

		$sql = "UPDATE `$table` SET $fields WHERE $conditions";

		$this->check_current_query = false;

		//debug($this->prepare( $sql, $values ));
		return $this->query( $this->prepare( $sql, $values ) );
	}

	/**
	 * Delete a row in the table
	 *
	 *     delete( 'table', array( 'ID' => 1 ) )
	 *     delete( 'table', array( 'ID' => 1 ), array( '%d' ) )
	 *
	 * @since 3.4.0
	 * @see   prepare()
	 * @see   $field_types
	 * @see   wp_set_wpdb_vars()
	 *
	 * @param string       $table        table name
	 * @param array        $where        A named array of WHERE clauses (in column => value pairs). Multiple clauses
	 *                                   will be joined with ANDs. Both $where columns and $where values should be
	 *                                   "raw".
	 * @param array|string $where_format Optional. An array of formats to be mapped to each of the values in $where. If
	 *                                   string, that format will be used for all of the items in $where. A format is
	 *                                   one of '%d', '%f', '%s' (integer, float, string). If omitted, all values in
	 *                                   $where will be treated as strings unless otherwise specified in
	 *                                   $field_types.
	 *
	 * @return int|false The number of rows updated, or false on error.
	 */
	public function delete( $table, $where, $where_format = null, $condition = 'AND' ) {
		if ( ! is_array( $where ) ) {
			return false;
		}

		$where = $this->process_fields( $table, $where, $where_format );
		if ( false === $where ) {
			return false;
		}

		$conditions = $values = [];
		foreach ( $where as $field => $value ) {
			$conditions[] = "`$field` = " . $value['format'];
			$values[]     = $value['value'];
		}

		$conditions = implode( ' ' . $condition . ' ', $conditions );

		$sql = "DELETE FROM `$table` WHERE $conditions";

		$this->check_current_query = false;

		return $this->query( $this->prepare( $sql, $values ) );
	}

	/**
	 * Processes arrays of field/value pairs and field formats.
	 *
	 * This is a helper method for wpdb's CRUD methods, which take field/value
	 * pairs for inserts, updates, and where clauses. This method first pairs
	 * each value with a format. Then it determines the charset of that field,
	 * using that to determine if any invalid text would be stripped. If text is
	 * stripped, then field processing is rejected and the query fails.
	 *
	 * @since  4.2.0
	 * @access protected
	 *
	 * @param string $table  Table name.
	 * @param array  $data   Field/value pair.
	 * @param mixed  $format Format for each field.
	 *
	 * @return array|bool Returns an array of fields that contain paired values
	 *                    and formats. Returns false for invalid values.
	 */
	protected function process_fields( $table, $data, $format ) {
		$data = $this->process_field_formats( $data, $format );
		$data = $this->process_field_charsets( $data, $table );
		if ( false === $data ) {
			return false;
		}

		$converted_data = $this->strip_invalid_text( $data );

		if ( $data !== $converted_data ) {
			return false;
		}

		return $data;
	}

	/**
	 * Prepares arrays of value/format pairs as passed to wpdb CRUD methods.
	 *
	 * @since  4.2.0
	 * @access protected
	 *
	 * @param array $data   Array of fields to values.
	 * @param mixed $format Formats to be mapped to the values in $data.
	 *
	 * @return array Array, keyed by field names with values being an array
	 *               of 'value' and 'format' keys.
	 */
	protected function process_field_formats( $data, $format ) {
		$formats = $original_formats = (array) $format;

		foreach ( $data as $field => $value ) {
			$value = [
				'value'  => $value,
				'format' => '%s',
			];

			if ( ! empty( $format ) ) {
				$value['format'] = array_shift( $formats );
				if ( ! $value['format'] ) {
					$value['format'] = reset( $original_formats );
				}
			} elseif ( isset( $this->field_types[ $field ] ) ) {
				$value['format'] = $this->field_types[ $field ];
			}

			$data[ $field ] = $value;
		}

		return $data;
	}

	/**
	 * Adds field charsets to field/value/format arrays generated by
	 * the {@see process_field_formats()} method.
	 *
	 * @since  4.2.0
	 * @access protected
	 *
	 * @param array  $data  As it comes from the {@see process_field_formats()} method.
	 * @param string $table Table name.
	 *
	 * @return The same array as $data with additional 'charset' keys.
	 */
	protected function process_field_charsets( $data, $table ) {
		foreach ( $data as $field => $value ) {
			if ( '%d' === $value['format'] || '%f' === $value['format'] ) {
				// We can skip this field if we know it isn't a string.
				// This checks %d/%f versus ! %s because it's sprintf() could take more.
				$value['charset'] = false;
			} elseif ( $this->check_ascii( $value['value'] ) ) {
				// If it's ASCII, then we don't need the charset. We can skip this field.
				$value['charset'] = false;
			} else {
				$value['charset'] = $this->get_col_charset( $table, $field );

				// This isn't ASCII. Don't have strip_invalid_text() re-check.
				$value['ascii'] = false;
			}

			$data[ $field ] = $value;
		}

		return $data;
	}

	/**
	 * Retrieve one variable from the database.
	 *
	 * Executes a SQL query and returns the value from the SQL result.
	 * If the SQL result contains more than one column and/or more than one row, this function returns the value in the
	 * column and row specified. If $query is null, this function returns the value in the specified column and row
	 * from the previous SQL result.
	 *
	 * @since 0.71
	 *
	 * @param string|null $query Optional. SQL query. Defaults to null, use the result from the previous query.
	 * @param int         $x     Optional. Column of value to return. Indexed from 0.
	 * @param int         $y     Optional. Row of value to return. Indexed from 0.
	 *
	 * @return string|null Database query result (as string), or null on failure
	 */
	public function get_var( $query = null, $x = 0, $y = 0 ) {
		$this->func_call = "\$db->get_var(\"$query\", $x, $y)";

		if ( $this->check_safe_collation( $query ) ) {
			$this->check_current_query = false;
		}

		if ( $query ) {
			$this->query( $query );
		}

		// Extract var out of cached results based x,y vals
		if ( ! empty( $this->last_result[ $y ] ) ) {
			$values = array_values( get_object_vars( $this->last_result[ $y ] ) );
		}

		// If there is a value return it else return null
		return ( isset( $values[ $x ] ) && $values[ $x ] !== '' ) ? $values[ $x ] : null;
	}

	/**
	 * Retrieve one row from the database.
	 *
	 * Executes a SQL query and returns the row from the SQL result.
	 *
	 * @since 0.71
	 *
	 * @param string|null $query  SQL query.
	 * @param string      $output Optional. one of ARRAY_A | ARRAY_N | OBJECT constants. Return an associative array
	 *                            (column => value, ...), a numerically indexed array (0 => value, ...) or an object (
	 *                            ->column = value ), respectively.
	 * @param int         $y      Optional. Row to return. Indexed from 0.
	 *
	 * @return mixed Database query result in format specified by $output or null on failure
	 */
	public function get_row( $query = null, $output = ARRAY_A, $y = 0 ) {
		$this->func_call = "\$db->get_row(\"$query\",$output,$y)";

		if ( $this->check_safe_collation( $query ) ) {
			$this->check_current_query = false;
		}

		if ( $query ) {
			$this->query( $query );
		} else {
			$this->Status = $this->last_error;

			return null;
		}

		if ( ! isset( $this->last_result[ $y ] ) ) {
			$this->Status = $this->last_error;

			return null;
		}

		if ( $output == OBJECT ) {
			$this->Status = 'OK';

			return $this->last_result[ $y ] ? $this->last_result[ $y ] : null;
		} elseif ( $output == ARRAY_A ) {
			$this->Status = 'OK';

			return $this->last_result[ $y ] ? get_object_vars( $this->last_result[ $y ] ) : null;
		} elseif ( $output == ARRAY_N ) {
			$this->Status = 'OK';

			return $this->last_result[ $y ] ? array_values( get_object_vars( $this->last_result[ $y ] ) ) : null;
		} elseif ( strtoupper( $output ) === OBJECT ) {
			// Back compat for OBJECT being previously case insensitive.
			$this->Status = 'OK';

			return $this->last_result[ $y ] ? $this->last_result[ $y ] : null;
		} else {
			$this->Status = $this->last_error;
			$this->print_error( " \$db->get_row(string query, output type, int offset) -- Output type must be one of: OBJECT, ARRAY_A, ARRAY_N" );
		}
	}

	/**
	 * Retrieve one column from the database.
	 *
	 * Executes a SQL query and returns the column from the SQL result.
	 * If the SQL result contains more than one column, this function returns the column specified.
	 * If $query is null, this function returns the specified column from the previous SQL result.
	 *
	 * @since 0.71
	 *
	 * @param string|null $query Optional. SQL query. Defaults to previous query.
	 * @param int         $x     Optional. Column to return. Indexed from 0.
	 *
	 * @return array Database query result. Array indexed from 0 by SQL result row number.
	 */
	public function get_col( $query = null, $x = 0 ) {
		if ( $this->check_safe_collation( $query ) ) {
			$this->check_current_query = false;
		}

		if ( $query ) {
			$this->query( $query );
		}

		$new_array = [];
		// Extract the column values
		for ( $i = 0, $j = count( $this->last_result ); $i < $j; $i ++ ) {
			$new_array[ $i ] = $this->get_var( null, $x, $i );
		}

		$this->Status = 'OK';

		return $new_array;
	}

	/**
	 * Retrieve an entire SQL result set from the database (i.e., many rows)
	 *
	 * Executes a SQL query and returns the entire SQL result.
	 *
	 * @since 0.71
	 *
	 * @param string $query  SQL query.
	 * @param string $output Optional. Any of ARRAY_A | ARRAY_N | OBJECT | OBJECT_K constants. With one of the first
	 *                       three, return an array of rows indexed from 0 by SQL result row number. Each row is an
	 *                       associative array (column => value, ...), a numerically indexed array (0 => value, ...),
	 *                       or an object. ( ->column = value ), respectively. With OBJECT_K, return an associative
	 *                       array of row objects keyed by the value of each row's first column's value. Duplicate keys
	 *                       are discarded.
	 *
	 * @return mixed Database query results
	 */
	public function get_rows( $query = null, $output = ARRAY_A ) {
		$this->func_call = "\$db->get_rows(\"$query\", $output)";

		if ( $this->check_safe_collation( $query ) ) {
			$this->check_current_query = false;
		}

		if ( $query ) {
			$this->query( $query );
		} else {
			$this->Status = $this->last_error;

			return null;
		}

		$new_array = array();

		if ( $output == OBJECT ) {
			// Return an integer-keyed array of row objects
			$this->Status = 'OK';

			return $this->last_result;
		} elseif ( $output == OBJECT_K ) {
			// Return an array of row objects with keys from column 1
			// (Duplicates are discarded)
			foreach ( $this->last_result as $row ) {
				$var_by_ref = get_object_vars( $row );
				$key        = array_shift( $var_by_ref );
				if ( ! isset( $new_array[ $key ] ) ) {
					$new_array[ $key ] = $row;
				}
			}

			$this->Status = 'OK';

			return $new_array;
		} elseif ( strtoupper( $output ) === OBJECT ) {
			// Back compat for OBJECT being previously case insensitive.
			$this->Status = 'OK';

			return $this->last_result;
		} else {
			// Return an integer-keyed array of...

			if ( $this->last_result ) {
				foreach ( (array) $this->last_result as $row ) {
					if ( $output == ARRAY_N ) {
						// ...integer-keyed row arrays
						$new_array[] = array_values( get_object_vars( $row ) );
					} else {
						// ...column name-keyed row arrays
						$new_array[] = get_object_vars( $row );
					}
				}
			}

			$this->Status = 'OK';

			return $new_array;
		}

		return null;
	}


	public function get_rows2( $Q, $page = 1, $pageSize = 50, $range = 3, $output = ARRAY_A, $encode = true ) {
		if ( $page < 1 ) {
			$page = 1;
		}
		if ( $pageSize < 1 ) {
			$pageSize = 1;
		}

		$Q = preg_replace( '/SELECT /', 'SELECT SQL_CALC_FOUND_ROWS ', $Q, 1 );

		$offset = ( $page - 1 ) * $pageSize;
		$Q      .= " LIMIT $offset, $pageSize";
		$rows   = $this->get_rows( $Q, $output );

		$q                = "SELECT FOUND_ROWS()";
		$total_records    = $this->get_var( $q );
		$this->last_query = str_replace( 'SQL_CALC_FOUND_ROWS ', '', $Q );
		if ( ! is_numeric( $total_records ) ) {
			return $total_records;
		}
		$pages = ceil( $total_records / $pageSize );
		if ( $pages == 0 ) {
			$pages = 1;
		}

		if ( $page <= $range ) {
			$start = 1;
		} else {
			$start = $page - $range;
		}
		$end = $start + $range + $range;
		if ( $pages < $end ) {
			$end = $pages;
		}
		$from = ( ( $page * $pageSize ) - $pageSize ) + 1;
		if ( $total_records == 0 ) {
			$from = 0;
		}
		$to = $pageSize * $page;
		if ( $to > $total_records ) {
			$to = $total_records;
		}
		$this->Status = 'OK';

		return [
			'page'          => $page,
			'pages'         => $pages,
			'total_records' => $total_records,
			'rows'          => $rows,
			'start'         => $start,
			'end'           => $end,
			'from'          => $from,
			'to'            => $to,
		];
	}

	/**
	 * Retrieves the character set for the given table.
	 *
	 * @since  4.2.0
	 * @access protected
	 *
	 * @param string $table Table name.
	 *
	 * @return string|WP_Error Table character set, {@see WP_Error} object if it couldn't be found.
	 */
	protected function get_table_charset( $table ) {
		$tablekey = strtolower( $table );

		/**
		 * Filter the table charset value before the DB is checked.
		 *
		 * Passing a non-null value to the filter will effectively short-circuit
		 * checking the DB for the charset, returning that value instead.
		 *
		 * @since 4.2.0
		 *
		 * @param string $charset The character set to use. Default null.
		 * @param string $table   The name of the table being checked.
		 */
		$charset = null;
		if ( null !== $charset ) {
			return $charset;
		}

		if ( isset( $this->table_charset[ $tablekey ] ) ) {
			return $this->table_charset[ $tablekey ];
		}

		$charsets = $columns = [];
		$results  = $this->get_rows( "SHOW FULL COLUMNS FROM `$table`", OBJECT );
		if ( ! $results ) {
			$this->Status = $this->last_error;

			return $this->last_error;
		}

		foreach ( $results as $column ) {
			$columns[ strtolower( $column->Field ) ] = $column;
		}

		$this->col_meta[ $tablekey ] = $columns;

		foreach ( $columns as $column ) {
			if ( ! empty( $column->Collation ) ) {
				list( $charset ) = explode( '_', $column->Collation );
				$charsets[ strtolower( $charset ) ] = true;
			}

			list( $type ) = explode( '(', $column->Type );

			// A binary/blob means the whole query gets treated like this.
			if ( in_array( strtoupper( $type ), [
				'BINARY',
				'VARBINARY',
				'TINYBLOB',
				'MEDIUMBLOB',
				'BLOB',
				'LONGBLOB',
			] ) ) {
				$this->table_charset[ $tablekey ] = 'binary';

				return 'binary';
			}
		}

		// utf8mb3 is an alias for utf8.
		if ( isset( $charsets['utf8mb3'] ) ) {
			$charsets['utf8'] = true;
			unset( $charsets['utf8mb3'] );
		}

		// Check if we have more than one charset in play.
		$count = count( $charsets );
		if ( 1 === $count ) {
			$charset = key( $charsets );
		} elseif ( 0 === $count ) {
			// No charsets, assume this table can store whatever.
			$charset = false;
		} else {
			// More than one charset. Remove latin1 if present and recalculate.
			unset( $charsets['latin1'] );
			$count = count( $charsets );
			if ( 1 === $count ) {
				// Only one charset (besides latin1).
				$charset = key( $charsets );
			} elseif ( 2 === $count && isset( $charsets['utf8'], $charsets['utf8mb4'] ) ) {
				// Two charsets, but they're utf8 and utf8mb4, use utf8.
				$charset = 'utf8mb4';
			} else {
				// Two mixed character sets. ascii.
				$charset = 'ascii';
			}
		}

		$this->table_charset[ $tablekey ] = $charset;
		$this->Status                     = 'OK';

		return $charset;
	}

	/**
	 * Retrieves the character set for the given column.
	 *
	 * @since  4.2.0
	 * @access public
	 *
	 * @param string $table  Table name.
	 * @param string $column Column name.
	 *
	 * @return mixed Column character set as a string. False if the column has no
	 *               character set. {@see WP_Error} object if there was an error.
	 */
	public function get_col_charset( $table, $column ) {
		$tablekey  = strtolower( $table );
		$columnkey = strtolower( $column );

		/**
		 * Filter the column charset value before the DB is checked.
		 *
		 * Passing a non-null value to the filter will short-circuit
		 * checking the DB for the charset, returning that value instead.
		 *
		 * @since 4.2.0
		 *
		 * @param string $charset The character set to use. Default null.
		 * @param string $table   The name of the table being checked.
		 * @param string $column  The name of the column being checked.
		 */
		$charset = "utf8";
		if ( null !== $charset ) {
			return $charset;
		}

		// Skip this entirely if this isn't a MySQL database.
		if ( false === $this->is_mysql ) {
			return false;
		}

		if ( empty( $this->table_charset[ $tablekey ] ) ) {
			// This primes column information for us.
			$table_charset = $this->get_table_charset( $table );
			if ( is_wp_error( $table_charset ) ) {
				return $table_charset;
			}
		}

		// If still no column information, return the table charset.
		if ( empty( $this->col_meta[ $tablekey ] ) ) {
			return $this->table_charset[ $tablekey ];
		}

		// If this column doesn't exist, return the table charset.
		if ( empty( $this->col_meta[ $tablekey ][ $columnkey ] ) ) {
			return $this->table_charset[ $tablekey ];
		}

		// Return false when it's not a string column.
		if ( empty( $this->col_meta[ $tablekey ][ $columnkey ]->Collation ) ) {
			return false;
		}

		@list( $charset ) = explode( '_', $this->col_meta[ $tablekey ][ $columnkey ]->Collation );

		return $charset;
	}

	/**
	 * Check if a string is ASCII.
	 *
	 * The negative regex is faster for non-ASCII strings, as it allows
	 * the search to finish as soon as it encounters a non-ASCII character.
	 *
	 * @since  4.2.0
	 * @access protected
	 *
	 * @param string $string String to check.
	 *
	 * @return bool True if ASCII, false if not.
	 */
	protected function check_ascii( $string ) {
		if ( is_array( $string ) || is_object( $string ) ) {
			return false;
		}
		if ( function_exists( 'mb_check_encoding' ) ) {
			if ( mb_check_encoding( $string, 'ASCII' ) ) {
				return true;
			}
		} elseif ( ! preg_match( '/[^\x00-\x7F]/', $string ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Check if the query is accessing a collation considered safe on the current version of MySQL.
	 *
	 * @since  4.2.0
	 * @access protected
	 *
	 * @param string $query The query to check.
	 *
	 * @return bool True if the collation is safe, false if it isn't.
	 */
	protected function check_safe_collation( $query ) {
		if ( $this->checking_collation ) {
			return true;
		}

		// We don't need to check the collation for queries that don't read data.
		$query = ltrim( $query, "\r\n\t (" );
		if ( preg_match( '/^(?:SHOW|DESCRIBE|DESC|EXPLAIN)\s/i', $query ) ) {
			return true;
		}

		// All-ASCII queries don't need extra checking.
		if ( $this->check_ascii( $query ) ) {
			return true;
		}

		$table = $this->get_table_from_query( $query );
		if ( ! $table ) {
			return false;
		}

		$this->checking_collation = true;
		$collation                = $this->get_table_charset( $table );
		$this->checking_collation = false;

		// Tables with no collation, or latin1 only, don't need extra checking.
		if ( false === $collation || 'latin1' === $collation ) {
			return true;
		}

		$table = strtolower( $table );
		if ( empty( $this->col_meta[ $table ] ) ) {
			return false;
		}

		// If any of the columns don't have one of these collations, it needs more sanity checking.
		foreach ( $this->col_meta[ $table ] as $col ) {
			if ( empty( $col->Collation ) ) {
				continue;
			}

			if ( ! in_array( $col->Collation, [
				'utf8_general_ci',
				'utf8_bin',
				'utf8mb4_general_ci',
				'utf8mb4_bin',
			], true ) ) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Strips any invalid characters based on value/charset pairs.
	 *
	 * @since  4.2.0
	 * @access protected
	 *
	 * @param array $data Array of value arrays. Each value array has the keys
	 *                    'value' and 'charset'. An optional 'ascii' key can be
	 *                    set to false to avoid redundant ASCII checks.
	 *
	 * @return array|WP_Error The $data parameter, with invalid characters removed from
	 *                        each value. This works as a passthrough: any additional keys
	 *                        such as 'field' are retained in each value array. If we cannot
	 *                        remove invalid characters, a {@see WP_Error} object is returned.
	 */
	// If any of the columns don't have one of these collations, it needs more sanity checking.
	protected function strip_invalid_text( $data ) {
		// Some multibyte character sets that we can check in PHP.
		$mb_charsets = [
			'ascii'   => 'ASCII',
			'big5'    => 'BIG-5',
			'eucjpms' => 'eucJP-win',
			'gb2312'  => 'EUC-CN',
			'ujis'    => 'EUC-JP',
			'utf32'   => 'UTF-32',
		];

		$supported_charsets = [];
		if ( function_exists( 'mb_list_encodings' ) ) {
			$supported_charsets = mb_list_encodings();
		}

		$db_check_string = false;

		foreach ( $data as &$value ) {
			$charset = $value['charset'];

			// Column isn't a string, or is latin1, which will will happily store anything.
			if ( false === $charset || 'latin1' === $charset ) {
				continue;
			}

			if ( ! is_string( $value['value'] ) ) {
				continue;
			}

			// ASCII is always OK.
			if ( ! isset( $value['ascii'] ) && $this->check_ascii( $value['value'] ) ) {
				continue;
			}

			// Convert the text locally.
			if ( $supported_charsets ) {
				if ( isset( $mb_charsets[ $charset ] ) && in_array( $mb_charsets[ $charset ], $supported_charsets ) ) {
					$value['value'] = mb_convert_encoding( $value['value'], $mb_charsets[ $charset ], $mb_charsets[ $charset ] );
					continue;
				}
			}

			// utf8 can be handled by regex, which is a bunch faster than a DB lookup.
			if ( 'utf8' === $charset || 'utf8mb3' === $charset || 'utf8mb4' === $charset ) {
				$regex = '/
					(
						(?: [\x00-\x7F]                  # single-byte sequences   0xxxxxxx
						|   [\xC2-\xDF][\x80-\xBF]       # double-byte sequences   110xxxxx 10xxxxxx
						|   \xE0[\xA0-\xBF][\x80-\xBF]   # triple-byte sequences   1110xxxx 10xxxxxx * 2
						|   [\xE1-\xEC][\x80-\xBF]{2}
						|   \xED[\x80-\x9F][\x80-\xBF]
						|   [\xEE-\xEF][\x80-\xBF]{2}';

				if ( 'utf8mb4' === $charset ) {
					$regex .= '
						|    \xF0[\x90-\xBF][\x80-\xBF]{2} # four-byte sequences   11110xxx 10xxxxxx * 3
						|    [\xF1-\xF3][\x80-\xBF]{3}
						|    \xF4[\x80-\x8F][\x80-\xBF]{2}
					';
				}

				$regex          .= '){1,50}                          # ...one or more times
					)
					| .                                  # anything else
					/x';
				$value['value'] = preg_replace( $regex, '$1', $value['value'] );
				continue;
			}

			// We couldn't use any local conversions, send it to the DB.
			$value['db'] = $db_check_string = true;
		}
		unset( $value ); // Remove by reference.

		if ( $db_check_string ) {
			$queries = [];
			foreach ( $data as $col => $value ) {
				if ( ! empty( $value['db'] ) ) {
					if ( ! isset( $queries[ $value['charset'] ] ) ) {
						$queries[ $value['charset'] ] = [];
					}

					// Split the CONVERT() calls by charset, so we can make sure the connection is right
					$queries[ $value['charset'] ][ $col ] = $this->prepare( "CONVERT( %s USING {$value['charset']} )", $value['value'] );
				}
			}

			$connection_charset = $this->charset;
			foreach ( $queries as $charset => $query ) {
				if ( ! $query ) {
					continue;
				}

				// Change the charset to match the string(s) we're converting
				if ( $charset !== $connection_charset ) {
					$connection_charset = $charset;
					$this->set_charset( $this->dbh, $charset );
				}

				$this->check_current_query = false;

				$row = $this->get_row( "SELECT " . implode( ', ', $query ), ARRAY_N );
				if ( ! $row ) {
					$this->set_charset( $this->dbh, $connection_charset );

					return false;
					//return new WP_Error( 'wpdb_strip_invalid_text_failure' );
				}

				$cols      = array_keys( $query );
				$col_count = count( $cols );
				for ( $ii = 0; $ii < $col_count; $ii ++ ) {
					$data[ $cols[ $ii ] ]['value'] = $row[ $ii ];
				}
			}

			// Don't forget to change the charset back!
			if ( $connection_charset !== $this->charset ) {
				if ( ! empty( $this ) ) {
					$this->set_charset( $this->dbh );
				}
			}
		}

		return $data;
	}

	/**
	 * Strips any invalid characters from the query.
	 *
	 * @since  4.2.0
	 * @access protected
	 *
	 * @param string $query Query to convert.
	 *
	 * @return string|WP_Error The converted query, or a {@see WP_Error} object if the conversion fails.
	 */
	protected function strip_invalid_text_from_query( $query ) {
		$table = $this->get_table_from_query( $query );
		if ( $table ) {
			$charset = $this->get_table_charset( $table );

			// We can't reliably strip text from tables containing binary/blob columns
			if ( 'binary' === $charset ) {
				return $query;
			}
		} else {
			$charset = $this->charset;
		}

		$data = [
			'value'   => $query,
			'charset' => $charset,
			'ascii'   => false,
		];

		$data             = $this->strip_invalid_text( [ $data ] );
		$data[0]['value'] = isset( $data[0]['value'] ) ? $data[0]['value'] : '';

		return $data[0]['value'];
	}

	/**
	 * Strips any invalid characters from the string for a given table and column.
	 *
	 * @since  4.2.0
	 * @access public
	 *
	 * @param string $table  Table name.
	 * @param string $column Column name.
	 * @param string $value  The text to check.
	 */
	public function strip_invalid_text_for_column( $table, $column, $value ) {
		if ( ! is_string( $value ) || $this->check_ascii( $value ) ) {
			return $value;
		}

		$charset = $this->get_col_charset( $table, $column );
		if ( ! $charset ) {
			// Not a string column.
			return $value;
		} else {
			$data = [
				$column => [
					'value'   => $value,
					'charset' => $charset,
					'ascii'   => false,
				],
			];
		}

		$data = $this->strip_invalid_text( $data );

		return $data[ $column ]['value'];
	}

	/**
	 * Find the first table name referenced in a query.
	 *
	 * @since  4.2.0
	 * @access protected
	 *
	 * @param string $query The query to search.
	 *
	 * @return string|false $table The table name found, or false if a table couldn't be found.
	 */
	protected function get_table_from_query( $query ) {
		// Remove characters that can legally trail the table name.
		$query = rtrim( $query, ';/-#' );

		// Allow (select...) union [...] style queries. Use the first query's table name.
		$query = ltrim( $query, "\r\n\t (" );

		/*
		 * Strip everything between parentheses except nested selects and use only 1,000
		 * chars of the query.
		 */
		$query = preg_replace( '/\((?!\s*select)[^(]*?\)/is', '()', substr( $query, 0, 1000 ) );

		// Quickly match most common queries.
		if ( preg_match( '/^\s*(?:' . 'SELECT.*?\s+FROM' . '|INSERT(?:\s+LOW_PRIORITY|\s+DELAYED|\s+HIGH_PRIORITY)?(?:\s+IGNORE)?(?:\s+INTO)?' . '|REPLACE(?:\s+LOW_PRIORITY|\s+DELAYED)?(?:\s+INTO)?' . '|UPDATE(?:\s+LOW_PRIORITY)?(?:\s+IGNORE)?' . '|DELETE(?:\s+LOW_PRIORITY|\s+QUICK|\s+IGNORE)*(?:\s+FROM)?' . ')\s+`?([\w-]+)`?/is', $query, $maybe ) ) {
			return $maybe[1];
		}

		// SHOW TABLE STATUS and SHOW TABLES
		if ( preg_match( '/^\s*(?:' . 'SHOW\s+TABLE\s+STATUS.+(?:LIKE\s+|WHERE\s+Name\s*=\s*)' . '|SHOW\s+(?:FULL\s+)?TABLES.+(?:LIKE\s+|WHERE\s+Name\s*=\s*)' . ')\W([\w-]+)\W/is', $query, $maybe ) ) {
			return $maybe[1];
		}

		// Big pattern for the rest of the table-related queries.
		if ( preg_match( '/^\s*(?:' . '(?:EXPLAIN\s+(?:EXTENDED\s+)?)?SELECT.*?\s+FROM' . '|DESCRIBE|DESC|EXPLAIN|HANDLER' . '|(?:LOCK|UNLOCK)\s+TABLE(?:S)?' . '|(?:RENAME|OPTIMIZE|BACKUP|RESTORE|CHECK|CHECKSUM|ANALYZE|REPAIR).*\s+TABLE' . '|TRUNCATE(?:\s+TABLE)?' . '|CREATE(?:\s+TEMPORARY)?\s+TABLE(?:\s+IF\s+NOT\s+EXISTS)?' . '|ALTER(?:\s+IGNORE)?\s+TABLE' . '|DROP\s+TABLE(?:\s+IF\s+EXISTS)?' . '|CREATE(?:\s+\w+)?\s+INDEX.*\s+ON' . '|DROP\s+INDEX.*\s+ON' . '|LOAD\s+DATA.*INFILE.*INTO\s+TABLE' . '|(?:GRANT|REVOKE).*ON\s+TABLE' . '|SHOW\s+(?:.*FROM|.*TABLE)' . ')\s+\(*\s*`?([\w-]+)`?\s*\)*/is', $query, $maybe ) ) {
			return $maybe[1];
		}

		return false;
	}

	/**
	 * Load the column metadata from the last query.
	 *
	 * @since  3.5.0
	 *
	 * @access protected
	 */
	protected function load_col_info() {
		if ( $this->col_info ) {
			return;
		}

		for ( $i = 0; $i < @mysqli_num_fields( $this->result ); $i ++ ) {
			$this->col_info[ $i ] = @mysqli_fetch_field( $this->result );
		}

	}

	/**
	 * Retrieve column metadata from the last query.
	 *
	 * @since 0.71
	 *
	 * @param string $info_type  Optional. Type one of name, table, def, max_length, not_null, primary_key,
	 *                           multiple_key, unique_key, numeric, blob, type, unsigned, zerofill
	 * @param int    $col_offset Optional. 0: col name. 1: which table the col's in. 2: col's max length. 3: if the col
	 *                           is numeric. 4: col's type
	 *
	 * @return mixed Column Results
	 */
	public function get_col_info( $info_type = 'name', $col_offset = - 1 ) {
		$this->load_col_info();

		if ( $this->col_info ) {
			if ( $col_offset == - 1 ) {
				$i         = 0;
				$new_array = [];
				foreach ( (array) $this->col_info as $col ) {
					$new_array[ $i ] = $col->{$info_type};
					$i ++;
				}

				return $new_array;
			} else {
				return $this->col_info[ $col_offset ]->{$info_type};
			}
		}
	}

	/**
	 * Starts the timer, for debugging purposes.
	 *
	 * @since 1.5.0
	 *
	 * @return bool
	 */
	public function timer_start() {
		$this->time_start = microtime( true );

		return true;
	}

	/**
	 * Stops the debugging timer.
	 *
	 * @since 1.5.0
	 *
	 * @return float Total time spent on the query, in seconds
	 */
	public function timer_stop() {
		return ( microtime( true ) - $this->time_start );
	}

	/**
	 * Wraps errors in a nice header and footer and dies.
	 *
	 * Will not die if $show_errors is false.
	 *
	 * @since 1.5.0
	 *
	 * @param string $message    The Error message
	 * @param string $error_code Optional. A Computer readable string to identify the error.
	 *
	 * @return false|void
	 */
	public function bail( $message, $error_code = '500' ) {
		if ( ! $this->show_errors ) {
			$this->error = $message;

			return false;
		}
		die( $message );
	}

	/**
	 * Whether MySQL database is at least the required minimum version.
	 *
	 * @since 2.5.0
	 * @uses  $wp_version
	 * @uses  $required_mysql_version
	 *
	 */
	/*public function check_database_version() {
		global $required_mysql_version;
		// Make sure the server has the required MySQL version
		if ( version_compare( $this->db_version(), $required_mysql_version, '<' ) ) {
			return sprintf( ( '<strong>ERROR</strong>: System requires MySQL %1$s or higher' ), $required_mysql_version );
		}
	}*/

	public function supports_collation() {
		_deprecated_function( __FUNCTION__, '3.5', 'has_cap( \'collation\' )' );

		return $this->has_cap( 'collation' );
	}

	/**
	 * The database character collate.
	 *
	 * @since 3.5.0
	 *
	 * @return string The database character collate.
	 */
	public function get_charset_collate() {
		$charset_collate = '';

		if ( ! empty( $this->charset ) ) {
			$charset_collate = "DEFAULT CHARACTER SET $this->charset";
		}
		if ( ! empty( $this->collate ) ) {
			$charset_collate .= " COLLATE $this->collate";
		}

		return $charset_collate;
	}

	/**
	 * Determine if a database supports a particular feature.
	 *
	 * @since 2.7.0
	 * @since 4.1.0 Support was added for the 'utf8mb4' feature.
	 *
	 * @see   db_version()
	 *
	 * @param string $db_cap The feature to check for. Accepts 'collation',
	 *                       'group_concat', 'subqueries', 'set_charset',
	 *                       or 'utf8mb4'.
	 *
	 * @return bool Whether the database feature is supported, false otherwise.
	 */
	public function has_cap( $db_cap ) {
		$version = $this->db_version();

		switch ( strtolower( $db_cap ) ) {
			case 'collation' :    // @since 2.5.0
			case 'group_concat' : // @since 2.7.0
			case 'subqueries' :   // @since 2.7.0
				return version_compare( $version, '4.1', '>=' );
			case 'set_charset' :
				return version_compare( $version, '5.0.7', '>=' );
			case 'utf8mb4' :      // @since 4.1.0
				return version_compare( $version, '5.5.3', '>=' );
		}

		return false;
	}

	/**
	 * Retrieve the name of the function that called wpdb.
	 *
	 * Searches up the list of functions until it reaches
	 * the one that would most logically had called this method.
	 *
	 * @since 2.5.0
	 *
	 * @return string The name of the calling function
	 */

	/**
	 * The database version number.
	 *
	 * @since 2.7.0
	 *
	 * @return null|string Null on failure, version number on success.
	 */
	public function db_version() {
		$server_info = mysqli_get_server_info( $this->dbh );

		return preg_replace( '/[^0-9.].*/', '', $server_info );
	}

	public function set_string( $string ) {
		return str_replace( "'", "''", $string );
	}

	public function get_option( $field_name, $default = "" ) {
		$this->add_column( [
			'table_name' => self::TableOptions,
			'col_name'   => 'type',
			'type'       => 'varchar',
			'length'     => 7,
			'default'    => 'string',
		] );

		$this->add_column( [
			'table_name' => self::TableOptions,
			'col_name'   => 'update_time',
			'type'       => 'datetime',
		] );

		if ( method_exists( $this, 'select_cols' ) ) {
			$row = $this->select_cols( 'field_value,type' )
			            ->where( 'field_name', $field_name )
			            ->get_row( self::TableOptions );
		} else {
			$row = $this->get_row( $this->prepare( 'SELECT field_value,type FROM ' . self::TableOptions . ' WHERE `field_name`=%s', $field_name ) );
		}

		if ( ! isset( $row['field_value'] ) ) {
			return $default;
		}

		if ( substr( $row['field_value'], 0, 9 ) == "**ARRAY**" ) {
			$row['field_value'] = substr( $row['field_value'], 9 );

			return json_decode( $row['field_value'] );
		} elseif ( function_exists( "is_json" ) && is_json( $row['field_value'] ) ) {
			if ( $field_name == 'abc' ) {
				var_dump( $row );
				die;
			}

			return json_decode( $row['field_value'], true );
		} else {
			switch ( $row['type'] ) {
				case 'bool':
				case 'boolean':
					return (bool) $row['field_value'];
					break;
				case 'int':
				case 'integer':
					return (int) $row['field_value'];
					break;
				case 'double':
				case 'float':
					return (float) $row['field_value'];
					break;
				case 'array':
					return json_decode( $row['field_value'], true );
					break;
				case 'object':
					return json_decode( $row['field_value'] );
					break;
				default:
					return (string) $row['field_value'];
			}

		}

	}

	public function set_option( $field_name, $field_value = "" ) {
		if ( is_array( $field_value ) ) {
			$field_value = json_encode( $field_value );
			$type        = 'array';
		} else if ( is_bool( $field_value ) ) {
			$type = 'bool';
			if ( $field_value ) {
				$field_value = 1;
			} else {
				$field_value = 0;
			}
		} else if ( is_object( $field_value ) ) {
			$field_value = json_encode( $field_value );
			$type        = 'object';
		} else if ( \General::is_really_int( $field_value ) ) {
			$type = 'int';
		} else if ( \General::is_really_float( $field_value ) ) {
			$type = 'float';
		} else {
			$type = 'string';
		}

		$data = [
			'field_name'  => $field_name,
			'field_value' => $field_value,
			'type'        => $type,
			'update_time' => \Dates::mysql_datetime(),
		];
		if ( method_exists( $this, 'where' ) ) {
			$cnt = $this->where( 'field_name', $field_name )->count_result( self::TableOptions );
		} else {
			$cnt = $this->get_var( $this->prepare( 'SELECT COUNT(*) FROM %t WHERE `field_name`=%s', self::TableOptions, $field_name ) );
		}
		if ( $cnt == 0 ) {
			$this->insert( self::TableOptions, $data );
		} else {
			$this->update( self::TableOptions, $data, [ 'field_name' => $field_name ] );
		}

		return $this->Status == 'OK';
	}

	public function remove_option( $field_name ) {
		$response = $this->delete( self::TableOptions, [ 'field_name' => $field_name ] );

		return $response > 0;
	}

	public function make_options_table_file() {
		$Q                  = "SELECT `field_name`, `field_value` FROM `options` WHERE 1";
		$options_table_rows = $this->get_rows( $Q );

		$options_table = [];
		foreach ( $options_table_rows as $options_table_row ) {
			$options_table[ $options_table_row['field_name'] ] = $options_table_row['field_value'];
		}

		$options_table = json_encode( $options_table, JSON_PRETTY_PRINT );
		@file_put_contents( $this->options_ini_file, $options_table );
	}

	public function lastid() {
		return mysqli_insert_id( $this->dbh );
	}

	public function is_column_exists( $columns_name, $table_name ) {
		$Q = "SELECT COUNT(*) FROM information_schema.COLUMNS WHERE TABLE_SCHEMA='" . DBNAME . "'
        AND COLUMN_NAME='$columns_name' AND TABLE_NAME='{$table_name}'";

		return $this->get_var( $Q ) > 0;
	}

	public function drop_table( $table, $ifexists = false ) {
		if ( $ifexists ) {
			$Q = "DROP TABLE IF EXISTS `$table`";
		} else {
			$Q = "DROP TABLE `$table`";
		}

		return $this->query( $Q );
	}

	public function empty_table( $table_name ) {
		$Q        = "TRUNCATE `{$table_name}`";
		$response = $this->query( $Q );

		if ( $response === false ) {
			return $this->last_error;
		}

		return "OK";
	}

	public function get_columns( $table_name ) {
		$Q    = "SHOW columns FROM `{$table_name}`";
		$rows = $this->get_rows( $Q );
		$cols = [];
		if ( empty( $rows ) ) {
			return $cols;
		}
		foreach ( $rows as $row ) {
			$cols[] = $row["Field"];
		}

		return $cols;
	}

	public function detect_mysql() {
		$paths = array( 'mysql' => '', 'mysqldump' => '' );
		global $db;
		if ( substr( PHP_OS, 0, 3 ) == 'WIN' ) {
			$mysql_install = $db->get_row( "SHOW VARIABLES LIKE 'basedir'" );
			if ( $mysql_install ) {
				$install_path       = str_replace( '\\', '/', $mysql_install["Value"] );
				$paths['mysql']     = $install_path . 'bin/mysql.exe';
				$paths['mysqldump'] = $install_path . 'bin/mysqldump.exe';
			} else {
				$paths['mysql']     = 'mysql.exe';
				$paths['mysqldump'] = 'mysqldump.exe';
			}
		} else {
			if ( function_exists( 'exec' ) ) {
				$paths['mysql']     = @exec( 'which mysql' );
				$paths['mysqldump'] = @exec( 'which mysqldump' );
			} else {
				$paths['mysql']     = 'mysql';
				$paths['mysqldump'] = 'mysqldump';
			}
		}

		return $paths;
	}

	public function execute_backup( $command ) {
		$backup_options = $this->get_option( 'dbmanager_options' );
		$this->check_backup_files();
		if ( substr( PHP_OS, 0, 3 ) == 'WIN' ) {
			$writable_dir = $backup_options['path'];
			$tmpnam       = $writable_dir . '/wp-dbmanager.bat';
			$fp           = fopen( $tmpnam, 'w' );
			fwrite( $fp, $command );
			fclose( $fp );
			@system( $tmpnam . ' > NUL', $error );
			@unlink( $tmpnam );
		} else {
			@passthru( $command, $error );
		}

		return $error;
	}

	function check_backup_files() {
		$database_files = array();
		if ( ! \File::is_emtpy_folder( DB_PATH ) ) {
			if ( $handle = opendir( DB_PATH ) ) {
				while ( false !== ( $file = readdir( $handle ) ) ) {
					if ( $file != '.' && $file != '..' && ( \File::get_extension( $file ) == 'sql' || \File::get_extension( $file ) == 'gz' ) ) {
						$database_files[] = $file;
					}
				}
				closedir( $handle );
				sort( $database_files );
			}
		}
		$Files = 10;
		if ( sizeof( $database_files ) >= $Files ) {
			for ( $x = 0; $x <= ( sizeof( $database_files ) - $Files ); $x ++ ) {
				@unlink( DB_PATH . $database_files[ $x ] );
			}
		}
	}

	function make_backup() {
		$this->check_backup_files();
		$this->set_option( "db_backup_time", time() );
		include_once( ROOT_PATH . "modules/mysql_dump.inc.php" );
		$dbpath     = DB_PATH . time() . "__-__" . DBNAME . ".sql";
		$gzdbpath   = $dbpath . ".gz";
		$mysql_dump = new MYSQL_DUMP( DBSERVER, DBUSER, DBPASS );
		$sql        = $mysql_dump->dumpDB( DBNAME, HAR_ALL_TABLES ); //Takes all database backups

		if ( $sql == false ) {
			echo $mysql_dump->error();
		} else {
			$mysql_dump->save_sql( $sql, $dbpath );
			$fp = gzopen( $gzdbpath, 'w9' );
			@gzwrite( $fp, file_get_contents( $dbpath ) );
			gzclose( $fp );
			if ( file_exists( $gzdbpath ) ) {
				@unlink( $dbpath );
			}
		}
	}

	function add_meta( $table_name, $field_name, $field_value, $primary_key, $field_label = "" ) {
		if ( is_array( $field_value ) || is_object( $field_value ) ) {
			$field_value = json_encode( $field_value );
		}

		if ( $field_label == "" || empty( $field_label ) ) {
			$field_label = ucwords( str_replace( "_", " ", trim( $field_name ) ) );
		}

		$data = array(
			"table_name"  => $table_name,
			"field_name"  => $field_name,
			"field_value" => $field_value,
			"field_label" => $field_label,
			"primary_key" => $primary_key,
		);
		$Q    = "SELECT `id` FROM `meta_data` WHERE `table_name`='{$table_name}' AND `field_name`='$field_name' AND `primary_key`='$primary_key'";
		$id   = $this->get_var( $Q );

		if ( ! $id ) {
			$response = $this->insert( "meta_data", $data );
			if ( $response !== false ) {
				$id = $this->lastid();
			}
		} else {
			$response = $this->update( "meta_data", $data, array( "id" => $id ) );
		}
		if ( $response === false ) {
			return $this->last_error;
		} else {
			return "OK" . $id;
		}
	}

	function get_meta( $table_name, $field_name, $primary_key ) {
		$Q = "SELECT `field_value` FROM `meta_data` WHERE `table_name`='{$table_name}' AND `field_name`='$field_name' AND `primary_key`='$primary_key'";

		$field_value = $this->get_var( $Q );
		if ( is_json( $field_value ) ) {
			$field_value = json_decode( $field_value, true );
		}

		return $field_value;
	}

	function delete_meta( $table_name, $field_name, $primary_key ) {
		$response = $this->delete( "meta_data", array(
			"table_name"  => $table_name,
			"field_name"  => $field_name,
			"primary_key" => $primary_key,
		) );
		if ( $response === false ) {
			return $this->last_error;
		} else {
			return "OK";
		}
	}

	public function profiler() {
		if ( ! \General::localhost() ) {
			return false;
		}

		$rows = $this->get_rows( "SHOW PROFILES" );

		$total = 0;
		$html  = "URL: " . \Urls::current_url_static() . "\n";
		foreach ( $rows as $key => $row ) {
			$html     .= "Query #" . ( $key + 1 ) . ": " . $row['Query'] . "\n";
			$duration = $this->get_var( "select sum(duration) from information_schema.profiling where query_id=" . $row["Query_ID"] );
			$html     .= "Duration: " . $duration . ", Query ID: " . $row['Query_ID'] . "\n\n";

			$total += $duration;
		}
		$html .= "Total Duration: " . $total . "\n";
		$html .= "=========================================================\n";

		debug( $html );
	}

	public function options_table_array() {
		if ( ! file_exists( $this->options_ini_file ) ) {
			return [];
		}
		$content = file_get_contents( $this->options_ini_file );

		if ( $content == '' ) {
			return [];
		}

		$content = json_decode( $content, true );

		if ( is_array( $content ) ) {
			return $content;
		}


		$Q                  = "SELECT `field_name`, `field_value` FROM `options` WHERE 1";
		$options_table_rows = $this->get_rows( $Q );
		foreach ( $options_table_rows as $options_table_row ) {
			$options_table[ $options_table_row['field_name'] ] = $options_table_row['field_value'];
		}

		return $options_table;
	}

	/**
	 * @param (string) $word
	 *
	 * @return bool
	 *
	 * Check if given word is MySQL reserved word or not.
	 */
	public function is_mysql_reserved_word( $word ) {
		$reserved_words = [
			"ACCESSIBLE",
			"ACCOUNT",
			"ACTION",
			"ADD",
			"AFTER",
			"AGAINST",
			"AGGREGATE",
			"ALGORITHM",
			"ALL",
			"ALTER",
			"ALWAYS",
			"ANALYSE",
			"ANALYZE",
			"AND",
			"ANY",
			"AS",
			"ASC",
			"ASCII",
			"ASENSITIVE",
			"AT",
			"AUTOEXTEND_SIZE",
			"AUTO_INCREMENT",
			"AVG",
			"AVG_ROW_LENGTH",
			"BACKUP",
			"BEFORE",
			"BEGIN",
			"BETWEEN",
			"BIGINT",
			"BINARY",
			"BINLOG",
			"BIT",
			"BLOB",
			"BLOCK",
			"BOOL",
			"BOOLEAN",
			"BOTH",
			"BTREE",
			"BY",
			"BYTE",
			"CACHE",
			"CALL",
			"CASCADE",
			"CASCADED",
			"CASE",
			"CATALOG_NAME",
			"CHAIN",
			"CHANGE",
			"CHANGED",
			"CHANNEL",
			"CHAR",
			"CHARACTER",
			"CHARSET",
			"CHECK",
			"CHECKSUM",
			"CIPHER",
			"CLASS_ORIGIN",
			"CLIENT",
			"CLOSE",
			"COALESCE",
			"CODE",
			"COLLATE",
			"COLLATION",
			"COLUMN",
			"COLUMNS",
			"COLUMN_FORMAT",
			"COLUMN_NAME",
			"COMMENT",
			"COMMIT",
			"COMMITTED",
			"COMPACT",
			"COMPLETION",
			"COMPRESSED",
			"COMPRESSION",
			"CONCURRENT",
			"CONDITION",
			"CONNECTION",
			"CONSISTENT",
			"CONSTRAINT",
			"CONSTRAINT_CATALOG",
			"CONSTRAINT_NAME",
			"CONSTRAINT_SCHEMA",
			"CONTAINS",
			"CONTEXT",
			"CONTINUE",
			"CONVERT",
			"CPU",
			"CREATE",
			"CROSS",
			"CUBE",
			"CURRENT",
			"CURRENT_DATE",
			"CURRENT_TIME",
			"CURRENT_TIMESTAMP",
			"CURRENT_USER",
			"CURSOR",
			"CURSOR_NAME",
			"DATA",
			"DATABASE",
			"DATABASES",
			"DATAFILE",
			"DATE",
			"DATETIME",
			"DAY",
			"DAY_HOUR",
			"DAY_MICROSECOND",
			"DAY_MINUTE",
			"DAY_SECOND",
			"DEALLOCATE",
			"DEC",
			"DECIMAL",
			"DECLARE",
			"DEFAULT",
			"DEFAULT_AUTH",
			"DEFINER",
			"DELAYED",
			"DELAY_KEY_WRITE",
			"DELETE",
			"DESC",
			"DESCRIBE",
			"DES_KEY_FILE",
			"DETERMINISTIC",
			"DIAGNOSTICS",
			"DIRECTORY",
			"DISABLE",
			"DISCARD",
			"DISK",
			"DISTINCT",
			"DISTINCTROW",
			"DIV",
			"DO",
			"DOUBLE",
			"DROP",
			"DUAL",
			"DUMPFILE",
			"DUPLICATE",
			"DYNAMIC",
			"EACH",
			"ELSE",
			"ELSEIF",
			"ENABLE",
			"ENCLOSED",
			"ENCRYPTION",
			"END",
			"ENDS",
			"ENGINE",
			"ENGINES",
			"ENUM",
			"ERROR",
			"ERRORS",
			"ESCAPE",
			"ESCAPED",
			"EVENT",
			"EVENTS",
			"EVERY",
			"EXCHANGE",
			"EXECUTE",
			"EXISTS",
			"EXIT",
			"EXPANSION",
			"EXPIRE",
			"EXPLAIN",
			"EXPORT",
			"EXTENDED",
			"EXTENT_SIZE",
			"FALSE",
			"FAST",
			"FAULTS",
			"FETCH",
			"FIELDS",
			"FILE",
			"FILE_BLOCK_SIZE",
			"FILTER",
			"FIRST",
			"FIXED",
			"FLOAT",
			"FLOAT4",
			"FLOAT8",
			"FLUSH",
			"FOLLOWS",
			"FOR",
			"FORCE",
			"FOREIGN",
			"FORMAT",
			"FOUND",
			"FROM",
			"FULL",
			"FULLTEXT",
			"FUNCTION",
			"GENERAL",
			"GENERATED",
			"GEOMETRY",
			"GEOMETRYCOLLECTION",
			"GET",
			"GET_FORMAT",
			"GLOBAL",
			"GRANT",
			"GRANTS",
			"GROUP",
			"GROUP_REPLICATION",
			"HANDLER",
			"HASH",
			"HAVING",
			"HELP",
			"HIGH_PRIORITY",
			"HOST",
			"HOSTS",
			"HOUR",
			"HOUR_MICROSECOND",
			"HOUR_MINUTE",
			"HOUR_SECOND",
			"IDENTIFIED",
			"IF",
			"IGNORE",
			"IGNORE_SERVER_IDS",
			"IMPORT",
			"IN",
			"INDEX",
			"INDEXES",
			"INFILE",
			"INITIAL_SIZE",
			"INNER",
			"INOUT",
			"INSENSITIVE",
			"INSERT",
			"INSERT_METHOD",
			"INSTALL",
			"INSTANCE",
			"INT",
			"INT1",
			"INT2",
			"INT3",
			"INT4",
			"INT8",
			"INTEGER",
			"INTERVAL",
			"INTO",
			"INVOKER",
			"IO",
			"IO_AFTER_GTIDS",
			"IO_BEFORE_GTIDS",
			"IO_THREAD",
			"IPC",
			"IS",
			"ISOLATION",
			"ISSUER",
			"ITERATE",
			"JOIN",
			"JSON",
			"KEY",
			"KEYS",
			"KEY_BLOCK_SIZE",
			"KILL",
			"LANGUAGE",
			"LAST",
			"LEADING",
			"LEAVE",
			"LEAVES",
			"LEFT",
			"LESS",
			"LEVEL",
			"LIKE",
			"LIMIT",
			"LINEAR",
			"LINES",
			"LINESTRING",
			"LIST",
			"LOAD",
			"LOCAL",
			"LOCALTIME",
			"LOCALTIMESTAMP",
			"LOCK",
			"LOCKS",
			"LOGFILE",
			"LOGS",
			"LONG",
			"LONGBLOB",
			"LONGTEXT",
			"LOOP",
			"LOW_PRIORITY",
			"MASTER",
			"MASTER_AUTO_POSITION",
			"MASTER_BIND",
			"MASTER_CONNECT_RETRY",
			"MASTER_DELAY",
			"MASTER_HEARTBEAT_PERIOD",
			"MASTER_HOST",
			"MASTER_LOG_FILE",
			"MASTER_LOG_POS",
			"MASTER_PASSWORD",
			"MASTER_PORT",
			"MASTER_RETRY_COUNT",
			"MASTER_SERVER_ID",
			"MASTER_SSL",
			"MASTER_SSL_CA",
			"MASTER_SSL_CAPATH",
			"MASTER_SSL_CERT",
			"MASTER_SSL_CIPHER",
			"MASTER_SSL_CRL",
			"MASTER_SSL_CRLPATH",
			"MASTER_SSL_KEY",
			"MASTER_SSL_VERIFY_SERVER_CERT",
			"MASTER_TLS_VERSION",
			"MASTER_USER",
			"MATCH",
			"MAXVALUE",
			"MAX_CONNECTIONS_PER_HOUR",
			"MAX_QUERIES_PER_HOUR",
			"MAX_ROWS",
			"MAX_SIZE",
			"MAX_STATEMENT_TIME",
			"MAX_UPDATES_PER_HOUR",
			"MAX_USER_CONNECTIONS",
			"MEDIUM",
			"MEDIUMBLOB",
			"MEDIUMINT",
			"MEDIUMTEXT",
			"MEMORY",
			"MERGE",
			"MESSAGE_TEXT",
			"MICROSECOND",
			"MIDDLEINT",
			"MIGRATE",
			"MINUTE",
			"MINUTE_MICROSECOND",
			"MINUTE_SECOND",
			"MIN_ROWS",
			"MOD",
			"MODE",
			"MODIFIES",
			"MODIFY",
			"MONTH",
			"MULTILINESTRING",
			"MULTIPOINT",
			"MULTIPOLYGON",
			"MUTEX",
			"MYSQL_ERRNO",
			"NAME",
			"NAMES",
			"NATIONAL",
			"NATURAL",
			"NCHAR",
			"NDB",
			"NDBCLUSTER",
			"NEVER",
			"NEW",
			"NEXT",
			"NO",
			"NODEGROUP",
			"NONBLOCKING",
			"NONE",
			"NOT",
			"NO_WAIT",
			"NO_WRITE_TO_BINLOG",
			"NULL",
			"NUMBER",
			"NUMERIC",
			"NVARCHAR",
			"OFFSET",
			"OLD_PASSWORD",
			"ON",
			"ONE",
			"ONLY",
			"OPEN",
			"OPTIMIZE",
			"OPTIMIZER_COSTS",
			"OPTION",
			"OPTIONALLY",
			"OPTIONS",
			"OR",
			"ORDER",
			"OUT",
			"OUTER",
			"OUTFILE",
			"OWNER",
			"PACK_KEYS",
			"PAGE",
			"PARSER",
			"PARSE_GCOL_EXPR",
			"PARTIAL",
			"PARTITION",
			"PARTITIONING",
			"PARTITIONS",
			"PASSWORD",
			"PHASE",
			"PLUGIN",
			"PLUGINS",
			"PLUGIN_DIR",
			"POINT",
			"POLYGON",
			"PORT",
			"PRECEDES",
			"PRECISION",
			"PREPARE",
			"PRESERVE",
			"PREV",
			"PRIMARY",
			"PRIVILEGES",
			"PROCEDURE",
			"PROCESSLIST",
			"PROFILE",
			"PROFILES",
			"PROXY",
			"PURGE",
			"QUARTER",
			"QUERY",
			"QUICK",
			"RANGE",
			"READ",
			"READS",
			"READ_ONLY",
			"READ_WRITE",
			"REAL",
			"REBUILD",
			"RECOVER",
			"REDOFILE",
			"REDO_BUFFER_SIZE",
			"REDUNDANT",
			"REFERENCES",
			"REGEXP",
			"RELAY",
			"RELAYLOG",
			"RELAY_LOG_FILE",
			"RELAY_LOG_POS",
			"RELAY_THREAD",
			"RELEASE",
			"RELOAD",
			"REMOVE",
			"RENAME",
			"REORGANIZE",
			"REPAIR",
			"REPEAT",
			"REPEATABLE",
			"REPLACE",
			"REPLICATE_DO_DB",
			"REPLICATE_DO_TABLE",
			"REPLICATE_IGNORE_DB",
			"REPLICATE_IGNORE_TABLE",
			"REPLICATE_REWRITE_DB",
			"REPLICATE_WILD_DO_TABLE",
			"REPLICATE_WILD_IGNORE_TABLE",
			"REPLICATION",
			"REQUIRE",
			"RESET",
			"RESIGNAL",
			"RESTORE",
			"RESTRICT",
			"RESUME",
			"RETURN",
			"RETURNED_SQLSTATE",
			"RETURNS",
			"REVERSE",
			"REVOKE",
			"RIGHT",
			"RLIKE",
			"ROLLBACK",
			"ROLLUP",
			"ROTATE",
			"ROUTINE",
			"ROW",
			"ROWS",
			"ROW_COUNT",
			"ROW_FORMAT",
			"RTREE",
			"SAVEPOINT",
			"SCHEDULE",
			"SCHEMA",
			"SCHEMAS",
			"SCHEMA_NAME",
			"SECOND",
			"SECOND_MICROSECOND",
			"SECURITY",
			"SELECT",
			"SENSITIVE",
			"SEPARATOR",
			"SERIAL",
			"SERIALIZABLE",
			"SERVER",
			"SESSION",
			"SET",
			"SHARE",
			"SHOW",
			"SHUTDOWN",
			"SIGNAL",
			"SIGNED",
			"SIMPLE",
			"SLAVE",
			"SLOW",
			"SMALLINT",
			"SNAPSHOT",
			"SOCKET",
			"SOME",
			"SONAME",
			"SOUNDS",
			"SOURCE",
			"SPATIAL",
			"SPECIFIC",
			"SQL",
			"SQLEXCEPTION",
			"SQLSTATE",
			"SQLWARNING",
			"SQL_AFTER_GTIDS",
			"SQL_AFTER_MTS_GAPS",
			"SQL_BEFORE_GTIDS",
			"SQL_BIG_RESULT",
			"SQL_BUFFER_RESULT",
			"SQL_CACHE",
			"SQL_CALC_FOUND_ROWS",
			"SQL_NO_CACHE",
			"SQL_SMALL_RESULT",
			"SQL_THREAD",
			"SQL_TSI_DAY",
			"SQL_TSI_HOUR",
			"SQL_TSI_MINUTE",
			"SQL_TSI_MONTH",
			"SQL_TSI_QUARTER",
			"SQL_TSI_SECOND",
			"SQL_TSI_WEEK",
			"SQL_TSI_YEAR",
			"SSL",
			"STACKED",
			"START",
			"STARTING",
			"STARTS",
			"STATS_AUTO_RECALC",
			"STATS_PERSISTENT",
			"STATS_SAMPLE_PAGES",
			"STATUS",
			"STOP",
			"STORAGE",
			"STORED",
			"STRAIGHT_JOIN",
			"STRING",
			"SUBCLASS_ORIGIN",
			"SUBJECT",
			"SUBPARTITION",
			"SUBPARTITIONS",
			"SUPER",
			"SUSPEND",
			"SWAPS",
			"SWITCHES",
			"TABLE",
			"TABLES",
			"TABLESPACE",
			"TABLE_CHECKSUM",
			"TABLE_NAME",
			"TEMPORARY",
			"TEMPTABLE",
			"TERMINATED",
			"TEXT",
			"THAN",
			"THEN",
			"TIME",
			"TIMESTAMP",
			"TIMESTAMPADD",
			"TIMESTAMPDIFF",
			"TINYBLOB",
			"TINYINT",
			"TINYTEXT",
			"TO",
			"TRAILING",
			"TRANSACTION",
			"TRIGGER",
			"TRIGGERS",
			"TRUE",
			"TRUNCATE",
			"TYPE",
			"TYPES",
			"UNCOMMITTED",
			"UNDEFINED",
			"UNDO",
			"UNDOFILE",
			"UNDO_BUFFER_SIZE",
			"UNICODE",
			"UNINSTALL",
			"UNION",
			"UNIQUE",
			"UNKNOWN",
			"UNLOCK",
			"UNSIGNED",
			"UNTIL",
			"UPDATE",
			"UPGRADE",
			"USAGE",
			"USE",
			"USER",
			"USER_RESOURCES",
			"USE_FRM",
			"USING",
			"UTC_DATE",
			"UTC_TIME",
			"UTC_TIMESTAMP",
			"VALIDATION",
			"VALUE",
			"VALUES",
			"VARBINARY",
			"VARCHAR",
			"VARCHARACTER",
			"VARIABLES",
			"VARYING",
			"VIEW",
			"VIRTUAL",
			"WAIT",
			"WARNINGS",
			"WEEK",
			"WEIGHT_STRING",
			"WHEN",
			"WHERE",
			"WHILE",
			"WITH",
			"WITHOUT",
			"WORK",
			"WRAPPER",
			"WRITE",
			"X509",
			"XA",
			"XID",
			"XML",
			"XOR",
			"YEAR",
			"YEAR_MONTH",
			"ZEROFILL",
		];

		$word = (string) $word;
		$word = strtoupper( trim( $word ) );

		return in_array( $word, $reserved_words );
	}

	function add_column( $args = [] ) {
		$default = [
			'table_name'     => '',
			'col_name'       => '',
			'new_col_name'   => null,
			'type'           => 'int',
			'length'         => '11',

			// Set default value
			'default'        => false,

			// Is null value allowed in this column
			'null'           => true,

			// Set true for Primary Key column
			'primary_key'    => false,

			// Set true for AUTO_INCREMENT column, Note: primary_key also true required
			'auto_increment' => false,

			// Set position of column. e.g. ('FIRST', 'BEFORE `id`') etc.
			'position'       => null,

			// Use this option for change column instead of add column
			'change'         => false,
		];
		$args    = \General::set_args( $args, $default );
		extract( $args );

		$length = (string) $args['length'];
		$type   = strtolower( trim( (string) $args['type'] ) );
		if ( $length == '' ) {
			switch ( $type ) {
				case 'integer':
				case 'int':
					$length = '11';
					break;
				case 'varchar':
					$length = '100';
					break;
				case 'char':
					$length = '10';
					break;
				case 'double':
					$length = '10,2';
					break;
				default:
					$length = '';
			}
		}
		switch ( $type ) {
			case 'text':
			case 'date':
			case 'datetime':
			case 'time':
			case 'boolean':
			case 'timestamp':
			case 'tinytext':
			case 'mediumtext':
			case 'longtext':
			case 'blob':
			case 'tinyblob':
			case 'longblob':
				$length = '';
		}

		if ( ! isset( $this->Global['table'] ) ) {
			$change = (bool) $args['change'];
			if ( $change ) {
				if ( empty( $new_col_name ) ) {
					$new_col_name = $args['col_name'];
				}

				$Q = $this->prepare( 'ALTER TABLE %t CHANGE COLUMN %t %t ' . strtoupper( $type ), $args['table_name'], $args['col_name'], $new_col_name );
			} else {
				$Q = $this->prepare( 'ALTER TABLE %t ADD COLUMN %t ' . strtoupper( $type ), $args['table_name'], $args['col_name'] );
			}
		} else {
			$Q = $this->prepare( '%t %w', $args['col_name'], strtoupper( $type ) );
		}


		if ( $length <> '' ) {
			$Q .= $this->prepare( '(%w)', $length );
		}
		$null = (bool) $args['null'];

		//		if ( !$null && ! is_null( $default ) ) {
		//			$Q .= " DEFAULT '{$default}'";
		//		}

		$auto_increment = (bool) $args['auto_increment'];
		$primary_key    = (bool) $args['primary_key'];

		if ( ! $primary_key && ! $auto_increment ) {
			if ( $null ) {
				$Q .= ' NULL';
			} else {
				$Q .= ' NOT NULL';
			}
			if ( is_null( $default ) ) {
				$Q .= ' DEFAULT NULL';
			} else if ( $default !== false ) {
				$Q .= " DEFAULT '{$default}'";
			}
		} else {
			$Q .= ' NOT NULL';
		}

		if ( $primary_key ) {
			$Q .= ' PRIMARY KEY';
		}
		if ( $auto_increment ) {
			$Q .= ' AUTO_INCREMENT';
		}

		## Setting column collation if required.
		switch ( $type ) {
			case 'text':
			case 'varchar':
			case 'char':
			case 'tinytext':
			case 'mediumtext':
			case 'longtext':
			case 'json':
				$Q .= $this->prepare( ' COLLATE %s', self::Collation );
				break;
			default:
				// Do nothing
		}

		if ( empty( $this->Global['table'] ) ) {
			if ( ! is_null( $args['position'] ) ) {
				$args['position'] = (string) $args['position'];
				$Q                .= ' ' . $args['position'];
			}
		}

		if ( isset( $this->Global['table'] ) ) {
			$this->Global['columns'][] = $Q;

			return $this;
		}

		$this->query( $Q );

		return $this->Status;
	}

	function end() {
		if ( empty( $this->Global['table'] ) ) {
			return 'Table name not set via $this->create_table()';
		}
		if ( empty( $this->Global['columns'] ) ) {
			return 'Table columns not added via $this->add_column()';
		}
		$Q = 'CREATE TABLE `' . $this->Global['table'] . '` (' . PHP_EOL;

		foreach ( $this->Global['columns'] as $key => $column ) {
			$comma = ( $key < count( $this->Global['columns'] ) - 1 ) ? ',' : '';
			$Q     .= "\t" . $column . $comma . PHP_EOL;
		}
		$Q .= ')' . PHP_EOL . 'COLLATE=\'' . self::Collation . '\'' . PHP_EOL;
		$Q .= 'ENGINE=' . self::Engine;

		$this->query( $Q );
		$this->Global = [];

		return $this->Status;
	}

	function create_table( $table_name ) {
		$this->Global          = [];
		$this->Global['table'] = $table_name;

		return $this;
	}

	public function db_disconnect() {
		@mysqli_close( $this->dbh );
	}
}