<?php
/*
 * Version: 1.0
 * Last update at 02-08-2018
 *
 * @noinspection SqlResolve, SpellCheckingInspection
 */

namespace DB;

class Select extends \DB {

	protected $select_from_table;
	protected $select_cols = '';
	protected $where = [];
	protected $where_operator = 'AND';
	protected $order_by = [];
	protected $order = [];
	protected $group_by = '';
	protected $join = '';
	protected $query_type = 'select';
	protected $update_data = [];
	protected $return_query = false;

	public $last_query;
	public $Status = '';
	public $insert_id = 0;

	protected $force_operator = null;
	protected $limit_offset = false;
	protected $limit_records = 0;

	public function __construct( $dbuser = '', $dbpassword = '', $dbname = '', $dbhost = '' ) {
		if ( $dbuser == '' ) {
			$dbuser = DBUSER;
		}
		if ( $dbpassword == '' ) {
			$dbpassword = DBPASS;
		}
		if ( $dbname == '' ) {
			$dbname = DBNAME;
		}
		if ( $dbhost == '' ) {
			$dbhost = DBSERVER;
		}
		parent::__construct( $dbuser, $dbpassword, $dbname, $dbhost );
	}

	public function __call( $name, $arguments ) {
		foreach ( $arguments as &$col ) {
			if ( parent::is_mysql_reserved_word( $col ) ) {
				$col = '`' . $col . '`';
			}
		}

		$str = strtoupper( $name ) . '(' . implode( ',', $arguments ) . ')';
		$this->select_cols( $str );

		return $this;
	}

	/**
	 * Clear all options.
	 */
	public function clear() {
		$this->select_cols       = '';
		$this->select_from_table = '';
		$this->where             = [];
		$this->where_operator    = 'AND';
		$this->order             = [];
		$this->order_by          = [];
		$this->group_by          = '';
		$this->join              = '';
		$this->query_type        = 'select';
		$this->update_data       = [];
		$this->return_query      = false;
		$this->limit_offset      = false;
		$this->limit_records     = 0;

	}

	/**
	 * @param $table_name
	 *
	 * Select from table.
	 */
	public function select_from( $table_name ) {
		$this->select_from_table = $table_name;

		return $this;
	}

	public function limit( $offset, $limit = 0 ) {
		$this->limit_offset  = $offset;
		$this->limit_records = $limit;

		return $this;
	}

	/**
	 * @param string $cols
	 *
	 * This method is optional, By default * will be selected.
	 */
	public function select_cols( $cols = '*' ) {
		//$cols = str_replace( '`', '', $cols );

		if ( $this->select_cols <> '' ) {
			$this->select_cols = $cols . ',' . $this->select_cols;
		} else {
			$this->select_cols = $cols;
		}

		$this->select_cols = $cols;

		return $this;
	}

	public function where_in( $column_name, $values ) {
		$values = \Str::make_array( $values );

		$this->where[] = parent::prepare( '%w %t IN ', $this->where_operator, $column_name ) . "('" . implode( "','", $values ) . "')";

		return $this;
	}

	public function where_not_in( $column_name, $values ) {
		$values = \Str::make_array( $values );

		$this->where[] = parent::prepare( '%w %t NOT IN ', $this->where_operator, $column_name ) . "('" . implode( "','", $values ) . "')";

		return $this;
	}

	public function where_between( $column_name, $value1, $value2 ) {
		if ( parent::is_mysql_reserved_word( $column_name ) ) {
			$column_name = '`' . $column_name . '`';
		}

		$this->where[] = parent::prepare( '%w %w BETWEEN %s AND %s', $this->where_operator, $column_name, $value1, $value2 );

		return $this;
	}

	/**
	 * @param $where_condition
	 *
	 * Set where operator.
	 */
	public function where( $where_condition ) {
		if ( ! is_null( $this->force_operator ) ) {
			$op = $this->force_operator;
		} else {
			$op = $this->where_operator;
		}

		$args = func_get_args();

		if ( func_num_args() == 2 && is_string( $args[0] ) ) {
			$newargs[0] = [ $args[0] => $args[1] ];
			$args       = $newargs;
		}

		foreach ( $args as $where_condition ) {
			if ( is_string( $where_condition ) ) {
				$this->where[] = $op . ' ' . $where_condition;
			} else if ( is_array( $where_condition ) ) {
				foreach ( $where_condition as $column => $value ) {
					$column          = trim( (string) $column );
					$column_operator = '=';
					if ( substr( $column, - 2 ) == '!=' ) {
						$column          = substr( $column, 0, - 2 );
						$column_operator = '!=';
					} else if ( substr( $column, - 2 ) == '<>' ) {
						$column          = substr( $column, 0, - 2 );
						$column_operator = '<>';
					} else if ( substr( $column, - 1 ) == '>' ) {
						$column          = substr( $column, 0, - 1 );
						$column_operator = '>';
					} else if ( substr( $column, - 2 ) == '>=' ) {
						$column          = substr( $column, 0, - 2 );
						$column_operator = '>=';
					} else if ( substr( $column, - 2 ) == '<=' ) {
						$column          = substr( $column, 0, - 2 );
						$column_operator = '<=';
					} else if ( substr( $column, - 1 ) == '<' ) {
						$column          = substr( $column, 0, - 1 );
						$column_operator = '<';
					}

					if ( parent::is_mysql_reserved_word( $column ) ) {
						$column = '`' . $column . '`';
					}

					if ( \General::is_really_int( $value ) ) {
						$this->where[] = parent::prepare( '%w ' . $column . ' %w %d', $op, $column_operator, $value );
					} else if ( \General::is_really_float( $value ) ) {
						$this->where[] = parent::prepare( '%w ' . $column . ' %w %f', $op, $column_operator, $value );
					} else if ( is_array( $value ) ) {
						//$value         = json_encode( $value );
						$this->where[] = parent::prepare( '%w ' . $column . ' IN (\'', $op ) . implode( "','", $value ) . "')";
					} else if ( is_object( $value ) ) {
						$value         = json_encode( $value );
						$this->where[] = parent::prepare( '%w ' . $column . ' %w %s', $op, $column_operator, $value );
					} else if ( is_bool( $value ) ) {
						$value         = $value ? 'TRUE' : 'FALSE';
						$this->where[] = parent::prepare( '%w ' . $column . ' %w %w', $op, $column_operator, $value );
					} else if ( is_null( $value ) ) {
						$value         = 'IS NULL';
						$this->where[] = parent::prepare( '%w ' . $column . ' %w', $op, $value );
					} else if ( \Dates::is_datetime( $value ) ) {
						$value         = \Dates::mysql_datetime( $value );
						$this->where[] = parent::prepare( '%w ' . $column . ' %w %s', $op, $column_operator, $value );
					} else if ( \Dates::is_date( $value ) ) {
						$value         = \Dates::mysql_date( $value );
						$this->where[] = parent::prepare( '%w %w %w %s', $op, $column, $column_operator, $value );
					} else {
						if ( substr( $value, 0, 1 ) == '%' && substr( $value, - 1 ) == '%' ) {
							$this->where[] = parent::prepare( '%w ' . $column . ' LIKE %s', $op, '%' . substr( $value, 1, - 1 ) . '%' );
						} else if ( substr( $value, 0, 1 ) == '%' ) {
							$this->where[] = parent::prepare( '%w ' . $column . ' LIKE %s', $op, '%' . substr( $value, 1 ) );
						} else if ( substr( $value, - 1 ) == '%' ) {
							$this->where[] = parent::prepare( '%w ' . $column . ' LIKE %s', $op, substr( $value, 0, - 1 ) . '%' );
						} else {
							$this->where[] = parent::prepare( '%w ' . $column . ' %w %s', $op, $column_operator, $value );
						}
					}

				}
			}
		}

		$this->not            = false;
		$this->force_operator = null;

		return $this;
	}

	/**
	 * @param $where_condition
	 *
	 * Set where condition with AND operator.
	 */
	public function where_and( $where_condition ) {
		$this->force_operator = 'AND';

		call_user_func_array( [ $this, 'where' ], func_get_args() );

		return $this;
	}

	/**
	 * @param $where_condition
	 *
	 * Set where condition with OR operator.
	 */
	public function where_or( $where_condition ) {
		$this->force_operator = 'OR';

		call_user_func_array( [ $this, 'where' ], func_get_args() );

		return $this;
	}

	/**
	 * @param string $operator
	 *
	 * Set default operator for where clause.
	 */
	public function where_operator( $operator = 'AND' ) {
		$this->where_operator = strtoupper( trim( $operator ) );

		return $this;
	}

	/**
	 * @param string $output_type
	 *
	 * @return mixed
	 *
	 * Get single row from MySQL table
	 */
	public function get_row( $table_name = '', $output_type = ARRAY_A, $y = 0 ) {
		# You can select OBJECT_K for object output

		$q = strtoupper( trim( $table_name ) );
		if ( substr( $q, 0, 7 ) == 'SELECT ' ) {
			return parent::get_row( $table_name, $output_type, $y );
		}

		if ( $table_name <> '' ) {
			$this->select_from_table = $table_name;
		}

		$this->query_type = 'select';
		$query            = $this->generate_query();
		if ( $this->return_query ) {
			$this->return_query = false;

			return $this->last_query;
		}

		$row = parent::get_row( $query, $output_type );
		$this->clear();

		return $row;
	}

	/**
	 * @return mixed
	 *
	 * Return single value from MySQL
	 */
	public function get_var( $table_name = '', $x = 0, $y = 0 ) {
		$q = strtoupper( trim( $table_name ) );
		if ( substr( $q, 0, 7 ) == 'SELECT ' ) {

			return parent::get_var( $table_name, $x, $y );
		}

		if ( $table_name <> '' ) {
			$this->select_from_table = '`' . $table_name . '`';
		}

		$this->query_type = 'select';
		$query            = $this->generate_query();
		if ( $this->return_query ) {
			$this->return_query = false;

			return $this->last_query;
		}

		$var = parent::get_var( $query );
		$this->clear();

		return $var;
	}

	/**
	 * @param string $output_type
	 *
	 * @return mixed
	 *
	 * Get databases row according to conditions.
	 */
	public function get_rows( $table_name = '', $output_type = ARRAY_A ) {
		$q = strtoupper( trim( $table_name ) );
		if ( substr( $q, 0, 7 ) == 'SELECT ' || substr( $q, 0, 5 ) == 'SHOW ' ) {
			return parent::get_rows( $table_name, $output_type );
		}

		if ( $table_name <> '' ) {
			if ( parent::is_mysql_reserved_word( $table_name ) ) {
				$this->select_from_table = '`' . $table_name . '`';
			} else {
				$this->select_from_table = $table_name;
			}

		}

		# You can select OBJECT_K for object output
		$this->query_type = 'select';
		$query            = $this->generate_query();
		if ( $this->return_query ) {
			$this->return_query = false;

			return $this->last_query;
		}

		$rows = parent::get_rows( $query, $output_type );
		$this->clear();

		return $rows;
	}

	/**
	 * @param int    $page
	 * @param int    $page_size
	 * @param int    $range
	 * @param string $output_type
	 *
	 * @return mixed
	 *
	 * Get table result with pagination.
	 */
	public function get_rows2( $page = 1, $page_size = null, $range = null, $output_type = null, $extra1 = null, $extra2 = null ) {
		# You can select OBJECT_K for object output

		$q = strtoupper( trim( $page ) );
		if ( substr( $q, 0, 7 ) == 'SELECT ' ) {
			if ( is_null( $page_size ) ) {
				$page_size = 1;
			}
			if ( is_null( $range ) ) {
				$range = 50;
			}
			if ( is_null( $output_type ) ) {
				$output_type = 3;
			}
			if ( is_null( $extra1 ) ) {
				$extra1 = ARRAY_A;
			}
			if ( is_null( $extra2 ) ) {
				$extra2 = true;
			}

			return parent::get_rows2( $page, $page_size, $range, $output_type, $extra1, $extra2 );
		}

		if ( ! \General::is_really_int( $page ) ) {
			if ( is_null( $page_size ) ) {
				$page_size = 1;
			}
			if ( is_null( $range ) ) {
				$range = 50;
			}
			if ( is_null( $output_type ) ) {
				$output_type = 3;
			}
			if ( is_null( $extra1 ) ) {
				$extra1 = ARRAY_A;
			}
			if ( is_null( $extra2 ) ) {
				$extra2 = true;
			}

			return parent::get_rows2( $page, $page_size, $range, $output_type, $extra1, $extra2 );
		}

		$this->query_type = 'select';
		$query            = $this->generate_query();

		if ( $this->return_query ) {
			$this->return_query = false;

			return $this->last_query;
		}

		$rows = parent::get_rows2( $query, $page, $page_size, $range, $output_type );
		$this->clear();

		return $rows;
	}

	/**
	 * Generate query according to where clause etc.
	 */
	public function generate_query() {
		if ( parent::is_mysql_reserved_word( $this->select_from_table ) ) {
			$this->select_from_table = '`' . $this->select_from_table . '`';
		}

		if ( $this->query_type == 'delete' ) {
			$query = 'DELETE FROM ' . $this->select_from_table . ' ';
		} else if ( $this->query_type == 'update' ) {
			$query = 'UPDATE ' . $this->select_from_table . ' SET 1';
			foreach ( $this->update_data as $column_name => $value ) {
				$alias_value = trim( strtoupper( $value ) );
				if ( substr( $column_name, - 1 ) == '-' ) {
					## Increment operator
					$column_name = substr( $column_name, 0, - 1 );
					if ( parent::is_mysql_reserved_word( $column_name ) ) {
						$column_name = '`' . $column_name . '`';
					}
					$value = $column_name . ' - 1';
				} else if ( substr( $column_name, - 1 ) == '+' ) {
					## Increment operator
					$column_name = substr( $column_name, 0, - 1 );
					if ( parent::is_mysql_reserved_word( $column_name ) ) {
						$column_name = '`' . $column_name . '`';
					}
					$value = $column_name . ' + 1';
				} else if ( substr( $column_name, - 1 ) == '~' ) {
					## Use of reverse operator
					$column_name = substr( $column_name, 0, - 1 );
					if ( parent::is_mysql_reserved_word( $column_name ) ) {
						$column_name = '`' . $column_name . '`';
					}
					$value = 'NOT ' . $column_name;
				} else if ( $alias_value == 'NOW()' || $alias_value == 'CURDATE()' || $alias_value == 'DATE()' ) {
					if ( $alias_value == 'DATE()' ) {
						$alias_value = 'CURDATE()';
					}
					$value = $alias_value;
				} else if ( \General::is_really_int( $value ) || \General::is_really_float( $value ) ) {
					// Do nothing
				} else if ( is_array( $value ) || is_object( $value ) ) {
					$value = "'" . json_encode( $value ) . "'";
				} else if ( \Dates::is_datetime( $value ) ) {
					$value = parent::prepare( '%s', \Dates::mysql_datetime( $value ) );
				} else if ( \Dates::is_date( $value ) ) {
					$value = parent::prepare( '%s', \Dates::mysql_date( $value ) );
				} else if ( is_bool( $value ) ) {
					if ( $value ) {
						$value = 'TRUE';
					} else {
						$value = 'FALSE';
					}
				} else {
					if ( substr( $value, 0, 1 ) == '`' && substr( $value, - 1 ) == '`' ) {
						// Do nothing
					} else {
						if ( is_null( $value ) ) {
							$value = null;
						} else {
							$functions   = $this->get_functions_in_query();
							$is_function = false;
							foreach ( $functions as $function ) {
								if ( $function == substr( $alias_value, 0, strlen( $function ) ) ) {
									$is_function = true;
									break;
								}
							}
							if ( $is_function ) {
								// Do nothing
							} else {
								$value = parent::prepare( '%s', $value );
							}
						}
					}
				}

				if ( parent::is_mysql_reserved_word( $column_name ) ) {
					$column_name = '`' . $column_name . '`';
				}

				if ( ! is_null( $value ) ) {
					$query .= ',' . $column_name . ' = ' . $value;
				} else {
					$query .= ',' . $column_name;
				}
			}
			$query = str_replace( ' SET 1,', ' SET ', $query );
		} else {
			if ( $this->select_cols == '' ) {
				$this->select_cols = '*';
			}
			$query = 'SELECT ' . $this->select_cols . ' FROM ' . $this->select_from_table;
			if ( ! empty( $this->join ) ) {
				$query .= ' JOIN ' . $this->join[0] . ' ON ' . $this->join[1];
			}
		}

		$query         .= ' WHERE 1 AND';
		$with_operator = true;

		if ( ! empty( $this->where ) ) {
			foreach ( $this->where as $key => $query_part ) {
				if ( substr( $query_part, 0, 3 ) == 'OR ' ) {
					$op         = 'OR';
					$query_part = substr( $query_part, 3 );
				} else if ( substr( $query_part, 0, 4 ) == 'AND ' ) {
					$op         = 'AND';
					$query_part = substr( $query_part, 4 );
				} else {
					$op = $this->where_operator;
				}

				if ( $key == 0 ) {
					$op = 'AND';
				}

				if ( $query_part == '(' ) {
					$query         .= ' ' . $op . ' (';
					$with_operator = false;
				} else if ( $query_part == ')' ) {
					$query .= ')';
				} else if ( $with_operator ) {
					$query .= ' ' . $op . ' ' . $query_part;
				} else {
					if ( substr( $query_part, 0, 3 ) == 'OR ' ) {
						$query_part = substr( $query_part, 3 );
					} else if ( substr( $query_part, 0, 4 ) == 'AND ' ) {
						$query_part = substr( $query_part, 4 );
					}

					$query         .= $query_part;
					$with_operator = true;
				}
			}
		}

		$this->group_by = trim( (string) $this->group_by );
		if ( $this->group_by <> '' ) {
			$query .= ' GROUP BY ' . $this->group_by;
		}

		if ( ! empty( $this->order_by ) ) {
			foreach ( $this->order_by as $key => $order_by ) {
				if ( $key == 0 ) {
					$query .= ' ORDER BY';
				}
				if ( $key > 0 ) {
					$query .= ',';
				}
				$query .= ' ' . $order_by . ' ' . $this->order[ $key ];
			}
		}

		$query = trim( $query );
		$query = str_replace( 'WHERE 1 AND ', 'WHERE 1 ', $query );
		$query = str_replace( 'WHERE AND ', 'AND ', $query );
		$query = str_replace( 'AND AND', 'AND', $query );
		$query = str_replace( '( AND ', '( ', $query );
		$query = str_replace( '( OR ', '( ', $query );
		if ( substr( $query, - 3 ) == 'AND' ) {
			$query = substr( $query, 0, - 3 );
		}
		if ( substr( $query, - 2 ) == 'OR' ) {
			$query = substr( $query, 0, - 2 );
		}

		if ( $this->limit_offset ) {
			$query .= ' LIMIT ' . $this->limit_offset;
			if ( (int) $this->limit_records > 0 ) {
				$query .= ', ' . $this->limit_records;
			}
		}

		$this->last_query = $query;

		return $query;
	}

	/**
	 * Set start group in query ( in query
	 */
	public function start_group() {
		$this->where[] = '(';

		return $this;
	}

	/**
	 * Set end group in query ) in query.
	 */
	public function end_group() {
		$this->where[] = ')';

		return $this;
	}

	/**
	 * @param        $order_by
	 * @param string $order
	 *
	 * Set order by column and order
	 */
	public function order( $order_by, $order = 'asc' ) {
		if ( parent::is_mysql_reserved_word( $order_by ) ) {
			$order_by = '`' . $order_by . '`';
		}

		$this->order_by[] = $order_by;
		$this->order[]    = strtoupper( $order );

		return $this;
	}

	/**
	 * @param $table_name
	 *
	 * @return mixed
	 * This method will return whole MySQL table.
	 */
	public function get_table( $table_name ) {
		if ( $this->is_mysql_reserved_word( $table_name ) ) {
			$table_name = '`' . $table_name . '`';
		}
		$query            = 'SELECT ' . $this->select_cols . ' FROM ' . $table_name;
		$this->last_query = $query;
		if ( $this->return_query ) {
			$this->return_query = false;

			return $this->last_query;
		}

		$rows = parent::get_rows( $query );
		$this->clear();

		return $rows;
	}

	/**
	 * @return mixed
	 *
	 * Count result after
	 */
	public function count_result( $table_name = '' ) {
		if ( $table_name <> '' ) {
			$this->select_from_table = $table_name;
		}
		$this->select_cols = 'COUNT(*)';
		$this->query_type  = 'select';
		$query             = $this->generate_query();
		if ( $this->return_query ) {
			$this->return_query = false;

			return $this->last_query;
		}

		$this->last_query = $query;
		$count            = parent::get_var( $query );
		$this->clear();

		return $count;
	}

	/**
	 * @param $column
	 *
	 * Set group by column name
	 */
	public function group_by( $column ) {
		$this->group_by = $column;

		return $this;
	}

	/**
	 * @param $table_name
	 * @param $join_condition
	 *
	 * Set join table with condition.
	 */
	public function join( $table_name, $join_condition ) {
		if ( parent::is_mysql_reserved_word( $table_name ) ) {
			$table_name = '`' . $table_name . '`';
		}
		$this->join = [ $table_name, $join_condition ];

		return $this;
	}

	public function update_data( $column_name, $value = null ) {
		## Do not use reserved word check here.!

		if ( is_array( $column_name ) ) {
			foreach ( $column_name as $column_name => $value ) {
				$this->update_data[ $column_name ] = $value;
			}
		} else {
			$this->update_data[ $column_name ] = $value;
		}

		return $this;
	}

	/**
	 * @param $table_name
	 *
	 * @return mixed
	 *
	 * After update this function will return number of updated rows.
	 */
	public function update( $table, $data = '', $where = '', $format = null, $where_format = null, $condition_operator = 'AND' ) {
		if ( is_array( $data ) ) {
			return parent::update( $table, $data, $where, $format, $where_format, $condition_operator );
		}

		$this->query_type        = 'update';
		$this->select_from_table = $table;

		$query            = $this->generate_query();
		$this->last_query = $query;

		if ( $this->return_query ) {
			$this->return_query = false;

			return $this->last_query;
		}

		$this->clear();

		$response = parent::query( $query );
		if ( $response === false ) {
			$this->Status = $this->last_error;
		} else {
			$this->Status = 'OK';
		}

		return $response;
	}

	private function insert_query( $table ) {
		$this->query_type        = 'update';
		$this->select_from_table = $table;

		$query = $this->generate_query();
		$query = str_replace( "UPDATE ", "INSERT ". "INTO ", $query );

		if ( strpos( $query, ' WHERE 1' ) !== false ) {
			$query = substr( $query, 0, strpos( $query, ' WHERE 1' ) );
		}

		$this->last_query = $query;

		$this->return_query = false;

		return $this->last_query;
	}

	/**
	 * @param string $table_name
	 *
	 * @param string $where
	 * @param null   $where_format
	 * @param string $condition
	 *
	 * @return mixed
	 *
	 * After update this function will return number of updated rows.
	 */
	public function delete( $table_name = '', $where = '', $where_format = null, $condition = 'AND' ) {
		if ( is_array( $where ) ) {
			return parent::delete( $table_name, $where, $where_format, $condition );
		}

		if ( $table_name <> '' ) {
			$this->select_from_table = '`' . $table_name . '`';
		}

		$this->query_type        = 'delete';
		$this->select_from_table = $table_name;

		$query = $this->generate_query();

		if ( $this->return_query ) {
			$this->return_query = false;

			return $this->last_query;
		}

		$this->clear();

		$response = parent::query( $query );
		if ( $response === false ) {
			$this->Status = $this->last_error;
		}
		$this->Status = 'OK';

		return $response;
	}

	private function get_functions_in_query() {
		## Put MySQL function name in upper case.
		return [
			'FIND_IN_SET',
			'FIND_IN_SET_X',
			'IFEMPTY',
			'STRIP_TAGS',
			'CONCAT',
			'LEFT',
			'RIGHT',
			'TRIM',
			'LOCATE',
			'MID',
			'CHAR_LENGTH',
			'CHARACTER_LENGTH',
			'UPPER',
			'LOWER',
			'TRIM',
			'RTRIM',
			'LTRIM',
			'REPLACE',
			'MAX',
			'MIN',
			'POSITION',
			'REPEAT',
			'REVERSE',
			'RPAD',
			'SPACE',
			'STRCMP',
			'SUBSTR',
			'SUBSTRING',
			'SUBSTRING_INDEX',
			'UCASE',
			'INSTR',
		];
	}
}