<?php

class EmailTemplates {

	var $Table = "email_templates";
	var $TableAssignment = "email_templates_x_assigned";
	var $DB;
	var $AssignedTo;

	function __construct() {
		global $db;
		$this->DB = $db;

		$this->AssignedTo = [
			"assign_email_to_teacher" => __( "Send email to teacher when student upload assignment" ),
		];
	}

	function get_email_templates( $args = "" ) {
		$default = [
			"page"     => "1",
			"pagesize" => "20",
		];
		$args    = set_args( $args, $default );
		extract( $args );

		$Q = "SELECT * FROM `{$this->Table}` WHERE 1";

		$rows = $this->DB->get_rows2( $Q, $page, $pagesize );

		return $rows;
	}

	function save( $args = "" ) {
		$default = [
			"id"      => 0,
			"name"    => "",
			"subject" => "",
			"message" => "",
		];
		$args    = set_args( $args, $default );

		if ( empty( $args["name"] ) ) {
			return __( "Template name missing." );
		}
		if ( empty( $args["subject"] ) ) {
			return __( "Template email subject missing." );
		}
		if ( empty( $args["message"] ) ) {
			return __( "Template email message missing" );
		}

		if ( empty( $args["id"] ) ) {
			$response = $this->DB->insert( $this->Table, $args );
			if ( $response !== false ) {
				$id = $this->DB->insert_id;
			}
		} else {
			$response = $this->DB->update( $this->Table, $args, [ "id" => $args["id"] ] );
			$id       = $args["id"];
		}

		if ( $response === false ) {
			return $this->DB->last_error;
		}

		return "OK" . $id;
	}

	function save_template_assignments( $args = [] ) {
		$default = [
			"assigned_id" => [],
			"email_id"    => [],
		];
		$args    = set_args( $args, $default );

		if ( count( $args["assigned_id"] ) <> count( $args["email_id"] ) ) {
			return __( "Invalid data type to save." );
		}

		$this->DB->empty_table( $this->TableAssignment );

		for ( $x = 0; $x < count( $args["email_id"] ); $x ++ ) {
			$data     = [
				"email_id"    => $args["email_id"][ $x ],
				"assigned_id" => $args["assigned_id"][ $x ],
			];
			$response = $this->DB->insert( $this->TableAssignment, $data );
			if ( $response === false ) {
				return $this->DB->last_error;
			}
		}

		return "OK";
	}

	public function get_email_template( $args = "" ) {
		if ( ! is_paid_fekara() ) {
			return null;
		}
		$default = [
			"assignment_id" => "",
		];
		$args    = set_args( $args, $default );

		$Q = "SELECT t.* FROM `{$this->Table}` t, `{$this->TableAssignment}` t2 WHERE";
		$Q .= " t2.`assigned_id`='{$args['assignment_id']}' AND t2.`email_id`=t.id";

		$row = $this->DB->get_row( $Q );

		return $row;
	}

	public function get_template_id_from_assignment( $assignmen_id ) {
		$Q = "SELECT `email_id` FROM `{$this->TableAssignment}` WHERE `assigned_id`='$assignmen_id'";

		$email_id = $this->DB->get_var( $Q );
		if ( empty( $email_id ) ) {
			return "0";
		}

		return $email_id;
	}

	public function get_template_fields( $field_key, $data_array = [] ) {
		if ( ! isset( $this->AssignedTo[ $field_key ] ) ) {
			return [];
		}

		$fields = [];
		switch ( $field_key ) {
			case "assign_email_to_teacher":
				## When student will upload assignment.
				//$fields = $this->DB->get_columns( $this->TableAssignment );
				if ( ! empty( $data_array ) ) {
					foreach ( $data_array as $key => $val ) {
						$fields[ "{" . $key . "}" ] = $val;
					}
				} else {
					$fields[] = "student_name";
					$fields[] = "student_roll";
					$fields[] = "teacher_name";
					$fields[] = "uploaded_files_list";
					$fields[] = "teacher_link";
				}
				break;
		}

		return $fields;
	}

	public function get_all_assigned_templates() {
		$output = [];
		$Q      = "SELECT GROUP_CONCAT(`email_id`) FROM `{$this->TableAssignment}`";
		$emails = \Str::make_array( $this->DB->get_var( $Q ) );

		$Q        = "SELECT GROUP_CONCAT(`assigned_id`) FROM `{$this->TableAssignment}`";
		$assigned = \Str::make_array( $this->DB->get_var( $Q ) );

		$output = array_combine( $emails, $assigned );

		return $output;
	}
}