<?php

use Symfony\Component\Filesystem\Filesystem;

class File {

	function __construct() {
		register_shutdown_function( [ &$this, '__destruct' ] );
	}

	function __destruct() {
		return true;
	}

	public static function bytes_format( $size ) {
		$filesizename = [ " Bytes", " KB", " MB", " GB", " TB", " PB", " EB", " ZB", " YB" ];

		return $size ? round( $size / pow( 1024, ( $i = floor( log( $size, 1024 ) ) ) ), 2 ) . $filesizename[ $i ] : '0 Bytes';
	}

	function globistr( $string = '', $mbEncoding = ''/*optional e.g.'UTF-8'*/ ) {
		//returns a case insensitive Version of the searchPattern for glob();
		// e.g.: globistr('./*.jpg') => './*.[jJ][pP][gG]'
		// e.g.: glob(dirname(__FILE__).'/'.globistr('*.jpg')) => '/.../*.[jJ][pP][gG]'

		// known problems: globistr('./[abc].jpg') => FALSE:'./[[aA][bB][cC]].[jJ][pP][gG]'
		//(Problem: existing Pattern[abc] will be overwritten)
		// known solution: './[abc].'.globistr('jpg') => RIGHT: './[abc].[jJ][pP][gG]'
		//(Solution: globistr() only caseSensitive Part, not everything)
		$return = "";
		if ( $mbEncoding !== '' ) { //multiByte Version
			$string = mb_convert_case( $string, MB_CASE_LOWER, $mbEncoding );
		} else { //standard Version (not multiByte,default)
			$string = strtolower( $string );
		}
		$mystrlen = strlen( $string );
		for ( $i = 0; $i < $mystrlen; $i ++ ) {
			if ( $mbEncoding !== '' ) {//multiByte Version
				$myChar = mb_substr( $string, $i, 1, $mbEncoding );
				//$myUpperChar = mb_strtoupper($myChar,$mbEncoding);
				$myUpperChar = mb_convert_case( $myChar, MB_CASE_UPPER, $mbEncoding );
			} else {
				$myChar      = substr( $string, $i, 1 );
				$myUpperChar = strtoupper( $myChar );
			}
			if ( $myUpperChar !== $myChar ) { //there is a lower- and upperChar, / Char is case sentitive
				$return .= '[' . $myChar . $myUpperChar . ']'; //adding both Versions : [xX]
			} else {//only one case Version / Char is case insentitive
				$return .= $myChar; //adding '1','.','*',...
			}
		}

		return $return;
	}

	function extension( $filename, $lower = false ) {
		/*if ( $lower ) {
			return strtolower( end( explode( ".", $filename ) ) );
		} else {
			return end( explode( ".", $filename ) );
		}*/

		return pathinfo( $filename, PATHINFO_EXTENSION );
	}

	public static function folder_list( $startdir ) {
		$ignoredDirectory[] = '.';
		$ignoredDirectory[] = '..';
		$directorylist      = [];
		if ( is_dir( $startdir ) ) {
			if ( $dh = opendir( $startdir ) ) {
				while ( ( $folder = readdir( $dh ) ) !== false ) {
					if ( ! ( array_search( $folder, $ignoredDirectory ) > - 1 ) ) {
						if ( is_dir( $startdir . $folder ) ) {
							$directorylist[ $startdir . $folder ]['name'] = $folder;
							$directorylist[ $startdir . $folder ]['path'] = $startdir;
						}
					}
				}
				closedir( $dh );
			}
		}

		return ( $directorylist );
	}

	public static function files_list( $path ) {
		$path  = str_replace( "\\", "/", $path );
		$files = [];
		//$path = $this->globistr($path);
		foreach ( glob( $path ) as $file ) {
			$files[] = $file;
		}

		return $files;
	}

	public static function get_temp_path() {
		if ( ! empty( $_ENV['TMP'] ) ) {
			return realpath( $_ENV['TMP'] );
		}
		if ( ! empty( $_ENV['TMPDIR'] ) ) {
			return realpath( $_ENV['TMPDIR'] );
		}
		if ( ! empty( $_ENV['TEMP'] ) ) {
			return realpath( $_ENV['TEMP'] );
		}
		$tempfile = tempnam( __FILE__, '' );
		if ( file_exists( $tempfile ) ) {
			unlink( $tempfile );

			return realpath( dirname( $tempfile ) );
		}

		return null;
	}

	public static function get_extension( $filename, $lower = false ) {
		return pathinfo( $filename, PATHINFO_EXTENSION );

		/*$filename = explode( "?", $filename );
		$filename = $filename[0];

		if ( $lower ) {
			return strtolower( end( explode( ".", $filename ) ) );
		} else {
			return end( explode( ".", $filename ) );
		}*/
	}

	// Remove folder and all files/folders in it.
	public static function rrmdir( $dir_path ) {
		if ( ! is_dir( $dir_path ) ) {
			return false;
		}
		$dir_path = rtrim( $dir_path, DIRECTORY_SEPARATOR );
		$dir_path = rtrim( $dir_path, "/" );

		$objects = scandir( $dir_path );
		foreach ( $objects as $object ) {
			if ( $object == '.' || $object == '..' ) {
				continue;
			}

			$path = $dir_path . DIRECTORY_SEPARATOR . $object;
			if ( is_dir( $path ) ) {
				self::rrmdir( $path );
			} else {
				@unlink( $path );
			}
		}

		if ( is_dir( $dir_path ) ) {
			return @rmdir( $dir_path );
		} else {
			return true;
		}
	}

	public static function is_image( $filename ) {
		$ext = self::get_extension( $filename );

		return ( $ext == "jpg" || $ext == "jpeg" || $ext == "png" || $ext == "gif" );
	}

	public static function server_file( $fileName, $fileTitle = null, $contentType = 'application/octet-stream' ) {
		if ( ! file_exists( $fileName ) ) {
			throw New \Exception( sprintf( 'File not found: %s', $fileName ) );
		}

		if ( ! is_readable( $fileName ) ) {
			throw New \Exception( sprintf( 'File not readable: %s', $fileName ) );
		}


		### Remove headers that might unnecessarily clutter up the output
		header_remove( 'Cache-Control' );
		header_remove( 'Pragma' );


		### Default to send entire file
		$byteOffset = 0;
		$byteLength = $fileSize = filesize( $fileName );

		header( 'Accept-Ranges: bytes', true );
		header( sprintf( 'Content-Type: %s', $contentType ), true );

		if ( $fileTitle ) {
			header( sprintf( 'Content-Disposition: attachment; filename="%s"', $fileTitle ) );
		}

		### Parse Content-Range header for byte offsets, looks like "bytes=11525-" OR "bytes=11525-12451"
		if ( isset( $_SERVER['HTTP_RANGE'] ) && preg_match( '%bytes=(\d+)-(\d+)?%i', $_SERVER['HTTP_RANGE'], $match ) ) {
			### Offset signifies where we should begin to read the file
			$byteOffset = (int) $match[1];

			### Length is for how long we should read the file according to the browser, and can never go beyond the file size
			if ( isset( $match[2] ) ) {
				$byteLength = min( (int) $match[2], $byteLength - $byteOffset );
			}

			header( "HTTP/1.1 206 Partial content" );
			header( sprintf( 'Content-Range: bytes %d-%d/%d', $byteOffset, $byteLength - 1, $fileSize ) );  ### Decrease by 1 on byte-length since this definition is zero-based index of bytes being sent
		}

		$byteRange = $byteLength - $byteOffset;

		header( sprintf( 'Content-Length: %d', $byteRange ) );
		header( sprintf( 'Expires: %s', date( 'D, d M Y H:i:s', time() + 60 * 60 * 24 * 90 ) . ' GMT' ) );


		$buffer     = '';    ### Variable containing the buffer
		$bufferSize = 512 * 16; ### Just a reasonable buffer size
		$bytePool   = $byteRange; ### Contains how much is left to read of the byteRange

		if ( ! $handle = fopen( $fileName, 'r' ) ) {
			throw New \Exception( sprintf( "Could not get handle for file %s", $fileName ) );
		}

		if ( fseek( $handle, $byteOffset, SEEK_SET ) == - 1 ) {
			throw New \Exception( sprintf( "Could not seek to byte offset %d", $byteOffset ) );
		}


		while ( $bytePool > 0 ) {
			$chunkSizeRequested = min( $bufferSize, $bytePool ); ### How many bytes we request on this iteration

			### Try readin $chunkSizeRequested bytes from $handle and put data in $buffer
			$buffer = fread( $handle, $chunkSizeRequested );

			### Store how many bytes were actually read
			$chunkSizeActual = strlen( $buffer );

			### If we didn't get any bytes that means something unexpected has happened since $bytePool should be zero already
			if ( $chunkSizeActual == 0 ) {
				### For production servers this should go in your php error log, since it will break the output
				trigger_error( 'Chunksize became 0', E_USER_WARNING );
				break;
			}

			### Decrease byte pool with amount of bytes that were read during this iteration
			$bytePool -= $chunkSizeActual;

			### Write the buffer to output
			print $buffer;

			### Try to output the data to the client immediately
			flush();
		}

		exit();
	}

	public static function is_image_file( $filepath ) {
		if ( ! file_exists( $filepath ) ) {
			return false;
		}
		$image_info = @getimagesize( $filepath );
		if ( ! is_array( $image_info ) ) {
			return false;
		}
		if ( empty( $image_info ) ) {
			return false;
		}

		return true;
	}

	public static function get_file_name( $url ) {
		$url = basename( $url );

		if ( strpos( $url, "?" ) !== false ) {
			$url = explode( "?", $url );
			$url = $url[0];
		}
		if ( strpos( $url, "#" ) !== false ) {
			$url = explode( "#", $url );
			$url = $url[0];
		}

		return $url;
	}

	public static function is_emtpy_folder( $folder ) {
		if ( is_dir( $folder ) ) {
			$handle = opendir( $folder );
			while ( ( gettype( $name = readdir( $handle ) ) != 'boolean' ) ) {
				if ( $name != '.htaccess' ) {
					$name_array[] = $name;
				}
			}
			$folder_content = "";
			foreach ( $name_array as $temp ) {
				$folder_content .= $temp;
			}

			if ( $folder_content == '...' ) {
				return true;
			} else {
				return false;
			}
			closedir( $handle );
		} else {
			return true;
		}
	}
}