<?php

class General {

	/*	function __construct() {
			register_shutdown_function( [ &$this, '__destruct' ] );
		}

		function __destruct() {
			return true;
		}*/

	public static function strtodb( $str ) {
		return str_replace( "'", "''", $str );
	}

	public static function strtodbc( $str ) {
		return str_replace( "'", "''", trim( $str ) );
	}

	public static function gd_info() {
		if ( extension_loaded( 'gd' ) and imagetypes() & IMG_PNG and imagetypes() & IMG_GIF and imagetypes() & IMG_JPG and imagetypes() & IMG_WBMP ) {

			return true;
		} else {
			return false;
		}
	}

	public static function upload_limit() {
		$max_upload   = (int) ( ini_get( 'upload_max_filesize' ) );
		$max_post     = (int) ( ini_get( 'post_max_size' ) );
		$memory_limit = (int) ( ini_get( 'memory_limit' ) );
		$upload_mb    = min( $max_upload, $max_post, $memory_limit );

		return $upload_mb;
	}

	public static function xtrim( $string ) {
		$string = trim( $string, " \t\n\r\0\x0Bÿ" );

		return $string;
	}

	public static function set_args( $args, $default ) {
		if ( ! is_array( $args ) ) {
			$args = stripslashes( $args );
			parse_str( $args, $args );
		}
		if ( ! is_array( $args ) ) {
			return [];
		}
		if ( ! is_array( $default ) ) {
			return $default;
		}
		foreach ( $default as $k => $v ) {
			if ( ! isset( $args[ $k ] ) ) {
				$args[ $k ] = $v;
			}
		}
		if ( get_magic_quotes_gpc() ) {
			$args = array_map_recursive( "stripslashes", $args );
		}

		return $args;
	}

	public static function myip() {
		if ( getenv( 'HTTP_CLIENT_IP' ) ) {
			$ip = getenv( 'HTTP_CLIENT_IP' );
		} elseif ( getenv( 'HTTP_X_FORWARDED_FOR' ) ) {
			$ip = getenv( 'HTTP_X_FORWARDED_FOR' );
		} elseif ( getenv( 'HTTP_X_FORWARDED' ) ) {
			$ip = getenv( 'HTTP_X_FORWARDED' );
		} elseif ( getenv( 'HTTP_FORWARDED_FOR' ) ) {
			$ip = getenv( 'HTTP_FORWARDED_FOR' );
		} elseif ( getenv( 'HTTP_FORWARDED' ) ) {
			$ip = getenv( 'HTTP_FORWARDED' );
		} else {
			$ip = $_SERVER['REMOTE_ADDR'];
		}

		return $ip;
	}

	public static function is_offline() {
		return is_file( ROOT_PATH . ".offline" );
	}

	public static function localhost() {
		//return ($_SERVER["HTTP_HOST"]=="localhost" || $_SERVER["HTTP_HOST"]=="127.0.0.1" || $_SERVER["HTTP_HOST"]=="school2.localhost");

		if ( isset( $_SERVER["SERVER_ADDR"] ) && ( $_SERVER["SERVER_ADDR"] == "127.0.0.1" || $_SERVER["SERVER_ADDR"] == "::1" ) ) {
			return true;
		}

		return ( $_SERVER["REMOTE_ADDR"] == "::1" || $_SERVER["REMOTE_ADDR"] == "127.0.0.1" || $_SERVER["HTTP_HOST"] == "localhost" || $_SERVER["HTTP_HOST"] == "127.0.0.1" || $_SERVER["HTTP_HOST"] == "fekara.pk" );
	}

	public static function debug( $message, $append = true ) {
		/*if (is_array($message)) $message=json_encode($message);
		elseif (is_object($message)) $message = print_r($message,true);*/

		if ( is_null( $message ) ) {
			$message = "NULL";
		} else if ( is_bool( $message ) && $message ) {
			$message = "TRUE";
		} else if ( is_bool( $message ) && ! $message ) {
			$message = "FALSE";
		}

		$message = print_r( $message, true );
		if ( self::localhost() ) {
			if ( $append ) {
				file_put_contents( ROOT_PATH . "logs.txt", $message . "\n", FILE_APPEND );
			} else {
				file_put_contents( ROOT_PATH . "logs.txt", $message );
			}
		} else {
			$bt   = debug_backtrace();
			$file = $bt[0]["file"];
			$line = $bt[0]["line"];

			$message .= "<br><br>debug function in file<br><b>" . $file;
			$message .= "GET:<pre>" . print_r( $_GET, true ) . "</pre>";
			$message .= "POST:<pre>" . print_r( $_POST, true ) . "</pre>";
			$message .= "</b><br>Line # <b>" . $line . "</b>\nTime: " . \Dates::my_datetime();

			return send_mail( "shakeel@fekara.com", "Fekara Client Debug", $message );
		}

		return false;
	}

	/**
	 * @author Shakeel Ahmed Siddiqi <shakeel@shakeel.pk>
	 * @return boolean
	 *
	 * Return if server is running on PHP7 or not.
	 */
	public static function is_php7() {
		return version_compare( PHP_VERSION, '7.0', ">=" );
	}

	/**
	 * @author Shakeel Ahmed Siddiqi <shakeel@shakeel.pk>
	 * @return bool
	 *
	 * Returns true if https is used in URL.
	 */
	public static function is_ssl() {
		return ( ! empty( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] != 'off' );
	}

	/**
	 * @return bool
	 *             Check if GD library is installed or not.
	 */
	public static function get_gd_info() {
		if ( extension_loaded( 'gd' ) and imagetypes() & IMG_PNG and imagetypes() & IMG_GIF and imagetypes() & IMG_JPG and imagetypes() & IMG_WBMP ) {

			return true;
		} else {
			return false;
		}
	}

	/**
	 * @return mixed
	 *              Get upload file limit by Server.
	 */
	public static function get_upload_limit() {
		$max_upload   = (int) ( ini_get( 'upload_max_filesize' ) );
		$max_post     = (int) ( ini_get( 'post_max_size' ) );
		$memory_limit = (int) ( ini_get( 'memory_limit' ) );
		$upload_mb    = min( $max_upload, $max_post, $memory_limit );

		return $upload_mb;
	}

	public static function url_to_name( $url ) {
		$old = array( '_', '-' );
		$new = array( " ", ' ' );
		$url = str_replace( $old, $new, $url );
		$url = str_replace( '   ', ' ', $url );
		$url = str_replace( '  ', ' ', $url );

		$url = ucwords( strtolower( $url ) );

		return $url;
	}

	public static function my_flush() {
		echo( str_repeat( ' ', 256 ) );
		// check that buffer is actually set before flushing
		if ( ob_get_length() ) {
			@ob_flush();
			@flush();
			@ob_end_flush();
		}
		@ob_start();
		echo '<script language="javascript">window.scrollBy(0,1000);</script>';
	}

	/**
	 * @param $val
	 *
	 * @return bool
	 *             Check if given value is real integer or not.
	 */
	public static function is_really_int( $val ) {
		return filter_var( $val, FILTER_VALIDATE_INT ) !== false;
	}

	public static function is_really_float( $val ) {
		return filter_var( $val, FILTER_VALIDATE_FLOAT ) !== false;
	}

}