<?php

class Guardian {

	var $Table = "guardians";
	var $TableXStudents = "guardians_x_students";
	var $DB;

	function __construct() {
		global $db;
		$this->DB = $db;
		$this->convert_phone_formats();
	}

	function convert_phone_formats() {
		$Q    = "SELECT `id`,`phone` FROM `{$this->Table}` WHERE (`phone` IS NOT NULL OR TRIM(`phone`)<>'') AND `validate_phone`<>'1'";
		$rows = $this->DB->get_rows( $Q );
		foreach ( $rows as $row ) {
			$this->DB->update( $this->Table, [
				"phone"          => formatize_phone_number( $row["phone"] ),
				"validate_phone" => "1",
			], [ "id" => $row["id"] ] );
		}
	}

	function get_guardian_panel_url() {
		return DOMAIN_URL . "guardians/";
	}

	function get_guardian_panel_path() {
		return ROOT_PATH . "guardians/";
	}

	function is_guardian_logged_in() {
		if ( ! isset( $_SESSION[ $this->get_session_var() ] ) && isset( $_COOKIE[ $this->get_session_var() ] ) ) {
			$id  = $_COOKIE[ $this->get_session_var() ];
			$row = $this->DB->get_row( "SELECT * FROM `$this->Table` WHERE `id`='$id' AND `active`='1'" );
			if ( isset( $row['id'] ) ) {
				$_SESSION[ $this->get_session_var() ] = $row;
				set_cookie( $this->get_session_var(), $row['id'] );
			}
		}

		return isset( $_SESSION[ $this->get_session_var() ] );
	}

	/**
	 * @author Shakeel Ahmed Siddiqi <shakeel@shakeel.pk>
	 *
	 * @param string $params
	 *
	 * @return mixed|string|array
	 *
	 * This function logins Guardian's panel.
	 */
	function login( $params = "" ) {
		$default = [
			"email"       => "",
			"phone"       => "",
			"password"    => "",
			"remember_me" => "0",
		];
		$params  = set_args( $params, $default );
		extract( $params );

		if ( empty( $email ) && empty( $phone ) ) {
			return __( "Please provide email and/or phone as username" );
		}

		if ( empty( $password ) ) {
			return __( "Password missing." );
		}

		if ( \Str::is_email( $phone ) && ! \Str::is_email( $email ) ) {
			$email = $phone;
			$phone = '';
		}

		$Q = $this->DB->prepare( "SELECT * FROM `{$this->Table}` WHERE (`email`=%s OR `phone`=%s) AND `active`='1'", $email, formatize_phone_number( $phone ) );

		$row = $this->DB->get_row( $Q );
		if ( ! isset( $row['id'] ) ) {
			return __( "Guardian data not found" );
		}

		if ( $row['password'] == '' ) {
			if ( empty( $row['email'] ) ) {
				$new_password = password_hash( $row['phone'], PASSWORD_BCRYPT );
			} else {
				$new_password = password_hash( $row['email'], PASSWORD_BCRYPT );
			}

			$r = $this->DB->update( $this->Table, [ "password" => $new_password ], [ "id" => $row['id'] ] );
			if ( $r !== false ) {
				$row['password'] = $new_password;
			}
		}

		if ( ! password_verify( $password, $row['password'] ) ) {
			return __( "Invalid password." );
		}

		# Updating login_history
		$history = [ "login_time" => \Dates::my_datetime(), "login_ip" => \General::myip() ];
		$r       = $this->DB->update( $this->Table, [ "login_history" => json_encode( $history ) ], [ "id" => $row["id"] ] );

		$_SESSION[ $this->get_session_var() ] = $row;
		if ( $remember_me == '1' ) {
			set_cookie( $this->get_session_var(), $row['id'] );
		}

		return "OK";
	}

	function logout() {
		unset( $_SESSION[ $this->get_session_var() ] );
		if ( isset( $_COOKIE[ $this->get_session_var() ] ) ) {
			set_cookie( $this->get_session_var(), "", strtotime( "-1 day" ) );
		}
	}

	function get_guardians( $params = "" ) {
		$default = [
			"order"    => "DESC",
			"orderby"  => "id",
			"active"   => "",       # -1 for all, 1 or 0
			"page"     => "1",
			"pagesize" => "50",
			"search"   => "",
			"id"       => "",
		];
		$params  = set_args( $params, $default );
		extract( $params );

		$Q = "SELECT * FROM `$this->Table` WHERE 1";

		if ( ! empty( $id ) ) {
			$id = \Str::make_array( $id );
			$Q  .= " AND `id` IN ('" . implode( "','", $id ) . "')";
		}

		if ( $active <> "" ) {
			$Q .= " AND `active`='$active'";
		}
		if ( ! empty( $search ) ) {
			$search = \Str::make_array( $search );
		}

		$Q    .= " ORDER BY `$orderby` $order";
		$rows = $this->DB->get_rows2( $Q, $page, $pagesize );

		return $rows;
	}


	function get_guardians_data() {
		$Q    = "SELECT * FROM `$this->Table` WHERE 1";
		$rows = $this->DB->get_rows( $Q );

		foreach ( $rows as &$row ) {
			$row["student_ids"] = $this->get_attached_student_ids( "guardian_id=" . $row["id"] );
		}

		return $rows;
	}


	function save( $params = "" ) {
		$default = [
			"id"          => "0",
			"email"       => "",
			"phone"       => "",
			"password1"   => "",
			"password2"   => "",
			"active"      => "1",
			"student_ids" => "",
		];
		$params  = set_args( $params, $default );
		extract( $params );

		if ( empty( $phone ) ) {
			return __( "Please provide valid phone number." );
		}

		if ( ! empty( $email ) && ! \Str::is_email( $email ) ) {
			return __( "Please provide valid email address." );
		}

		if ( ! empty( $id ) && empty( $email ) ) {
			unset( $params["email"] );
		}

		$student_ids = \Str::make_array( $params["student_ids"] );
		unset( $params["student_ids"] );

		$Q = $this->DB->prepare( "SELECT COUNT(*) FROM `$this->Table` WHERE `id`<>%d AND `email`=%s", $id, $email );
		global $db;
		if ( $db->get_var( $Q ) > 0 ) {
			return __( "Email address '%1' already registered with someone else.", $email );
		}

		$Q = $this->DB->prepare( "SELECT COUNT(*) FROM `$this->Table` WHERE `phone`=%s AND `id`<>%d", $phone, $id );
		if ( $this->DB->get_var( $Q ) > 0 ) {
			return __( "Phone number '%1' already registered.", $phone );
		}

		if ( $password1 <> $password2 ) {
			return __( "Second password do not match with first password." );
		}
		if ( empty( $id ) ) {
			if ( empty( $password1 ) ) {
				return __( "Please provide password." );
			}
			$params['password'] = password_hash( $password1, PASSWORD_BCRYPT );
			unset( $params['password1'], $params['password2'], $params['id'] );
			$r = $this->DB->insert( $this->Table, $params );
			if ( $r !== false ) {
				$id = $this->DB->insert_id;
			}
		} else {
			if ( ! empty( $password1 ) ) {
				$params['password'] = password_hash( $params['password1'], PASSWORD_BCRYPT );
			}
			unset( $params['password1'], $params['password2'], $params['id'] );
			$r = $this->DB->update( $this->Table, $params, [ "id" => $id ] );
		}

		if ( $r === false ) {
			$this->DB->last_error;
		}

		$this->DB->delete( $this->TableXStudents, [ "guardian_id" => $id ] );
		foreach ( $student_ids as $student_id ) {
			$data = [
				"student_id"  => $student_id,
				"guardian_id" => $id,
			];
			$this->DB->insert( $this->TableXStudents, $data );
		}

		return "OK";
	}

	function my_id() {
		if ( ! $this->is_guardian_logged_in() ) {
			return "0";
		}
		if ( ! isset( $_SESSION[ $this->get_session_var() ]["id"] ) ) {
			return "0";
		}

		return $_SESSION[ $this->get_session_var() ]["id"];
	}

	function get_session_var() {
		return \Admins\UserInfo::get_system_var() . "_guardian";
	}

	public function get_guardian( $params = "" ) {
		$default = [
			"field" => "*",
			"id"    => "0",
		];
		$params  = set_args( $params, $default );
		extract( $params );

		if ( empty( $id ) && $this->is_guardian_logged_in() ) {
			if ( isset( $_SESSION[ $this->get_session_var() ]["id"] ) ) {
				$id = $_SESSION[ $this->get_session_var() ]["id"];
			} else {
				$id = 0;
			}
		}

		$Q   = "SELECT * FROM `{$this->Table}` WHERE `id`='$id'";
		$row = $this->DB->get_row( $Q );

		if ( $fields == "*" ) {
			return $row;
		} else {
			$fields = \Str::make_array( $fields );
			$out    = [];
			foreach ( $fields as $field ) {
				if ( isset( $row[ $field ] ) ) {
					$out[ $field ] = $row[ $field ];
				}
			}

			return $out;
		}
	}


	public function get_guardiandata( $id ) {
		global $db;
		$Q   = "SELECT `studentids` FROM `{$this->Table}` WHERE `id`='$id'";
		$row = $db->get_row( $Q );
		$ids = "";
		if ( ! empty( $row ) ) {
			$arr = explode( ",", $row["studentids"] );
			foreach ( $arr as $a ) {
				$ids .= "'" . $a . "',";
			}
			if ( $ids != "" ) {
				$ids  = trim( $ids, "," );
				$v    = "id,name,class_id";
				$Q    = "SELECT $v FROM `students` WHERE `id` IN($ids)";
				$rows = $db->get_rows( $Q );
			} else {
				$rows = "NULL";
			}

			return $rows;
		}

		return $row;
	}

	/**
	 * @param $id
	 * @param $new_password
	 *
	 * This function will be used by Administrator to change password and do not required current password.
	 */
	function change_password_by_admin( $id, $new_password ) {
		if ( empty( $new_password ) ) {
			return __( "Please provide new password" );
		}

		$new_password = password_hash( $new_password, PASSWORD_BCRYPT );
		$response     = $this->DB->update( $this->Table, [ "password" => $new_password ], [ "id" => $id ] );
		if ( $response === false ) {
			return $this->DB->last_error;
		}

		return "OK";
	}


	function change_password( $params = "" ) {
		$default = [
			"current_password" => "",
			"password1"        => "",
			"password2"        => "",
		];
		$params  = set_args( $params, $default );
		extract( $params );

		if ( empty( $current_password ) || empty( $password1 ) || empty( $password2 ) ) {
			return "Please provide all passwords.";
		}
		if ( $password1 <> $password2 ) {
			return "Password verification is not correct.";
		}
		if ( ! is_password_strong( $password1 ) ) {
			return "Password is weak, Required small letters, capital letters and digits with at least 6 characters.";
		}

		if ( ! $this->is_guardian_logged_in() ) {
			return "You'r not logged in";
		}
		$id = $this->my_id();

		$Q = "SELECT `password` FROM `$this->Table` WHERE `id`='$id'";
		global $db;
		$password = $db->get_var( $Q );

		if ( ! password_verify( $current_password, $password ) ) {
			return "You've provided invalid current password.";
		}
		$new_password = password_hash( $password1, PASSWORD_BCRYPT );
		$r            = $db->update( $this->Table, [ "password" => $new_password ], [ "id" => $id ] );

		if ( $r === false ) {
			return $db->last_error;
		} else {
			return "OK";
		}
	}

	function editParentData( $data = "" ) {
		global $db;
		extract( $data );
		if ( ! \Str::is_email( $email ) ) {
			return "Please provide valid email address.";
		} else {
			$Q    = "SELECT * FROM `{$this->Table}` WHERE `email`='$email' AND `id`!=$id";
			$rows = $db->get_rows( $Q );
			if ( count( $rows ) > 0 ) {
				return ( "This Email already exits (" . $email . ")" );
			}
		}
		if ( ! empty( $studentids ) ) {
			$implode            = implode( ',', $data['studentids'] );
			$data['studentids'] = $implode;
		}
		$response = $db->update( $this->Table, $data, [ "id" => $id ] );

		if ( $response === false ) {
			return $db->last_error;
		} else {
			return "OK";
		}
	}

	function get_guardian_data( $data = "" ) {
		global $db;
		extract( $data );
		$v    = "id,email,studentids";
		$Q    = "SELECT $v FROM `{$this->Table}` WHERE `id`='$id' ";
		$rows = $db->get_rows( $Q );
		if ( ! empty( $rows ) ) {
			$rows["status"] = "OK";
		} else {
			$rows["status"] = "NULL";
		}

		return $rows;
	}

	function count_attached_students( $guardian_id ) {
		$Q = "SELECT COUNT(*) FROM `{$this->TableXStudents}` WHERE `guardian_id`='$guardian_id'";

		return $this->DB->get_var( $Q );
	}

	function get_attached_students( $args = "" ) {
		$default = [
			"guardian_id" => "",
			"page"        => "1",
			"pagesize"    => "50",
			"mysql_cols"  => "s.*",
		];
		$args    = set_args( $args, $default );
		extract( $args );

		$Q = "SELECT {$mysql_cols} FROM `students_view` s, `{$this->TableXStudents}` g WHERE s.`id`=g.`student_id`";
		if ( ! empty( $guardian_id ) ) {
			$guardian_id = \Str::make_array( $guardian_id );
			$Q           .= " AND g.`guardian_id` IN ('" . implode( "','", $guardian_id ) . "')";
		}

		return $this->DB->get_rows2( $Q );
	}

	function get_attached_student_ids( $args = "" ) {
		$default = [
			"guardian_id" => "",
		];
		$args    = set_args( $args, $default );
		extract( $args );
		$Q = "SELECT `student_id` FROM `{$this->TableXStudents}` WHERE `guardian_id`='$guardian_id'";

		$rows = $this->DB->get_rows( $Q );
		$ids  = [];
		foreach ( $rows as $row ) {
			$ids[] = $row["student_id"];
		}

		return $ids;
	}
}