<?php
/**
 * SimpleImage
 *
 * @description   Class to manipulate with images
 * @version       1.0.0
 * @copyright (c) 2013 Juraj Puchký - Devtech
 * @license       GPLv3
 * @autor         Juraj Puchký - Devtech <sjurajpuchky@seznam.cz>
 *                http://www.phpclasses.org/browse/file/48884.html
 *                Sample Codes Below
 *                include_once 'SimpleImage.php';
 *                $image = $this->load('http://files.phpclasses.org/graphics/phpclasses/elephpant.png');
 *                $resizedImage = $this->resizeToHeight ( $image, 30 );
 *                $this->save ( $resizedImage, "resized.png" );
 *
 * unset($image);
 * unset($resizedImage);
 */

#defined ( '_JEXEC' ) or die ( 'Restricted access' );
class Image {

	public static function load( $filename ) {
		$image_info = getimagesize( $filename );
		$image_type = $image_info [2];
		$image      = null;

		try {
			switch ( $image_type ) {
				case IMAGETYPE_JPEG :
					if ( !function_exists("imagecreatefromjpeg") ) {
						echo "PHP function 'imagecreatefromjpeg' not found on server. Please install this function on server.";
						exit;
					}
					$image = imagecreatefromjpeg( $filename );
					break;
				case IMAGETYPE_GIF :
					if ( !function_exists("imagecreatefromgif") ) {
						echo "PHP function 'imagecreatefromgif' not found on server. Please install this function on server.";
						exit;
					}
					$image = imagecreatefromgif( $filename );
					break;
				case IMAGETYPE_PNG :
					if ( !function_exists("imagecreatefrompng") ) {
						echo "PHP function 'imagecreatefrompng' not found on server. Please install this function on server.";
						exit;
					}
					$image = imagecreatefrompng( $filename );
					break;
				default :
					return false;
			}
		} catch ( Exception $e ) {
			error_log( $e->getMessage() );

			return false;
		}

		return $image;
	}

	public static function save( $image, $filename, $image_type = IMAGETYPE_JPEG, $compression = 75, $permissions = null ) {
		try {
			switch ( $image_type ) {
				case IMAGETYPE_JPEG :
					imagejpeg( $image, $filename, $compression );
					break;
				case IMAGETYPE_GIF :
					imagegif( $image, $filename );
					break;
				case IMAGETYPE_PNG :
					imagepng( $image, $filename );
					break;
				default :
					return false;
			}
			if ( $permissions !== null ) {
				chmod( $filename, $permissions );
			}
		} catch ( Exception $e ) {
			error_log( $e->getMessage() );

			return false;
		}

		return true;
	}

	public static function getWidth( $image ) {
		return imagesx( $image );
	}

	public static function getHeight( $image ) {
		return imagesy( $image );
	}

	public static function resizeToHeight( $image, $height ) {
		$ratio = $height / self::getHeight( $image );
		$width = self::getWidth( $image ) * $ratio;

		return self::resize( $image, $width, $height );
	}

	public static function resizeToWidth( $image, $width ) {
		$ratio  = $width / self::getWidth( $image );
		$height = self::getheight( $image ) * $ratio;

		return self::resize( $image, $width, $height );
	}

	public static function scale( $image, $scale ) {
		$width  = self::getWidth( $image ) * $scale / 100;
		$height = self::getheight( $image ) * $scale / 100;

		return self::resize( $image, $width, $height );
	}

	public static function resize( $image, $width, $height ) {
		$new_image = imagecreatetruecolor( $width, $height );
		imagecopyresampled( $new_image, $image, 0, 0, 0, 0, $width, $height, self::getWidth( $image ), self::getHeight( $image ) );

		return $new_image;
	}

	public function make_square( $image, $widthheight ) {
		$percent    = 1;
		$width      = self::getWidth( $image );
		$height     = self::getHeight( $image );
		$new_width  = $width * $percent;
		$new_height = $height * $percent;

		$imagethumbsize_w = $widthheight;
		$imagethumbsize_h = $widthheight;
		$width            = $imagethumbsize_w;
		$height           = $imagethumbsize_h;

		list( $width_orig, $height_orig ) = [ self::getWidth( $image ), self::getHeight( $image ) ];
		if ( $width_orig < $height_orig ) {
			$height = ( $imagethumbsize_w / $width_orig ) * $height_orig;
		} else {
			$width = ( $imagethumbsize_h / $height_orig ) * $width_orig;
		}
		if ( $width < $imagethumbsize_w ) {
			$width  = $imagethumbsize_w;
			$height = ( $imagethumbsize_w / $width_orig ) * $height_orig;;
		}
		if ( $height < $imagethumbsize_h ) {
			$height = $imagethumbsize_h;
			$width  = ( $imagethumbsize_h / $height_orig ) * $width_orig;
		}
		$thumb = imagecreatetruecolor( $width, $height );
		imagecopyresampled( $thumb, $image, 0, 0, 0, 0, $width, $height, $width_orig, $height_orig );
		$thumb2 = imagecreatetruecolor( $imagethumbsize_w, $imagethumbsize_h );
		$w1     = ( $width / 2 ) - ( $imagethumbsize_w / 2 );
		$h1     = ( $height / 2 ) - ( $imagethumbsize_h / 2 );
		imagecopyresampled( $thumb2, $thumb, 0, 0, $w1, $h1, $imagethumbsize_w, $imagethumbsize_h, $imagethumbsize_w, $imagethumbsize_h );

		return $thumb2;
	}

	public function resize_image_smart( $orig, $dest_width = null, $dest_height = null ) {
		$orig_width        = imagesx( $orig );
		$orig_height       = imagesy( $orig );
		$vertical_offset   = 0;
		$horizontal_offset = 0;
		if ( $dest_width == null ) {
			if ( $dest_height == null ) {
				die( '$dest_width and $dest_height cant both be null!' );
			}
			// height is locked
			$dest_width = $dest_height * $orig_width / $orig_height;
		} else {
			if ( $dest_height == null ) {
				// width is locked
				$dest_height = $dest_width * $orig_height / $orig_width;
			} else {
				// both dimensions are locked
				$vertical_offset   = $dest_height - ( $orig_height * $dest_width ) / $orig_width;
				$horizontal_offset = $dest_width - ( $dest_height * $orig_width ) / $orig_height;
				if ( $vertical_offset < 0 ) {
					$vertical_offset = 0;
				}
				if ( $horizontal_offset < 0 ) {
					$horizontal_offset = 0;
				}
			}
		}
		$img = imagecreatetruecolor( $dest_width, $dest_height );
		imagesavealpha( $img, true );
		imagealphablending( $img, false );
		$transparent = imagecolorallocatealpha( $img, 0, 0, 0, 127 );
		imagefill( $img, 0, 0, $transparent );
		imagecopyresampled( $img, $orig, round( $horizontal_offset / 2 ), round( $vertical_offset / 2 ), 0, 0, round( $dest_width - $horizontal_offset ), round( $dest_height - $vertical_offset ), $orig_width, $orig_height );

		return $img;
	}
}