<?php

class LibraryBooks {

	var $DB;
	var $Table = "library_books";
	var $TableAssign = "library_books_assignments";

	function __construct() {
		global $db;
		$this->DB = $db;

		$this->DB->query( "UPDATE `{$this->Table}` SET `book_qty_available`=0 WHERE `book_qty_available` IS NULL" );
	}

	function get_book_stocks( $args = "" ) {
		$default = [
			"orderby"   => "book_name",
			"order"     => "ASC",
			"search"    => "",
			"pagesize"  => "10",
			"page"      => "1",
			"not_empty" => "0",
			"active"    => "",
		];

		$args = set_args( $args, $default );
		extract( $args );

		$Q = "SELECT * FROM `{$this->Table}` WHERE 1";

		if ( trim( $search ) <> "" ) {
			$search = $this->DB->set_string( $search );
			$Q      .= " AND (`book_name` LIKE '%{$search}%' OR `book_department` LIKE '%{$search}%')";
		}

		if ( $not_empty === true || strtoupper( $not_empty ) == "YES" || $not_empty == "1" ) {
			$Q .= " AND `book_qty_available`>0";
		}

		if ( $active == "1" || $active == "0" ) {
			$Q .= " AND `active`='$active'";
		}

		$Q .= " ORDER BY `$orderby` $order";

		$rows = $this->DB->get_rows2( $Q, $page, $pagesize );

		return $rows;
	}

	function save_book_in_stock( $args = "" ) {
		$default = [
			"id"              => "0",
			"book_name"       => "",
			"book_department" => "",
			"book_qty"        => "",
			"active"          => "1",
		];
		$args    = set_args( $args, $default );

		if ( empty( trim( $args["book_name"] ) ) ) {
			return __( "Book name missing" );
		}
		$args["book_qty"] = (int) $args["book_qty"];
		$id               = $args["id"];

		$Q = $this->DB->prepare( "SELECT COUNT(*) FROM `{$this->Table}` WHERE `book_name`=%s AND `id`<>%d", $args["book_name"], $id );
		if ( $this->DB->get_var( $Q ) > 0 ) {
			return __( "Book %1 already in database", $args["book_name"] );
		}

		unset( $args["id"] );
		if ( empty( $id ) ) {
			$response = $this->DB->insert( $this->Table, $args );
			if ( $response !== false ) {
				$id = $this->DB->insert_id;
			}
		} else {
			$response = $this->DB->update( $this->Table, $args, [ "id" => $id ] );
		}

		if ( $response === false ) {
			return $this->DB->last_error;
		} else {
			if ( ! empty( $id ) ) {
				$this->update_assign_book_stats( $id );
			}

			return "OK";
		}
	}

	function get_departments() {
		$Q     = "SELECT DISTINCT `book_department` bd FROM `{$this->Table}` WHERE `book_department`<>''";
		$depts = $this->DB->get_rows( $Q );

		$Out = [];
		foreach ( $depts as $dept ) {
			$Out[] = $dept['bd'];
		}

		return $Out;
	}

	function get_assigned_books( $args = "" ) {
		$default = [
			"book_id"     => "",
			"student_id"  => "",
			"assigned_by" => "",
			"search"      => "",
			"pagesize"    => "20",
			"page"        => "1",
			"orderby"     => "id",
			"order"       => "DESC",
		];

		$args = set_args( $args, $default );
		extract( $args );

		$Q = "SELECT a.*, b.book_name, c.name, d.name student_name FROM `{$this->TableAssign}` a, `{$this->Table}` b, `admins` c, `students` d
			WHERE a.book_id=b.id AND a.assigned_by=c.id AND a.student_id=d.id";

		if ( ! empty( $book_id ) ) {
			$book_id = \Str::make_array( $book_id );
			$Q       .= " AND `book_id` IN ('" . implode( "','", $book_id ) . "')";
		}
		if ( ! empty( $student_id ) ) {
			$student_id = \Str::make_array( $student_id );
			$Q          .= " AND `student_id` IN ('" . implode( "','", $student_id ) . "')";
		}
		if ( ! empty( $assigned_by ) ) {
			$assigned_by = \Str::make_array( $assigned_by );
			$Q           .= " AND `assigned_by` IN ('" . implode( "','", $assigned_by ) . "')";
		}
		$search = trim( $search );
		if ( $search <> "" ) {
			$searches[] = $this->DB->prepare( "`book_name` LIKE %s", '%' . $search . '%' );
			$searches[] = $this->DB->prepare( "`book_qty`=%d", $search );
			if ( strtolower( $search ) == "active" ) {
				$searches[] = "`active`=1";
			} else if ( strtolower( $search ) == "inactive" ) {
				$searches[] = "`active`=0";
			}
			$searches[] = $this->DB->prepare( "d.name LIKE %s", '%' . $search . '%' );
			$searches[] = $this->DB->prepare( "b.book_name LIKE %s", '%' . $search . '%' );

			$Q .= " AND (" . implode( " OR ", $searches ) . ")";
		}

		$Q .= " ORDER BY {$orderby} {$order}";

		$rows = $this->DB->get_rows2( $Q, $page, $pagesize );

		return $rows;
	}

	function assign_book( $args = "" ) {
		$default = [
			"book_id"      => "",
			"student_id"   => "",
			"assigned_qty" => "1",
		];

		$args = set_args( $args, $default );
		extract( $args );

		if ( empty( $book_id ) ) {
			return __( "Please select book" );
		}

		if ( empty( $student_id ) ) {
			return __( "Please select student" );
		}

		$assigned_qty = (int) $assigned_qty;

		if ( $assigned_qty < 1 ) {
			return __( "Please select 1 or more book quantity" );
		}
		$this->update_assign_book_stats( $book_id );
		$row = $this->DB->get_row( "SELECT `book_qty_available`,`active` FROM `{$this->Table}` WHERE `id`='{$book_id}'" );

		if ( ! isset( $row["active"] ) ) {
			return __( "Book not available in stock" );
		}

		if ( $row["active"] == 0 ) {
			return __( "Book is not active and can't assign." );
		}

		$row["book_qty_available"] = (int) $row["book_qty_available"];

		if ( empty( $row["book_qty_available"] ) ) {
			return __( "This book is not available in library stock." );
		}

		if ( $row["book_qty_available"] < $assigned_qty ) {
			return __( "We've only %1 book(s) in stock", $row["book_qty_available"] );
		}

		$args["student_id"] = (int) $args["student_id"];
		$S                  = new Student();
		if ( ! $S->is_active_student( $args["student_id"] ) ) {
			return __( "Student is not available or active in system" );
		}
		$args["assigned_by"]   = \Admins\UserInfo::get_my_id();
		$args["assigned_time"] = \Dates::mysql_datetime();
		$args["assigned_ip"]   = \General::myip();

		$response = $this->DB->insert( $this->TableAssign, $args );

		if ( $response === false ) {
			return $this->DB->last_error;
		}

		$this->update_assign_book_stats( $book_id );

		return "OK";
	}

	private function update_assign_book_stats( $book_id ) {
		$Q = "SELECT IFNULL(SUM(`assigned_qty`),0) FROM `{$this->TableAssign}` WHERE `book_id`='{$book_id}'";

		$sum = $this->DB->get_var( $Q );
		$Q   = "UPDATE `{$this->Table}` SET `book_qty_available`=`book_qty`-$sum WHERE `id`='{$book_id}'";
		$this->DB->query( $Q );
	}

	function return_books( $id ) {
		$book_id = $this->DB->get_var( "SELECT `book_id` FROM `{$this->TableAssign}` WHERE `id`='{$id}'" );

		$response = $this->DB->delete( $this->TableAssign, [ "id" => $id ] );
		if ( $response === false ) {
			return $this->DB->last_error;
		}

		$this->update_assign_book_stats( $book_id );

		return "OK";
	}
}