<?php

/**
 * Class MySQL
 *
 * This will check and set MySQL tables.
 */
class MySQL {

	/*var $Tables = [
		"course_material",
		"teacher_upload_file",
		"datesheet",
		"weekly_lecture_schedule",
		"account_heads",
		"account_transactions",
		"admins",
		"attendance",
		"batches",
		"batch_classes",
		"batch_classes_x_subjects",
		"batch_classes_x_tution_fee",
		"classes",
		"exams",
		"fee_paid",
		"fine_slabs",
		"holidays",
		"mark_sheets",
		"mark_sheets_data",
		"messages",
		"meta_data",
		"options",
		"questions",
		"question_cats",
		"salaries",
		"sms_queue",
		"students",
		"student_classes_subjects",
		"student_x_subject",
		"subjects",
		"teachers",
		"teacher_classes_subjects",
		"test_schedules",
		"users_permissions",
	];*/
	var $TablesStructure;
	var $DB;
	var $charset = 'utf8mb4';
	var $collation = 'utf8mb4_unicode_ci';

	public function __construct() {
		global $db;
		if ( ! $db ) {
			$this->DB = new DB( DBUSER, DBPASS, DBNAME, DBSERVER );
		} else {
			$this->DB = $db;
		}

		if ( $this->DB->get_option( "db_version" ) == md5_file( __FILE__ ) ) {
			return false;
		}

		## type, length, pk/primary_key, ai/auto_increment, default
		$this->TablesStructure["library_books"]             = [
			"id"                 => [ "type" => "int", "pk" => "1", "ai" => 1 ],
			"book_name"          => [ "type" => "varchar", "length" => "250" ],
			"book_qty"           => [ "type" => "tinyint", "length" => "4" ],
			"book_qty_available" => [ "type" => "tinyint", "length" => "4" ],
			"book_department"    => [ "type" => "varchar", "length" => "250" ],
			"active"             => [ "type" => "tinyint", "length" => "1", "default" => "1" ],
		];
		$this->TablesStructure["library_books_assignments"] = [
			"id"            => [ "type" => "int", "pk" => "1", "ai" => 1 ],
			"book_id"       => [ "type" => "int", ],
			"student_id"    => [ "type" => "int", ],
			"assigned_by"   => [ "type" => "int", ],
			"assigned_time" => [ "type" => "datetime", ],
			"assigned_ip"   => [ "type" => "varchar", "length" => "50" ],
			"assigned_qty"  => [ "type" => "tinyint", "length" => "4" ],
		];
		$this->TablesStructure["account_heads"]             = [
			"id"   => [ "type" => "int", "pk" => "1", "ai" => 1 ],
			"head" => [ "type" => "varchar", "length" => "250" ],
		];

		$this->TablesStructure["account_transactions"] = [
			"id"             => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"description"    => [ "type" => "varchar", "length" => "250" ],
			"student_id"     => [ "type" => "int", "default" => "0" ],
			"fee_paid_id"    => [ "type" => "int", "default" => "0" ],           // 	ID from fee_paid table
			"teacher_id"     => [ "type" => "int" ],
			"employee_id"    => [ "type" => "int" ],
			"amount"         => [ "type" => "double", "length" => "12,2" ],
			"date"           => [ "type" => "date" ],
			"added_by"       => [ "type" => "int" ],
			"added_time"     => [ "type" => "date" ],
			"added_ip"       => [ "type" => "varchar", "length" => "50" ],
			"receipt_number" => [ "type" => "varchar", "length" => "50" ],
			"daily_expense"  => [ "type" => "tinyint", "length" => "1" ],
			"payment_mode"   => [ "type" => "varchar", "length" => "20" ],
			"amount_out"     => [ "type" => "double", "length" => "10,2" ],
			"head"           => [ "type" => "varchar", "length" => "100" ],
		];

		// create admin table
		$this->TablesStructure["admins"] = [
			"id"         => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"username"   => [ "type" => "varchar", "length" => 100, ],
			"password"   => [ "type" => "varchar", "length" => 100, ],
			"login_time" => [ "type" => "datetime" ],
			"login_ip"   => [ "type" => "varchar", "length" => 100, ],
			"is_admin"   => [ "type" => "tinyint", "length" => 1, "default" => 0 ],
			"name"       => [ "type" => "varchar", "length" => 100, ],
			"active"     => [ "type" => "tinyint", "length" => 1, ],
			"email"      => [ "type" => "varchar", "length" => 100, ],
			"phone"      => [ "type" => "varchar", "length" => 100, ],
			"token"      => [ "type" => "varchar", "length" => 250, ],
			"active"     => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
		];

		// attendance table
		$this->TablesStructure["attendance"] = [
			"id"         => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"student_id" => [ "type" => "int", "length" => 11, ],
			"attendance" => [ "type" => "char", "length" => 1, ],
			"date"       => [ "type" => "date" ],
			"subject_id" => [ "type" => "int", "length" => 11, ],
			"added_by"   => [ "type" => "int", "length" => 11, ],
			"added_ip"   => [ "type" => "varchar", "length" => 50, ],
			"added_time" => [ "type" => "datetime" ],
			"edit_by"    => [ "type" => "int", "length" => 11, ],
			"class_id"   => [ "type" => "int", "length" => 11, ],
			"batch_id"   => [ "type" => "int", "length" => 11, ],
		];


		// batches table
		$this->TablesStructure["batches"] = [
			"id"         => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"batch_name" => [ "type" => "varchar", "length" => 50, ],
			"active"     => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"added_by"   => [ "type" => "int", "length" => 11, ],
			"added_time" => [ "type" => "datetime" ],
			"added_ip"   => [ "type" => "varchar", "length" => 50, ],
			"start_date" => [ "type" => "date" ],
			"end_date"   => [ "type" => "date" ],
		];


		// sessions table
		//        $this->TablesStructure["sessions"] = [
		//            "id" => ["type" => "int", "pk" => "1", "ai" => "1",],
		//            "session_name" => ["type" => "varchar", "length" => 50,],
		//            "session_time" => ["type" => "varchar", "length" => 50,],
		//            "active" => ["type" => "tinyint", "length" => 1,"default" => 1],
		//            "added_by" => ["type" => "int" , "length" => 11,],
		//            "added_time" => ["type" => "datetime"],
		//            "added_ip" => ["type" => "varchar", "length" => 50,],
		//            "start_date" => ["type" => "date"],
		//            "end_date" => ["type" => "date"],
		//            "subjects" => ["type" => "text"],
		//            "students" => ["type" => "text"],
		//            "teachers" => ["type" => "text"]
		//        ];
		$this->TablesStructure["sessions"] = [
			"id"           => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"session_name" => [ "type" => "varchar", "length" => 50, ],
			"classes"      => [ "type" => "text" ],
			"active"       => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"added_by"     => [ "type" => "int", "length" => 11, ],
			"added_time"   => [ "type" => "datetime" ],
			"added_ip"     => [ "type" => "varchar", "length" => 50, ],
			"start_date"   => [ "type" => "date" ],
			"end_date"     => [ "type" => "date" ],
		];

		// guardian
		$this->TablesStructure["guardians"] = [
			"id"             => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"email"          => [ "type" => "varchar", "length" => 255, ],
			"password"       => [ "type" => "text" ],
			"active"         => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"added_by"       => [ "type" => "int", "length" => 11, ],
			"added_time"     => [ "type" => "datetime" ],
			"added_ip"       => [ "type" => "varchar", "length" => 50, ],
			"login_history"  => [ "type" => "text" ],
			"phone"          => [ "type" => "varchar", "length" => 50 ],
			"validate_phone" => [ "type" => "tinyint", "length" => 1, "default" => "0" ],
		];

		$this->TablesStructure["guardians_x_students"] = [
			"guardian_id" => [ "type" => "int" ],
			"student_id"  => [ "type" => "int" ],
		];

		// sessions classes
		$this->TablesStructure["sessions_classes"] = [
			"id"         => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"sessionid"  => [ "type" => "int", "length" => 11, ],
			"classid"    => [ "type" => "int", "length" => 11, ],
			"active"     => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			//"batch_id" => ["type" => "int" , "length" => 11,],
			"added_by"   => [ "type" => "int", "length" => 11, ],
			"added_time" => [ "type" => "datetime" ],
			"added_ip"   => [ "type" => "varchar", "length" => 50, ],
		];

		// batch_classes table
		$this->TablesStructure["batch_classes"] = [
			"id"              => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"class_name"      => [ "type" => "varchar", "length" => 50, ],
			"batch_id"        => [ "type" => "int", "length" => 11, ],
			//			"class_id"        => [ "type" => "int", "length" => 11, ],
			//			"subjects"        => [ "type" => "text" ],
			"active"          => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"added_by"        => [ "type" => "int", "length" => 11, ],
			"added_time"      => [ "type" => "datetime" ],
			"added_ip"        => [ "type" => "varchar", "length" => 50, ],
			"fee"             => [ "type" => "decimal", "length" => "10,2", ],
			"admission_fee"   => [ "type" => "decimal", "length" => "10,2", ],
			"attendance_type" => [ "type" => "char", "length" => 1, "default" => "D" ],
		];

		// batch_classes_x_subjects table
		$this->TablesStructure["batch_classes_x_subjects"] = [
			"class_id"   => [ "type" => "int" ],
			"subject_id" => [ "type" => "int" ],
		];

		// batch_classes_x_tution_fee table
		$this->TablesStructure["batch_classes_x_tution_fee"] = [
			"id"           => [ "type" => "bigint", "length" => 20, "pk" => "1", "ai" => "1", ],
			"class_id"     => [ "type" => "int", "length" => 11, ],
			"from_date"    => [ "type" => "date" ],
			"to_date"      => [ "type" => "date" ],
			"due_date"     => [ "type" => "date" ],
			"last_payment" => [ "type" => "date" ],
			"added_by"     => [ "type" => "int", "length" => 11 ],
			"added_time"   => [ "type" => "datetime" ],
			"added_ip"     => [ "type" => "varchar", "length" => 50 ],
			"slab_name"    => [ "type" => "varchar", "length" => 50 ],
			"fee_amount"   => [ "type" => "decimal", "length" => "10,2" ],
			"payable_done" => [ "type" => "tinyint", "length" => 1, "default" => "0" ],
			"type"         => [ "type" => "varchar", "length" => 15, "default" => "Tution" ],
			"fine_id"      => [ "type" => "int", "length" => 11, "default" => "0" ],
		];

		// classes table
		$this->TablesStructure["classes"] = [
			"id"         => [ "type" => "int", "length" => 11, "pk" => "1", "ai" => "1", ],
			"name"       => [ "type" => "varchar", "length" => 100, ],
			"added_by"   => [ "type" => "int", "length" => 11, ],
			"added_time" => [ "type" => "datetime" ],
			"added_ip"   => [ "type" => "varchar", "length" => 100 ],
		];

		// course_material table
		$this->TablesStructure["course_material"] = [
			"id"          => [ "type" => "INT", "primary_key" => "1", "auto_increment" => "1", ],
			"classes"     => [ "type" => "text", "default" => "NULL" ],
			"teachers"    => [ "type" => "text", "default" => "NULL" ],
			"subjects"    => [ "type" => "text", "default" => "NULL", ],
			"name"        => [ "type" => "text", "default" => "NULL", ],
			"description" => [ "type" => "text", "default" => "NULL", ],
			"uploadfile"  => [ "type" => "text", "default" => "NULL", ],
			"added_by"    => [ "type" => "int", "length" => "11", "default" => "null", ],
			"added_time"  => [ "type" => "datetime", "default" => "null", ],
			"added_ip"    => [ "type" => "varchar", "length" => "50" ],
		];

		// datesheet table
		$this->TablesStructure["datesheet"] = [
			"id"            => [ "type" => "INT", "length" => 11, "primary_key" => "1", "auto_increment" => "1", ],
			"class_id"      => [ "type" => "text", "default" => "NULL" ],
			"subject_name"  => [ "type" => "text", "default" => "NULL", ],
			"subject_marks" => [ "type" => "text", "default" => "NULL", ],
			"subject_time"  => [ "type" => "time" ],
			"subject_date"  => [ "type" => "date" ],
			"added_by"      => [ "type" => "int", "length" => 11, ],
			"added_time"    => [ "type" => "datetime" ],
			"added_ip"      => [ "type" => "varchar", "length" => 50, ],
		];

		// exams table
		$this->TablesStructure["exams"] = [
			"id"          => [ "type" => "INT", "length" => 11, "primary_key" => "1", "auto_increment" => "1", ],
			"name"        => [ "type" => "varchar", "length" => 200, ],
			"description" => [ "type" => "text" ],
			"date"        => [ "type" => "date" ],
			"active"      => [ "type" => "tinyint", "length" => 1, ],
			"added_by"    => [ "type" => "int", "length" => 11, ],
			"added_time"  => [ "type" => "datetime" ],
			"added_ip"    => [ "type" => "varchar", "length" => 50, ],
		];

		// fee_paid table
		$this->TablesStructure["fee_paid"] = [
			"id"                => [ "type" => "INT", "length" => 11, "primary_key" => "1", "auto_increment" => "1", ],
			"student_id"        => [ "type" => "int", "length" => 11 ],
			"fee_type"          => [ "type" => "varchar", "length" => 10, ],
			"amount"            => [ "type" => "decimal", "length" => "10,2" ],
			"description"       => [ "type" => "text" ],
			"added_by"          => [ "type" => "int", "length" => 11, ],
			"added_time"        => [ "type" => "datetime" ],
			"added_ip"          => [ "type" => "varchar", "length" => 50, ],
			"fee_title"         => [ "type" => "varchar", "length" => 100 ],
			"fee_date"          => [ "type" => "date" ],
			"paid"              => [ "type" => "tinyint", "length" => 1, "default" => 0 ],
			"due_date"          => [ "type" => "date" ],
			"last_payment_date" => [ "type" => "date" ],
			"paid_system_time"  => [ "type" => "datetime" ],
			"paid_by"           => [ "type" => "int", "length" => 11 ],
			"paid_ip"           => [ "type" => "varchar", "length" => 50 ],
			"receipt_number"    => [ "type" => "varchar", "length" => 20 ],
			"fee_slab_id"       => [ "type" => "bigint", "length" => 20, "default" => 0 ],
			"fine"              => [ "type" => "decimal", "length" => "10,2", "default" => "0.00" ],
		];

		// fine_slabs table
		$this->TablesStructure["fine_slabs"] = [
			"id"             => [ "type" => "INT", "length" => 11, "primary_key" => "1", "auto_increment" => "1", ],
			"name"           => [ "type" => "varchar", "length" => 50 ],
			"amount"         => [ "type" => "decimal", "length" => "10,2" ],
			"amount_type"    => [ "type" => "varchar", "length" => 10 ],
			"fine_implement" => [ "type" => "varchar", "length" => 10 ],
			"added_by"       => [ "type" => "int", "length" => 11 ],
			"added_time"     => [ "type" => "datetime" ],
			"added_ip"       => [ "type" => "varchar", "length" => 50 ],
		];

		// holidays table
		$this->TablesStructure["holidays"] = [
			"id"         => [ "type" => "INT", "length" => 11, "primary_key" => "1", "auto_increment" => "1", ],
			"title"      => [ "type" => "varchar", "length" => 100 ],
			"from_date"  => [ "type" => "date" ],
			"to_date"    => [ "type" => "date" ],
			"added_by"   => [ "type" => "int", "length" => 11 ],
			"added_time" => [ "type" => "datetime" ],
			"added_ip"   => [ "type" => "varchar", "length" => 50 ],
		];

		// mark_sheets table
		$this->TablesStructure["mark_sheets"] = [
			"schedule_id"   => [ "type" => "INT", "length" => 11 ],
			"week"          => [ "type" => "varchar", "length" => 50 ],
			"total_marks"   => [ "type" => "decimal", "length" => "6,2" ],
			"start_date"    => [ "type" => "date" ],
			"passing_marks" => [ "type" => "decimal", "length" => "6,2" ],
			"batch_id"      => [ "type" => "INT", "length" => 11 ],
		];

		// mark_sheets_data table
		$this->TablesStructure["mark_sheets_data"] = [
			"schedule_id" => [ "type" => "INT", "length" => 11 ],
			"student_id"  => [ "type" => "int", "length" => 10 ],
			"marks"       => [ "type" => "decimal", "length" => "6,2" ],
			"attendance"  => [ "type" => "tinyint", "length" => 1 ],
			"batch_id"    => [ "type" => "INT", "length" => 11 ],
		];

		// messages table
		$this->TablesStructure["messages"] = [
			"id"           => [ "type" => "INT", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"from"         => [ "type" => "int", "length" => 11 ],
			"to"           => [ "type" => "int", "length" => 11 ],
			"subject"      => [ "type" => "varchar", "length" => 250 ],
			"message"      => [ "type" => "text" ],
			"sending_time" => [ "type" => "datetime" ],
			"viewed"       => [ "type" => "tinyint", "length" => 1, "default" => 0 ],
			"sending_ip"   => [ "type" => "varchar", "length" => 50 ],
			"reading_ip"   => [ "type" => "varchar", "length" => 50 ],
			"reading_time" => [ "type" => "datetime" ],
			"status"       => [ "type" => "varchar", "length" => 10 ],
			"deleted"      => [ "type" => "tinyint", "length" => 1, "default" => 0 ],
		];

		// meta_data table
		$this->TablesStructure["meta_data"] = [
			"id"          => [ "type" => "bigint", "length" => 20, "primary_key" => "1", "auto_increment" => "1" ],
			"table_name"  => [ "type" => "varchar", "length" => 100 ],
			"field_name"  => [ "type" => "varchar", "length" => 100 ],
			"field_value" => [ "type" => "longtext" ],
			"primary_key" => [ "type" => "int", "length" => 11 ],
			"field_label" => [ "type" => "varchar", "length" => 200 ],
		];

		// options table
		$this->TablesStructure["options"] = [
			"id"          => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"field_name"  => [ "type" => "text" ],
			"field_value" => [ "type" => "text" ],
		];

		// questions table
		$this->TablesStructure["questions"] = [
			"id"             => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"question"       => [ "type" => "text" ],
			"cat_id"         => [ "type" => "int", "length" => 11 ],
			"answers"        => [ "type" => "text" ],
			"correct_answer" => [ "type" => "text" ],
			"type"           => [ "type" => "tinyint", "length" => 4 ],
			"active"         => [ "type" => "tinyint", "length" => 1 ],
			"added_by"       => [ "type" => "int", "length" => 11 ],
			"added_time"     => [ "type" => "datetime" ],
			"added_ip"       => [ "type" => "varchar", "length" => 50 ],
		];

		// question_cats table
		$this->TablesStructure["question_cats"] = [
			"id"         => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"cat_name"   => [ "type" => "varchar", "length" => 200 ],
			"active"     => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"added_by"   => [ "type" => "int", "length" => 11 ],
			"added_time" => [ "type" => "datetime" ],
			"added_ip"   => [ "type" => "varchar", "length" => 50 ],
		];

		// salaries table
		$this->TablesStructure["salaries"] = [
			"id"             => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"salary"         => [ "type" => "double", "length" => "10,2" ],
			"deduction"      => [ "type" => "double", "length" => "10,2" ],
			"deduction_desc" => [ "type" => "varchar", "length" => 255 ],
			"teacher_id"     => [ "type" => "int", "length" => 11 ],
			"employee_id"    => [ "type" => "int", "length" => 11 ],
			"salary_month"   => [ "type" => "date" ],
			"payment_date"   => [ "type" => "date" ],
			"added_by"       => [ "type" => "int", "length" => 11 ],
			"added_time"     => [ "type" => "datetime" ],
			"added_ip"       => [ "type" => "varchar", "length" => 50 ],
			"description"    => [ "type" => "varchar", "length" => 255 ],
			"paid"           => [ "type" => "tinyint", "length" => 1 ],
		];

		// sms_queue table
		$this->TablesStructure["sms_queue"] = [
			"id"         => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"to"         => [ "type" => "varchar", "length" => 15 ],
			"message"    => [ "type" => "text" ],
			"user_id"    => [ "type" => "int", "length" => 11 ],
			"ip"         => [ "type" => "varchar", "length" => 50 ],
			"error"      => [ "type" => "text" ],
			"added_time" => [ "type" => "datetime" ],
		];

		// students table deleted
		$this->TablesStructure["students"] = [
			"id"                  => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"roll"                => [ "type" => "varchar", "length" => 15 ],
			"name"                => [ "type" => "varchar", "length" => 100 ],
			"father_name"         => [ "type" => "varchar", "length" => 100 ],
			"guardian_name"       => [ "type" => "varchar", "length" => 100 ],
			"phone"               => [ "type" => "varchar", "length" => 50 ],
			"cell"                => [ "type" => "varchar", "length" => 100 ],
			"guardian_cell"       => [ "type" => "varchar", "length" => 100 ],
			"address"             => [ "type" => "varchar", "length" => 250 ],
			"city"                => [ "type" => "varchar", "length" => 50 ],
			"state"               => [ "type" => "varchar", "length" => 50 ],
			"email"               => [ "type" => "varchar", "length" => 100 ],
			"guardian_email"      => [ "type" => "varchar", "length" => 100 ],
			"active"              => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"added_by"            => [ "type" => "int", "length" => 11 ],
			"added_time"          => [ "type" => "datetime" ],
			"added_ip"            => [ "type" => "varchar", "length" => 100 ],
			"admission_date"      => [ "type" => "date" ],
			"dob"                 => [ "type" => "date" ],
			"nic"                 => [ "type" => "varchar", "length" => 15 ],
			"gender"              => [ "type" => "char", "length" => 1, "default" => "M" ],
			"blood_group"         => [ "type" => "varchar", "length" => 7 ],
			"class_id"            => [ "type" => "int", "length" => 11 ],
			"admission_number"    => [ "type" => "int", "length" => 15 ],
			"nationality"         => [ "type" => "varchar", "length" => 50 ],
			"is_sms"              => [ "type" => "tinyint", "length" => 1, "default" => 0 ],
			"is_email"            => [ "type" => "tinyint", "length" => 1, "default" => 0 ],
			"guardian_relation"   => [ "type" => "varchar", "length" => 20, "default" => 0 ],
			"guardian_dob"        => [ "type" => "date", "default" => "0000-00-00" ],
			"guardian_education"  => [ "type" => "varchar", "length" => 30, "default" => 0 ],
			"guardian_occupation" => [ "type" => "varchar", "length" => 30, "default" => 0 ],
			"guardian_income"     => [ "type" => "decimal", "length" => "15,2", "default" => "0.00" ],
			"guardian_address"    => [ "type" => "varchar", "length" => 200 ],
			"guardian_city"       => [ "type" => "varchar", "length" => 30 ],
			"guardian_state"      => [ "type" => "varchar", "length" => 30 ],
			"guardian_phone"      => [ "type" => "varchar", "length" => 50 ],
			"guardian_remarks"    => [ "type" => "text", "not_null" => 1 ],
			"birth_place"         => [ "type" => "varchar", "length" => 50 ],
			"prev_institute1"     => [ "type" => "varchar", "length" => 100 ],
			"prev_course1"        => [ "type" => "varchar", "length" => 100 ],
			"prev_year1"          => [ "type" => "varchar", "length" => 4 ],
			"prev_marks1"         => [ "type" => "int", "length" => 11 ],
			"fee"                 => [ "type" => "decimal", "length" => "10,2" ],
			"admission_fee"       => [ "type" => "decimal", "length" => "10,2" ],
			"deleted"             => [ "type" => "tinyint", "length" => 1, "default" => 0, "not_null" => 1 ],
			"tution_fee"          => [ "type" => "decimal", "length" => "10,2" ],
			"annual_funds"        => [ "type" => "decimal", "length" => "10,2" ],
			"practical_charges"   => [ "type" => "decimal", "length" => "10,2" ],
			"test_session_fee"    => [ "type" => "decimal", "length" => "10,2" ],
			"admin_notes"         => [ "type" => "text" ],
			"student_category"    => [ "type" => "varchar", "length" => 150, "default" => "General" ],
			"password"            => [ "type" => "varchar", "length" => 100 ],
			"login_history"       => [ "type" => "varchar", "length" => 250 ],
			"religion"            => [ "type" => "varchar", "length" => 100 ],
			"sessions"            => [ "type" => "int", "length" => 11 ],
			"batch_id"            => [ "type" => "int", "length" => 11 ],
		];


		// student_classes_subjects table
		$this->TablesStructure["student_classes_subjects"] = [
			"student_id" => [ "type" => "int", "length" => 11 ],
			"class_id"   => [ "type" => "int", "length" => 11 ],
			"subject_id" => [ "type" => "int", "length" => 11 ],
		];

		// student_x_subject table
		$this->TablesStructure["student_x_subject"] = [
			"student_id" => [ "type" => "int", "length" => 11 ],
			"subject"    => [ "type" => "varchar", "length" => 100 ],
		];

		// subjects table
		$this->TablesStructure["subjects"] = [
			"id"         => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"subject"    => [ "type" => "varchar", "length" => 100 ],
			"short_name" => [ "type" => "varchar", "length" => 100 ],
			"order_by"   => [ "type" => "tinyint", "length" => 4 ],
			"added_by"   => [ "type" => "int", "length" => 11 ],
			"added_time" => [ "type" => "datetime" ],
			"added_ip"   => [ "type" => "varchar", "length" => 100 ],
		];

		// teachers table
		$this->TablesStructure["teachers"] = [
			"id"                => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"name"              => [ "type" => "varchar", "length" => 100 ],
			"short_name"        => [ "type" => "varchar", "length" => 15 ],
			"cell"              => [ "type" => "varchar", "length" => 50 ],
			"address"           => [ "type" => "varchar", "length" => 250 ],
			"email"             => [ "type" => "varchar", "length" => 100 ],
			"date_of_joining"   => [ "type" => "date" ],
			"added_by"          => [ "type" => "int", "length" => 11 ],
			"added_time"        => [ "type" => "datetime" ],
			"added_ip"          => [ "type" => "varchar", "length" => 50 ],
			"salary"            => [ "type" => "double", "length" => "10,2" ],
			"active"            => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"password"          => [ "type" => "varchar", "length" => 250 ],
			"classassign"       => [ "type" => "text" ],
			"courseassign"      => [ "type" => "text" ],
			"courseassignClass" => [ "type" => "text" ],
		];

		// teachers assign classes
		$this->TablesStructure["teachers_x_classes"] = [
			//			"id"          => [ "type" => "int", "primary_key" => "1", "auto_increment" => "1" ],
			"teacher_id" => [ "type" => "int" ],
			"class_id"   => [ "type" => "int" ],
		];

		// teachers assign subjects classes
		$this->TablesStructure["teachers_assign_subjects"] = [
			"id"          => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"teacherid"   => [ "type" => "int", "length" => 11 ],
			"teachername" => [ "type" => "varchar", "length" => 255 ],
			"classname"   => [ "type" => "varchar", "length" => 255 ],
			"subjectname" => [ "type" => "varchar", "length" => 255 ],
			"added_by"    => [ "type" => "int", "length" => 11 ],
			"added_time"  => [ "type" => "datetime" ],
			"added_ip"    => [ "type" => "varchar", "length" => 50 ],
			"active"      => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
		];


		// teachers table
		$this->TablesStructure["otheremployees"] = [
			"id"              => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"name"            => [ "type" => "varchar", "length" => 100 ],
			"short_name"      => [ "type" => "varchar", "length" => 15 ],
			"cell"            => [ "type" => "varchar", "length" => 50 ],
			"address"         => [ "type" => "varchar", "length" => 250 ],
			"email"           => [ "type" => "varchar", "length" => 100 ],
			"date_of_joining" => [ "type" => "date" ],
			"added_by"        => [ "type" => "int", "length" => 11 ],
			"added_time"      => [ "type" => "datetime" ],
			"added_ip"        => [ "type" => "varchar", "length" => 50 ],
			"salary"          => [ "type" => "double", "length" => "10,2" ],
			"active"          => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"password"        => [ "type" => "varchar", "length" => 250 ],
			"designation"     => [ "type" => "varchar", "length" => 250 ],
			"document"        => [ "type" => "text" ],
			"nic"             => [ "type" => "text" ],

		];

		// teacher_comments_student table
		$this->TablesStructure["teacher_comments_student"] = [
			"id"          => [ "type" => "INT", "length" => 11, "primary_key" => "1", "auto_increment" => "1", ],
			"teacherid"   => [ "type" => "INT", "length" => 11 ],
			"teachername" => [ "type" => "varchar", "length" => 50 ],
			"studentid"   => [ "type" => "INT", "length" => 11 ],
			"studentname" => [ "type" => "varchar", "length" => 50 ],
			"comments"    => [ "type" => "text" ],
			"added_by"    => [ "type" => "int", "length" => 11 ],
			"added_time"  => [ "type" => "datetime" ],
			"added_ip"    => [ "type" => "varchar", "length" => 50 ],
		];

		// teacher_classes_subjects table
		$this->TablesStructure["teacher_classes_subjects"] = [
			"teacher_id" => [ "type" => "int", "length" => 11 ],
			"class_id"   => [ "type" => "int", "length" => 11 ],
			"subject_id" => [ "type" => "int", "length" => 11 ],
		];

		// teacher_upload_file table
		$this->TablesStructure["teacher_upload_file"] = [
			"id"           => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"teacherid"    => [ "type" => "text" ],
			"teacher_name" => [ "type" => "text" ],
			"class"        => [ "type" => "text" ],
			"file"         => [ "type" => "text" ],
			"file_name"    => [ "type" => "text" ],
			"added_by"     => [ "type" => "int", "length" => 11 ],
			"added_time"   => [ "type" => "datetime" ],
			"added_ip"     => [ "type" => "varchar", "length" => 50 ],
		];

		// teacher_upload_file table
		$this->TablesStructure["teacher_upload_assignment"] = [
			"id"           => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"teacherid"    => [ "type" => "int", "length" => 11 ],
			"classid"      => [ "type" => "int", "length" => 11 ],
			"subject_name" => [ "type" => "varchar", "length" => 255 ],
			"file"         => [ "type" => "text" ],
			"file_name"    => [ "type" => "text" ],
			"added_by"     => [ "type" => "int", "length" => 11 ],
			"added_time"   => [ "type" => "datetime" ],
			"added_ip"     => [ "type" => "varchar", "length" => 50 ],
		];


		// test_schedules table
		$this->TablesStructure["test_schedules"] = [
			"id"         => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"name"       => [ "type" => "varchar", "length" => 50 ],
			"class_id"   => [ "type" => "int", "length" => 11 ],
			"test_date"  => [ "type" => "date" ],
			"subject"    => [ "type" => "varchar", "length" => 150 ],
			"added_by"   => [ "type" => "int", "length" => 11 ],
			"added_time" => [ "type" => "datetime" ],
			"added_ip"   => [ "type" => "varchar", "length" => 50 ],
			"teacher"    => [ "type" => "varchar", "length" => 200 ],
			"start_date" => [ "type" => "date" ],
		];

		// users_permissions table
		$this->TablesStructure["users_permissions"] = [
			"admin_id"   => [ "type" => "int", "length" => 11 ],
			"permission" => [ "type" => "varchar", "length" => 100 ],
		];

		// weekly_lecture_schedule table
		$this->TablesStructure["weekly_lecture_schedule"] = [
			"id"           => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"teacherid"    => [ "type" => "text" ],
			"teacher_name" => [ "type" => "text" ],
			"classid"      => [ "type" => "text" ],
			"classname"    => [ "type" => "text" ],
			"time"         => [ "type" => "text" ],
			"monday"       => [ "type" => "text" ],
			"tuesday"      => [ "type" => "text" ],
			"wednesday"    => [ "type" => "text" ],
			"thursday"     => [ "type" => "text" ],
			"friday"       => [ "type" => "text" ],
			"saturday"     => [ "type" => "text" ],
			"added_by"     => [ "type" => "int", "length" => 11 ],
			"added_time"   => [ "type" => "datetime" ],
			"added_ip"     => [ "type" => "varchar", "length" => 50 ],
		];

		// students clone table
		$this->TablesStructure["students_clone"] = [
			"id"                  => [ "type" => "int", "length" => 11, "primary_key" => "1", "auto_increment" => "1" ],
			"student_id"          => [ "type" => "int", "length" => 11 ],
			"roll"                => [ "type" => "varchar", "length" => 15 ],
			"name"                => [ "type" => "varchar", "length" => 15 ],
			"father_name"         => [ "type" => "varchar", "length" => 100 ],
			"guardian_name"       => [ "type" => "varchar", "length" => 100 ],
			"phone"               => [ "type" => "varchar", "length" => 50 ],
			"cell"                => [ "type" => "varchar", "length" => 100 ],
			"guardian_cell"       => [ "type" => "varchar", "length" => 100 ],
			"address"             => [ "type" => "varchar", "length" => 250 ],
			"city"                => [ "type" => "varchar", "length" => 50 ],
			"state"               => [ "type" => "varchar", "length" => 50 ],
			"email"               => [ "type" => "varchar", "length" => 100 ],
			"guardian_email"      => [ "type" => "varchar", "length" => 100 ],
			"active"              => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"added_by"            => [ "type" => "int", "length" => 11 ],
			"added_time"          => [ "type" => "datetime" ],
			"added_ip"            => [ "type" => "varchar", "length" => 100 ],
			"admission_date"      => [ "type" => "date" ],
			"dob"                 => [ "type" => "date" ],
			"nic"                 => [ "type" => "varchar", "length" => 15 ],
			"gender"              => [ "type" => "char", "length" => 1, "default" => "M" ],
			"blood_group"         => [ "type" => "varchar", "length" => 7 ],
			"class_id"            => [ "type" => "int", "length" => 11 ],
			"admission_number"    => [ "type" => "int", "length" => 15 ],
			"nationality"         => [ "type" => "varchar", "length" => 50 ],
			"is_sms"              => [ "type" => "tinyint", "length" => 1, "default" => 0 ],
			"is_email"            => [ "type" => "tinyint", "length" => 1, "default" => 0 ],
			"guardian_relation"   => [ "type" => "varchar", "length" => 20, "default" => 0 ],
			"guardian_dob"        => [ "type" => "date", "default" => "0000-00-00" ],
			"guardian_education"  => [ "type" => "varchar", "length" => 30, "default" => 0 ],
			"guardian_occupation" => [ "type" => "varchar", "length" => 30, "default" => 0 ],
			"guardian_income"     => [ "type" => "decimal", "length" => "15,2", "default" => "0.00" ],
			"guardian_address"    => [ "type" => "varchar", "length" => 200 ],
			"guardian_city"       => [ "type" => "varchar", "length" => 30 ],
			"guardian_state"      => [ "type" => "varchar", "length" => 30 ],
			"guardian_phone"      => [ "type" => "varchar", "length" => 50 ],
			"guardian_remarks"    => [ "type" => "text", "not_null" => 1 ],
			"birth_place"         => [ "type" => "varchar", "length" => 50 ],
			"prev_institute1"     => [ "type" => "varchar", "length" => 100 ],
			"prev_course1"        => [ "type" => "varchar", "length" => 100 ],
			"prev_year1"          => [ "type" => "varchar", "length" => 4 ],
			"prev_marks1"         => [ "type" => "int", "length" => 11 ],
			"fee"                 => [ "type" => "decimal", "length" => "10,2" ],
			"admission_fee"       => [ "type" => "decimal", "length" => "10,2" ],
			"deleted"             => [ "type" => "tinyint", "length" => 1, "default" => 0, "not_null" => 1 ],
			"tution_fee"          => [ "type" => "decimal", "length" => "10,2" ],
			"annual_funds"        => [ "type" => "decimal", "length" => "10,2" ],
			"practical_charges"   => [ "type" => "decimal", "length" => "10,2" ],
			"test_session_fee"    => [ "type" => "decimal", "length" => "10,2" ],
			"admin_notes"         => [ "type" => "text" ],
			"student_category"    => [ "type" => "varchar", "length" => 150, "default" => "General" ],
			"password"            => [ "type" => "varchar", "length" => 100 ],
			"login_history"       => [ "type" => "varchar", "length" => 250 ],
			"religion"            => [ "type" => "varchar", "length" => 100 ],
			"sessions"            => [ "type" => "int", "length" => 11 ],
			"class_name"          => [ "type" => "varchar", "length" => 50 ],
			"class_active"        => [ "type" => "tinyint", "length" => 1 ],
			"attendance_type"     => [ "type" => "char", "length" => 1 ],
			"batch_id"            => [ "type" => "int", "length" => 11 ],
			"batch_name"          => [ "type" => "varchar", "length" => 50 ],
			"batch_active"        => [ "type" => "tinyint", "length" => 1 ],
		];

		// batches clone table
		$this->TablesStructure["batches_clone"] = [
			"id"         => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"batch_name" => [ "type" => "varchar", "length" => 50, ],
			"active"     => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"added_by"   => [ "type" => "int", "length" => 11, ],
			"added_time" => [ "type" => "datetime" ],
			"added_ip"   => [ "type" => "varchar", "length" => 50, ],
			"start_date" => [ "type" => "date" ],
			"end_date"   => [ "type" => "date" ],
		];

		// batch_classes clone table
		$this->TablesStructure["batch_classes_clone"] = [
			"id"              => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"class_name"      => [ "type" => "varchar", "length" => 50, ],
			"batch_id"        => [ "type" => "int", "length" => 11, ],
			"class_id"        => [ "type" => "int", "length" => 11, ],
			"subjects"        => [ "type" => "text" ],
			"active"          => [ "type" => "tinyint", "length" => 1, "default" => 1 ],
			"added_by"        => [ "type" => "int", "length" => 11, ],
			"added_time"      => [ "type" => "datetime" ],
			"added_ip"        => [ "type" => "varchar", "length" => 50, ],
			"fee"             => [ "type" => "decimal", "length" => "10,2", ],
			"admission_fee"   => [ "type" => "decimal", "length" => "10,2", ],
			"attendance_type" => [ "type" => "char", "length" => 1, "default" => "D" ],
		];

		$this->TablesStructure["fee_custom_fields"] = [
			"id"         => [ "type" => "int", "primary_key" => "1", "auto_increment" => "1" ],
			"field_name" => [ "type" => "varchar", "length" => "100" ],
			"field_type" => [ "type" => "varchar", "length" => "100" ],
		];

		$this->TablesStructure["fee_custom_fields_data"] = [
			"field_id"    => [ "type" => "int" ],
			"fee_id"      => [ "type" => "int" ],
			"field_value" => [ "type" => "varchar", "length" => "100" ],
		];

		$this->TablesStructure["translations"] = [
			"id"     => [ "type" => "int", "primary_key" => "1", "auto_increment" => "1" ],
			"msgstr" => [ "type" => "text" ],
			"msgval" => [ "type" => "text" ],
			"lang"   => [ "type" => "varchar", "length" => "2", "default" => "en" ],
		];


		$this->TablesStructure["langs"] = [
			"id"      => [ "type" => "int", "primary_key" => "1", "auto_increment" => "1" ],
			"code"    => [ "type" => "varchar", "length" => "2" ],
			"name"    => [ "type" => "varchar", "length" => "50" ],
			"default" => [ "type" => "tinyint", "length" => "1", "default" => "0" ],
			"active"  => [ "type" => "tinyint", "length" => "1", "default" => "1" ],
		];

		$this->TablesStructure["countries"] = [
			"id"           => [ "type" => "int", "primary_key" => "1", "auto_increment" => "1" ],
			"country_code" => [ "type" => "varchar", "length" => "2" ],
			"country_name" => [ "type" => "varchar", "length" => "100" ],
		];

		$this->TablesStructure["email_templates"] = [
			"id"      => [ "type" => "int", "primary_key" => "1", "auto_increment" => "1" ],
			"subject" => [ "type" => "varchar", "length" => "255" ],
			"message" => [ "type" => "text" ],
			"name"    => [ "type" => "varchar", "length" => "255" ],
		];

		$this->TablesStructure["email_templates_x_assigned"] = [
			"id"          => [ "type" => "int", "primary_key" => "1", "auto_increment" => "1" ],
			"email_id"    => [ "type" => "int" ],
			"assigned_id" => [ "type" => "varchar", "length" => "100" ],
		];

		$this->TablesStructure["time_table"] = [
			"id"         => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"teacher_id" => [ "type" => "int" ],
			"shift_id"   => [ "type" => "int" ],
			"day"        => [ "type" => "tinyint", "length" => "1" ],
			"class_id"   => [ "type" => "int" ],
			"subject"    => [ "type" => "varchar", "length" => "100" ],
			"added_by"   => [ "type" => "int" ],
			"added_time" => [ "type" => "datetime" ],
			"added_ip"   => [ "type" => "varchar", "length" => "50" ],
		];

		$this->TablesStructure["time_table_shifts"] = [
			"id"         => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"shift_name" => [ "type" => "varchar", "length" => "50" ],
			"start_time" => [ "type" => "time" ],
			"end_time"   => [ "type" => "time" ],
			"sort"       => [ "type" => "tinyint", "length" => "2" ],
		];

		$this->TablesStructure["admission_requests"] = [
			"id"                => [ "type" => "int", "pk" => "1", "ai" => "1", ],
			"full_name"         => [ "type" => "varchar", "length" => "50" ],
			"father_name"       => [ "type" => "varchar", "length" => "50" ],
			"dob"               => [ "type" => "date" ],
			"gender"            => [ "type" => "char", "length" => "1" ],
			"email"             => [ "type" => "varchar", "length" => "50" ],
			"cell"              => [ "type" => "varchar", "length" => "50" ],
			"address"           => [ "type" => "varchar", "length" => "250" ],
			"city"              => [ "type" => "varchar", "length" => "50" ],
			"country"           => [ "type" => "varchar", "length" => "50" ],
			"guardian_name"     => [ "type" => "varchar", "length" => "50" ],
			"guardian_relation" => [ "type" => "varchar", "length" => "50" ],
			"guardian_cell"     => [ "type" => "varchar", "length" => "50" ],
			"guardian_email"    => [ "type" => "varchar", "length" => "50" ],
			"class_id"          => [ "type" => "int", "default" => "0" ],
			"subjects_ids"      => [ "type" => "varchar", "length" => "200" ],
			"ip"                => [ "type" => "varchar", "length" => "50" ],
			"time"              => [ "type" => "datetime" ],
			"status"            => [ "type" => "tinyint", "length" => "1", "default" => "0" ],
		];

		$this->DB->query( "ALTER DATABASE `" . DBNAME . "` CHARACTER SET {$this->charset} COLLATE {$this->collation}" );

		## Creating tables in MySQL.
		foreach ( $this->TablesStructure as $table_name => $struct ) {
			if ( ! $this->is_mysql_table( $table_name ) ) {
				$query    = $this->generate_create_table_query( $table_name );
				$response = $this->DB->query( $query );
				if ( $response === false ) {
					if ( \General::localhost() ) {
						debug( $this->DB->last_error . " = " . $query );
					}
				}
			} else {
				$this->update_mysql_table_structure( $table_name );
			}

			$this->DB->query( "ALTER TABLE `{$table_name}` CONVERT TO CHARACTER SET {$this->charset}" );
			$this->DB->query( "ALTER TABLE `{$table_name}` DEFAULT CHARSET={$this->charset} COLLATE {$this->collation}" );
		}

		## Setting DB version.
		$this->DB->set_option( "db_version", md5_file( __FILE__ ) );
	}

	function update_mysql_table_structure( $table_name ) {
		$query        = "SHOW COLUMNS FROM `$table_name`";
		$mysql_fields = [];
		$rows         = $this->DB->get_rows( $query );
		foreach ( $rows as $row ) {
			$mysql_fields[] = $row["Field"];
		}

		foreach ( $this->TablesStructure[ $table_name ] as $field_name => $struct ) {
			if ( isset( $struct["pk"] ) ) {
				$struct["primary_key"] = $struct["pk"];
				unset( $struct["pk"] );
			}
			if ( isset( $struct["ai"] ) ) {
				$struct["auto_increment"] = $struct["ai"];
				unset( $struct["ai"] );
			}

			## If column from PHP array doesn't exists in MySQL table then add it.
			if ( ! in_array( $field_name, $mysql_fields ) ) {
				$this->add_mysql_columns( $table_name, $field_name, $struct );
			}

			## If MySQL column doesn't exists in PHP array then remove it from MySQL.
			foreach ( $mysql_fields as $mysql_field ) {
				if ( ! in_array( $mysql_field, array_keys( $this->TablesStructure[ $table_name ] ) ) ) {
					$this->delete_mysql_column( $table_name, $mysql_field );
				}
			}

			##
			foreach ( $rows as $row ) {
				if ( $row["Field"] == $field_name ) {
					$str1 = $row["Field"] . " " . $row["Type"] . " " . $row["Null"] . ( $row["Key"] == "PRI" ? " PRIMARY KEY " : " " ) . "DEFAULT" . ( is_null( $row["Default"] ) ? " NULL" : "'{$row["Default"]}'" ) . ( $row["Extra"] == "auto_increment" ? " AUTO_INCREMENT" : "" );

					$str2 = $field_name . " " . strtolower( $struct["type"] );
					if ( strtolower( $struct["type"] ) == "int" && empty( $struct["length"] ) ) {
						$struct["length"] = "11";
					}
					if ( ! empty( $struct["length"] ) ) {
						$str2 .= "(" . $struct["length"] . ")";
					}

					if ( isset( $struct["primary_key"] ) && $struct["primary_key"] == "1" ) {
						$str2 .= " NO";
					} else if ( empty( $struct["default"] ) || strtoupper( $struct["default"] ) == "NULL" ) {
						$str2 .= " YES";
					} else {
						$str2 .= " NO";
					}

					if ( isset( $struct["primary_key"] ) && $struct["primary_key"] == "1" ) {
						$str2 .= " PRIMARY KEY";
					}

					if ( empty( $struct["default"] ) || strtoupper( $struct["default"] ) == "NULL" ) {
						$str2 .= " DEFAULT NULL";
					} else {
						$str2 .= " DEFAULT '{$struct["default"]}'";
					}

					if ( ! empty( $struct["auto_increment"] ) && $struct["auto_increment"] == "1" ) {
						$str2 .= " AUTO_INCREMENT";
					}

					if ( $str1 <> $str2 ) {
						$this->update_mysql_column( $table_name, $field_name, $struct );
					}
				}
			}
		}
	}

	function update_mysql_column( $table_name, $field_name, $struct ) {
		$query = "ALTER TABLE `$table_name` CHANGE `$field_name` `$field_name` ";
		$query .= strtoupper( $struct["type"] );
		if ( ! empty( $struct["length"] ) ) {
			$query .= "(" . $struct["length"] . ")";
		}
		if ( empty( $struct["default"] ) || strtoupper( $struct["default"] ) == "NULL" ) {
			$query .= " NULL DEFAULT NULL";
		} else {
			$query .= " DEFAULT '" . $struct["default"] . "'";
		}
		$this->DB->query( $query );
	}

	function add_mysql_columns( $table_name, $field_name, $column_structure ) {
		$query = "ALTER TABLE `$table_name` ADD `$field_name`";

		$query .= " " . $column_structure["type"];

		if ( ! empty( $column_structure["length"] ) ) {
			$query .= " (" . $column_structure["length"] . ")";
		}
		if ( isset( $column_structure["not_null"] ) ) {
			if ( $column_structure["not_null"] == "1" ) {
				$query .= " NOT NULL";
			}
		}

		if ( isset( $column_structure["default"] ) ) {
			if ( strtoupper( $column_structure["default"] ) == "NULL" ) {
				$query .= " DEFAULT NULL";
			} else {
				$query .= " DEFAULT '{$column_structure["default"]}'";
			}
		}
		if ( isset( $column_structure["auto_increment"] ) && $column_structure["auto_increment"] == "1" ) {
			$query .= " AUTO_INCREMENT";
		}
		if ( isset( $fields["primary_key"] ) && $column_structure["primary_key"] == "1" ) {
			$query .= " PRIMARY KEY";
		}

		return $this->DB->query( $query );
	}

	function delete_mysql_column( $table_name, $column_name ) {
		$query = "ALTER TABLE `$table_name` DROP `$column_name`";

		return $this->DB->query( $query );
	}

	private function generate_create_table_query( $table ) {
		if ( ! isset( $this->TablesStructure[ $table ] ) ) {
			return false;
		}

		$query = "CREATE TABLE IF NOT EXISTS `$table` (\n";
		$F     = [];
		foreach ( $this->TablesStructure[ $table ] as $field_name => $fields ) {
			if ( isset( $fields["pk"] ) ) {
				$fields["primary_key"] = $fields["pk"];
				unset( $fields["pk"] );
			}
			if ( isset( $fields["ai"] ) ) {
				$fields["auto_increment"] = $fields["ai"];
				unset( $fields["ai"] );
			}

			$line = "\t`$field_name` " . strtoupper( $fields["type"] );
			if ( ! empty( $fields["length"] ) ) {
				$line .= " (" . $fields["length"] . ")";
			}
			if ( isset( $fields["not_null"] ) ) {
				if ( $fields["not_null"] == "1" ) {
					$line .= " NOT NULL";
				}
			}

			if ( isset( $fields["default"] ) ) {
				if ( strtoupper( $fields["default"] ) == "NULL" ) {
					$line .= " DEFAULT NULL";
				} else {
					$line .= " DEFAULT '{$fields["default"]}'";
				}
			}
			if ( isset( $fields["auto_increment"] ) && $fields["auto_increment"] == "1" ) {
				$line .= " AUTO_INCREMENT";
			}
			if ( isset( $fields["primary_key"] ) && $fields["primary_key"] == "1" ) {
				$line .= " PRIMARY KEY";
			}

			$F[] = $line;
		}

		$query .= implode( ",\n", $F );

		$query .= "\n);";

		return $query;
	}

	function is_mysql_table( $table_name ) {
		$Q   = "SHOW FULL TABLES LIKE '$table_name'";
		$row = $this->DB->get_row( $Q );
		if ( ! isset( $row['Table_type'] ) ) {
			return false;
		}

		return $row['Table_type'] == "BASE TABLE";
	}

	function is_mysql_view( $table_name ) {
		//if (!is_mysql_table( $table_name)) return false;
		$Q   = "SHOW FULL TABLES LIKE '$table_name'";
		$row = $this->DB->get_row( $Q );
		if ( ! isset( $row['Table_type'] ) ) {
			return false;
		}

		return $row['Table_type'] == "VIEW";
	}
}