<?php
/*
 * https://wkhtmltopdf.org/usage/wkhtmltopdf.txt
 */

class PDF {

	public static function generate_url( $args = [] ) {
		global $db;
		$default          = [
			'url'                  => '',
			'download'             => false,
			'filename'             => '',
			'pdf_page_orientation' => $db->get_option( 'pdf_page_orientation' ),
			'pdf_page_size'        => $db->get_option( 'pdf_page_size' ),
			'pdf_page_numbers'     => $db->get_option( 'pdf_page_numbers' ),
			'pdf_zoom'             => $db->get_option( 'pdf_zoom' ),
			'pdf_margin_top'       => $db->get_option( 'pdf_margin_top' ),
			'pdf_margin_bottom'    => $db->get_option( 'pdf_margin_bottom' ),
			'pdf_margin_right'     => $db->get_option( 'pdf_margin_right' ),
			'pdf_margin_left'      => $db->get_option( 'pdf_margin_left' ),
		];
		$args             = set_args( $args, $default );
		$args['download'] = (bool) $args['download'];

		//echo $args['url']; die;
		$token = urlencode( base64_encode( json_encode( [ 'url' => $args['url'] ] ) ) );

		if ( \General::localhost() ) {
			$url = 'https://fekara.bindia.dk/pdf_gen.php?token=' . $token;
		} else {
			$url = 'http://app.fekara.com/pdf_gen.php?token=' . $token;
		}


		if ( $args['download'] ) {
			$url .= '&download';
		}
		if ( trim( $args['filename'] ) <> '' ) {
			$url .= '&filename=' . trim( $args['filename'] );
		}
		foreach ( $args as $key => $val ) {
			if ( $key == 'url' || $key == 'download' || $key == 'filename' ) {
				continue;
			}
			if ( $val <> '' ) {
				$url .= '&' . $key . '=' . $val;
			}
		}

		return $url;
	}

	public static function pdf_head_html() {
		$html = "
			<meta charset=\"utf-8\">
			<link rel=\"stylesheet\" type=\"text/css\" href=\"https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.0.3/css/bootstrap.min.css\">
			<link href=\"" . ASSETS_URL . "css/custom.css?" . md5_file( ASSETS_PATH . 'css/custom.css' ) . "\" rel=\"stylesheet\" type=\"text/css\">
			<link href=\"" . \Theme::url() . "/style.css?" . md5_file( Theme::path() . 'style.css' ) . "\" rel=\"stylesheet\" type=\"text/css\">
			<style>
				tr { page-break-inside: avoid; }
				body {
			        overflow: visible !important;
			    }
			</style>
		";

		return $html;
	}

	public static function pdf_header( $title ) {
		global $db;
		$html = "
			<head><div style='text-align: center'>
				<h2><img style='max-height:80px;max-width:80px;float:left;' src='" . App::get_logo( "3" ) . "'>" . System . "</h2>";

		$html .= $db->get_option( "inst_address" ) . '<br>';
		$html .= $db->get_option( "inst_contacts" );
		$html .= '<hr>';
		$html .= "<h3>{$title}</h3>";
		$html .= "</div></head>";

		return $html;
	}

	public static function generate_html( $title, $array, $headings, $return = false ) {
		$html = "<html><head><title>{$title}</title>" . self::pdf_head_html() . "</head><body>
		";

		$html .= self::pdf_header( $title );


		$html .= "<table class='table table-bordered'><thead><tr class='heading'>";

		foreach ( $headings as $key => $heading ) {
			$css = '';
			if ( isset( $heading['css'] ) ) {
				$css = 'style="' . $heading['css'] . '"';
			}
			$html .= '<th ' . $css . '>' . $heading['name'] . '</th>';
		}

		$html .= "</tr></thead><tbody>";

		foreach ( $array as $key => $row ) {
			$html .= '<tr>';
			foreach ( $headings as $col => $heading ) {
				if ( $col == '{num}' ) {
					$html .= '<td>' . ( $key + 1 ) . '</td>';
				} else {
					$html .= '<td>' . $row[ $col ] . '</td>';
				}
			}
			$html .= '</tr>';
		}

		$html .= "</tbody></table></body></html>";

		if ( $return ) {
			return $html;
		} else {
			echo $html;
			die;
		}
	}

	public static function html_to_pdf( $html_text, $title = '', $download = true, $filename = '', $pdf_options = [] ) {
		global $db;
		$default     = [
			'pdf_page_orientation' => $db->get_option( 'pdf_page_orientation' ),
			'pdf_page_size'        => $db->get_option( 'pdf_page_size' ),
			'pdf_page_numbers'     => $db->get_option( 'pdf_page_numbers' ),
			'pdf_zoom'             => $db->get_option( 'pdf_zoom' ),
			'pdf_margin_top'       => $db->get_option( 'pdf_margin_top' ),
			'pdf_margin_bottom'    => $db->get_option( 'pdf_margin_bottom' ),
			'pdf_margin_right'     => $db->get_option( 'pdf_margin_right' ),
			'pdf_margin_left'      => $db->get_option( 'pdf_margin_left' ),
		];
		$pdf_options = set_args( $pdf_options, $default );


		$title = (string) $title;

		$html = '<html><head><meta charset="utf-8">';
		if ( $title <> '' ) {
			$html .= '<title>' . $title . '</title>';
		}
		$html .= self::pdf_head_html();
		$html .= '</head><body>';

		if ( $title <> '' ) {
			$html .= self::pdf_header( $title );
		}

		$html .= $html_text;
		$html .= '</body></html>';

		@unlink( ROOT_PATH . 'contents/temp.html' );
		$target_file = ROOT_PATH . 'contents/temp.html';
		file_put_contents( ROOT_PATH . 'contents/temp.html', $html );

		$pdf_options['url']      = DOMAIN_URL . 'contents/temp.html';
		$pdf_options['download'] = $download;
		$pdf_options['filename'] = $filename;

		return $url = self::generate_url( $pdf_options );
	}
}