<?php

class Str {

	function __construct() {
		register_shutdown_function( [ &$this, '__destruct' ] );
	}

	function __destruct() {
		return true;
	}

	function generate_password( $length = 8 ) {
		$chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-*&%$#@!()';
		$count = mb_strlen( $chars );

		for ( $i = 0, $result = ''; $i < $length; $i ++ ) {
			$index  = rand( 0, $count - 1 );
			$result .= mb_substr( $chars, $index, 1 );
		}

		return $result;
	}

	public static function strtodb( $str ) {
		return str_replace( "'", "''", $str );
	}

	public static function strtodbc( $str ) {
		return str_replace( "'", "''", trim( $str ) );
	}

	function two( $str ) {
		// This function divide string into two pieces and will return an array.
		$words  = str_word_count( $str, 1 );
		$first  = ceil( count( $words ) / 2 );
		$out[0] = "";
		for ( $x = 0; $x <= $first - 1; $x ++ ) {
			$out[0] .= $words[ $x ] . " ";
		}
		$out[1] = "";
		for ( $x = $first; $x < count( $words ); $x ++ ) {
			$out[1] .= $words[ $x ] . " ";
		}
		$out[0] = trim( $out[0] );
		$out[1] = trim( $out[1] );

		return $out;
	}

	public static function number_with_suffix( $num ) {
		if ( ! in_array( ( $num % 100 ), [ 11, 12, 13 ] ) ) {
			switch ( $num % 10 ) {
				// Handle 1st, 2nd, 3rd
				case 1:
					return $num . 'st';
				case 2:
					return $num . 'nd';
				case 3:
					return $num . 'rd';
			}
		}

		return $num . 'th';
	}

	public static function generate_api( $secretKey = "" ) {
		//$apiKey = time();
		if ( $secretKey == "" ) {
			$secretKey = System;
		}
		// Generates a random string of ten digits
		$salt = mt_rand();
		// Computes the signature by hashing the salt with the secret key as the key
		$signature = hash_hmac( 'sha256', $salt, $secretKey, true );
		// base64 encode...
		$encodedSignature = base64_encode( $signature );

		// urlencode...
		return str_replace( "+", "", $encodedSignature );
		//$encodedSignature = urlencode($encodedSignature);
	}

	public static function is_email( $email ) {
		return filter_var( $email, FILTER_VALIDATE_EMAIL );
	}

	public static function xtrim( $string ) {
		$s = trim( $string );
		$s = trim( $s, " \t\n\r\0\x0B" );
		//$s = trim($s, chr(239));
		$s = iconv( "UTF-8", "UTF-8//IGNORE", $s ); // drop all non utf-8 characters

		// this is some bad utf-8 byte sequence that makes mysql complain - control and formatting i think
		$s = preg_replace( '/(?>[\x00-\x1F]|\xC2[\x80-\x9F]|\xE2[\x80-\x8F]{2}|\xE2\x80[\xA4-\xA8]|\xE2\x81[\x9F-\xAF])/', ' ', $s );

		$s = preg_replace( '/\s+/', ' ', $s ); // reduce all multiple whitespace to a single space

		return $s;
	}

	/**
	 * @param        $arr
	 * @param string $sep
	 *
	 * @return array
	 */
	public static function make_array( $arr, $sep = "," ) {
		if ( is_array( $arr ) ) {
			return $arr;
		}
		$arr = explode( $sep, $arr );
		$arr = array_map( 'trim', $arr );
		$arr = array_filter( $arr );

		return $arr;
	}

	/**
	 * @param $ip_addr
	 *
	 * @return bool
	 */
	public static function is_ip( $ip_addr ) {
		//first of all the format of the ip address is matched
		if ( preg_match( "/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/", $ip_addr ) ) {
			//now all the intger values are separated
			$parts = explode( ".", $ip_addr );
			//now we need to check each part can range from 0-255
			foreach ( $parts as $ip_parts ) {
				if ( intval( $ip_parts ) > 255 || intval( $ip_parts ) < 0 ) {
					return false;
				} //if number is not within range of 0-255
			}

			return true;
		} else {
			return false;
		} //if format of ip address doesn't matches
	}

	/**
	 * @param        $phone_number
	 * @param string $region_code
	 *
	 * @return bool
	 * @throws \libphonenumber\NumberParseException
	 *                                             Check if given string is valid phone or not
	 */
	public static function is_phone( $phone_number, $region_code = '' ) {
		if ( empty( $region_code ) ) {
			global $db;
			$region_code = $db->get_option( "default_phone_country" );
			if ( empty( $region_code ) ) {
				$region_code = 'PK';
			}
		}
		// https://github.com/giggsey/libphonenumber-for-php
		$phoneNumberUtil = \libphonenumber\PhoneNumberUtil::getInstance();

		$phoneNumberObject = $phoneNumberUtil->parse( $phone_number, $region_code );

		return $phoneNumberUtil->isValidNumber( $phoneNumberObject );
	}

	/**
	 * @param $str
	 *
	 * @return string
	 *               Converts string into proper format. e.g. Shakeel Ahmed etc.
	 */
	public static function strtoproper( $str ) {
		return ucwords( strtolower( $str ) );
	}

	/**
	 * @param $arg
	 *
	 * @return string
	 *               Return string after convert bytes into human read format like 35Kb, 102Mb etc.
	 */
	public static function formatsize( $arg ) {
		if ( $arg > 0 ) {
			$j   = 0;
			$ext = array( " bytes", " Kb", " Mb", " Gb", " Tb" );
			while ( $arg >= pow( 1024, $j ) ) {
				++ $j;
			}

			return round( $arg / pow( 1024, $j - 1 ) * 100 ) / 100 . $ext[ $j - 1 ];
		} else {
			return "0 Mb";
		}
	}

	/**
	 * @param        $text
	 * @param string $allowed_tags
	 *
	 * @return false|null|string|string[]
	 *                                   Converts string(HTML) into neat html.
	 */
	public static function strip_word_html( $text, $allowed_tags = '<b><i><sup><sub><em><strong><u><br>' ) {
		mb_regex_encoding( 'UTF-8' );
		//replace MS special characters first
		$search  = array( '/&lsquo;/u', '/&rsquo;/u', '/&ldquo;/u', '/&rdquo;/u', '/&mdash;/u' );
		$replace = array( '\'', '\'', '"', '"', '-' );
		$text    = preg_replace( $search, $replace, $text );
		//make sure _all_ html entities are converted to the plain ascii equivalents - it appears
		//in some MS headers, some html entities are encoded and some aren't
		$text = html_entity_decode( $text, ENT_QUOTES, 'UTF-8' );
		//try to strip out any C style comments first, since these, embedded in html comments, seem to
		//prevent strip_tags from removing html comments (MS Word introduced combination)
		if ( mb_stripos( $text, '/*' ) !== false ) {
			$text = mb_eregi_replace( '#/\*.*?\*/#s', '', $text, 'm' );
		}
		//introduce a space into any arithmetic expressions that could be caught by strip_tags so that they won't be
		//'<1' becomes '< 1'(note: somewhat application specific)
		$text = preg_replace( array( '/<([0-9]+)/' ), array( '< $1' ), $text );
		$text = strip_tags( $text, $allowed_tags );
		//eliminate extraneous whitespace from start and end of line, or anywhere there are two or more spaces, convert it to one
		$text = preg_replace( array( '/^\s\s+/', '/\s\s+$/', '/\s\s+/u' ), array( '', '', ' ' ), $text );
		//strip out inline css and simplify style tags
		$search  = array(
			'#<(strong|b)[^>]*>(.*?)</(strong|b)>#isu',
			'#<(em|i)[^>]*>(.*?)</(em|i)>#isu',
			'#<u[^>]*>(.*?)</u>#isu',
		);
		$replace = array( '<b>$2</b>', '<i>$2</i>', '<u>$1</u>' );
		$text    = preg_replace( $search, $replace, $text );
		//on some of the ?newer MS Word exports, where you get conditionals of the form 'if gte mso 9', etc., it appears
		//that whatever is in one of the html comments prevents strip_tags from eradicating the html comment that contains
		//some MS Style Definitions - this last bit gets rid of any leftover comments */
		$num_matches = preg_match_all( "/\<!--/u", $text, $matches );
		if ( $num_matches ) {
			$text = preg_replace( '/\<!--(.)*--\>/isu', '', $text );
		}

		return $text;
	}

	/**
	 * @param $i
	 *
	 * @return string
	 *
	 */
	public static function yesno( $i ) {
		$i = (bool) $i;
		if ( $i ) {
			return "Yes";
		} else {
			return "No";
		}
	}
}