<?php

namespace Student;

use libphonenumber\NumberParseException;

class AdmissionRequests {

	var $Table = "admission_requests";
	protected $DB;
	var $Status;

	public function __construct() {
		global $db;
		$this->DB     = $db;
		$this->Status = (object) [
			"pending"  => "0",
			"approved" => "1",
			"declined" => "0",
		];
	}

	public function count_pending_requests() {
		$Q = $this->DB->prepare( "SELECT COUNT(*) FROM `{$this->Table}` WHERE `status`=%d", $this->Status->pending );

		return $this->DB->get_var( $Q );
	}

	function save( $args = "" ) {
		$default = [
			"full_name"         => "",
			"father_name"       => "",
			"dob"               => "",
			"gender"            => "",
			"email"             => "",
			"cell"              => "",
			"address"           => "",
			"city"              => "",
			"country"           => "",
			"guardian_name"     => "",
			"guardian_relation" => "",
			"guardian_cell"     => "",
			"guardian_email"    => "",
			"class_id"          => "0",
			"subjects_ids"      => [],
			"ip"                => \General::myip(),
			"time"              => \Dates::mysql_datetime(),
			"status"            => $this->Status->pending,
		];
		$args    = set_args( $args, $default );
		foreach ( $args as $k => $v ) {
			if ( is_string( $v ) ) {
				$args[ $k ] = trim( $args[ $k ] );
			}
		}
		extract( $args );

		if ( empty( $full_name ) ) {
			return "Full name missing";
		}
		if ( empty( $father_name ) ) {
			return "Father name missing";
		}
		if ( empty( $dob ) || ! \Dates::is_date( $dob ) ) {
			return "Valid date of birth required";
		}
		if ( empty( $gender ) ) {
			return "Gender missing";
		}
		if ( ! empty( $email ) && ! \Str::is_email( $email ) ) {
			return "Invalid email address";
		}
		if ( ! empty( $cell ) && ! \Str::is_phone( $cell ) ) {
			return "Invalid cell number";
		}
		if ( empty( $guardian_name ) ) {
			return "Guardian name missing";
		}
		if ( empty( $guardian_relation ) ) {
			return "Guardian relation missing";
		}
		try {
			if ( ! \Str::is_phone( $args['guardian_cell'] ) ) {
				return "Valid guardian cell required";
			}
		} catch ( NumberParseException $e ) {
			return "Valid guardian cell required";
		}
		if ( ! empty( $guardian_email ) && ! \Str::is_email( $guardian_email ) ) {
			return "Invalid guardian email";
		}
		if ( empty( $class_id ) ) {
			return "Please select class";
		}
		if ( ! empty( $args["subjects_ids"] ) ) {
			$args["subjects_ids"] = \Str::make_array( $args["subjects_ids"] );
			$args["subjects_ids"] = implode( ",", $args["subjects_ids"] );
		}
		$args["dob"] = \Dates::mysql_date( $args["dob"] );

		$response = $this->DB->insert( $this->Table, $args );

		if ( $response === false ) {
			return $this->DB->last_error;
		}

		return "OK";
	}

	public function get_admissions_requests( $args = "" ) {
		$default = [
			"mysql_cols" => "*",
			"page"       => "1",
			"pagesize"   => "20",
		];
		$args    = set_args( $args, $default );
		extract( $args );

		$Q = "SELECT {$args['mysql_cols']},
 			(SELECT `class_name` FROM `batch_classes` WHERE `id`=o.`class_id`) class_name,
 			(SELECT IFNULL(GROUP_CONCAT(`subject` SEPARATOR ', '),'') FROM `subjects` where find_in_set(`id`,o.subjects_ids)>0) subjects
 			FROM `{$this->Table}` o WHERE 1";

		$rows = $this->DB->get_rows2( $Q, $args['page'], $args['pagesize'] );

		return $rows;
	}

	public function delete( $id ) {
		$response = $this->DB->delete( $this->Table, [ "id" => $id ] );
		if ( $response === false ) {
			return $this->DB->last_error;
		}

		return "OK";
	}
}