<?php

namespace Student;

class Files extends \Student {

	const table_students_files = 'students_files';
	private $Path = ROOT_PATH . 'contents/students_x_teachers/';

	public function __construct() {
		parent::__construct();

		## Create if folder doesn't exists
		if ( ! is_dir( $this->Path ) ) {
			@mkdir( $this->Path );
		}
		$this->update_db();
	}

	/**
	 * @param $post
	 * @param $files
	 *
	 * @return string
	 *
	 * Upload student files into teacher (students x teachers)
	 */
	public function upload_file( $post, $files ) {
		$default = [
			'student_id'  => 0,
			'teacher_id'  => 0,
			'uploaded_by' => 'student',
		];
		$post    = set_args( $post, $default );

		$post['student_id'] = (int) $post['student_id'];
		$post['teacher_id'] = (int) $post['teacher_id'];

		if ( $post['student_id'] < 1 ) {
			return 'Student missing';
		}
		if ( $post['teacher_id'] < 1 ) {
			return 'Teacher missing';
		}

		if ( ! isset( $files['file']['error'] ) ) {
			return 'File upload file error';
		}
		if ( $files['file']['error'] > 1 ) {
			return 'File upload file error';
		}
		$file = $files['file'];

		$last_id = (int) $this->DB->select_cols( 'id' )
		                          ->order( 'id', 'DESC' )
		                          ->limit( 1 )
		                          ->get_var( self::table_students_files ) + 1;

		// Keep 100 files in one folder.
		$folder = $last_id <= 100 ? 1 : intval( $last_id / 100 );

		$data = [
			'student_id'  => $post['student_id'],
			'teacher_id'  => $post['teacher_id'],
			'filename'    => $file['name'],
			'uploaded_by' => $post['uploaded_by'],
			'folder'      => $folder,
		];

		$this->DB->insert( self::table_students_files, $data );
		$insert_id = $this->DB->insert_id;

		$new_path = $this->Path . $folder . '/';
		if ( ! is_dir( $new_path ) ) {
			@mkdir( $new_path );
		}

		$new_path .= $insert_id . '_' . $file['name'];

		@move_uploaded_file( $file['tmp_name'], $new_path );

		if ( ! is_file( $new_path ) ) {
			$this->DB->delete( self::table_students_files, [ 'id' => $insert_id ] );

			return 'File upload error';
		}

		return 'OK';
	}

	/**
	 * @param $id
	 *
	 * @return bool
	 *
	 * Deletes student x teacher file.
	 */
	public function delete_file( $id ) {
		$id  = (int) $id;
		$row = $this->DB->where( 'id', $id )
		                ->select_cols( 'id,filename,folder' )
		                ->get_row( self::table_students_files );

		if ( ! isset( $row['id'] ) ) {
			return false;
		}
		$path = $this->Path . $row['folder'] . '/' . $row['id'] . '_' . $row['filename'];
		if ( ! is_file( $path ) ) {
			return false;
		}
		@unlink( $path );
		$this->DB->delete( self::table_students_files, [ 'id' => $id ] );

		return true;
	}

	/**
	 * @param $student_id
	 *
	 * @return array
	 *
	 * Return array of files
	 */
	public function get_files( $student_id, $group_by_teacher = false, $args = [] ) {
		$default = [
			'teacher_id' => 0,
		];
		$args    = set_args( $args, $default );

		$this->DB->where( 'student_id', $student_id )->where( 'uploaded_by', 'student' );

		if ( ! empty( $args['teacher_id'] ) ) {
			$this->DB->where( 'teacher_id', $args['teacher_id'] );
		}

		$rows = $this->DB->order( 'teacher_id' )->order( 'id' )->get_rows( self::table_students_files );

		$teachers         = [];
		$group_by_teacher = (bool) $group_by_teacher;
		foreach ( $rows as $index => &$row ) {
			$row['file_path'] = $this->Path . $row['folder'] . '/' . $row['id'] . '_' . $row['filename'];
			if ( ! is_file( $row['file_path'] ) ) {
				$this->DB->delete( self::table_students_files, [ 'id' => $row['id'] ] );
				unset( $rows[ $index ] );
			}

			$row['file_size'] = filesize( $row['file_path'] );
			$row['file_url']  = strtr( $row['file_path'], [ ROOT_PATH => DOMAIN_URL ] );

			if ( $group_by_teacher ) {
				$teachers[ $row['teacher_id'] ][] = $row;
			}
		}

		if ( $group_by_teacher ) {
			return $teachers;
		}

		return $rows;
	}

	/**
	 * Update DB backup
	 */
	private function update_db() {
		$this->DB->create_table( self::table_students_files )->add_column( [
			'col_name'       => 'id',
			'primary_key'    => true,
			'auto_increment' => true,
			'type'           => 'int',
		] )->add_column( [
			'col_name' => 'student_id',
		] )->add_column( [
			'col_name' => 'teacher_id',
		] )->add_column( [
			'col_name' => 'filename',
			'type'     => 'varchar',
			'length'   => 250,
		] )->add_column( [
			'col_name' => 'uploaded_by',
			'type'     => 'varchar',
			'length'   => 7,
		] )->add_column( [
			'col_name' => 'folder',
			'type'     => 'varchar',
			'length'   => 5,
		] )->end();
	}

	/**
	 * @param $student_id
	 *
	 * @return int
	 *
	 * Count files uploaded by students
	 */
	public function files_counter( $student_id ) {
		$student_id = (int) $student_id;

		return $this->DB->where( 'student_id', $student_id )
		                ->where( 'uploaded_by', 'student' )
		                ->count_result( self::table_students_files );
	}
}