<?php

namespace Teacher;

class Files extends \Teacher {

	const table_files = 'students_files';
	private $Path = ROOT_PATH . 'contents/students_x_teachers/';

	public function __construct() {
		parent::__construct();
	}

	/**
	 * @param $teacher_id
	 *
	 * @return int
	 *
	 * Count files uploaded by teacher
	 */
	public function files_counter( $teacher_id ) {
		$teacher_id = (int) $teacher_id;

		return $this->DB->where( 'teacher_id', $teacher_id )
		                ->where( 'uploaded_by<>', 'student' )
		                ->count_result( self::table_files );
	}

	/**
	 * @param $student_id
	 *
	 * @return array
	 *
	 * Return array of files
	 */
	public function get_files( $teacher_id = 0, $group_by_student = false, $teacher_files = false ) {
		$teacher_id = (int) $teacher_id;
		if ( $teacher_id < 1 ) {
			$teacher_id = $this->get_teacher_id();
		}

		$this->DB->select_cols( 'f.*,s.name,s.class_name' )
		         ->where( 'teacher_id', $teacher_id )
		         ->join( 'students_view s', 's.id = student_id' )
		         ->order( 'student_id' )
		         ->order( 'id' );

		if ( $teacher_files ) {
			$this->DB->where( 'uploaded_by<>', 'student' );
		} else {
			$this->DB->where( 'uploaded_by', 'student' );
		}

		$rows = $this->DB->get_rows( self::table_files . ' f' );

		$students         = [];
		$group_by_student = (bool) $group_by_student;
		foreach ( $rows as $index => &$row ) {
			$row['file_path'] = $this->Path . $row['folder'] . '/' . $row['id'] . '_' . $row['filename'];
			if ( ! is_file( $row['file_path'] ) ) {
				$this->DB->delete( self::table_files, [ 'id' => $row['id'] ] );
				unset( $rows[ $index ] );
			}

			$row['file_size'] = filesize( $row['file_path'] );
			$row['file_url']  = strtr( $row['file_path'], [ ROOT_PATH => DOMAIN_URL ] );

			if ( $group_by_student ) {
				$students[ $row['student_id'] ][] = $row;
			}
		}

		if ( $group_by_student ) {
			return $students;
		}

		return $rows;
	}


	/**
	 * @param $id
	 *
	 * @return bool
	 *
	 * Delete attached file.
	 */
	public function delete_file( $id ) {
		$id  = (int) $id;
		$row = $this->DB->where( 'id', $id )->select_cols( 'id,filename,folder' )->get_row( self::table_files );

		if ( ! isset( $row['id'] ) ) {
			return false;
		}
		$path = $this->Path . $row['folder'] . '/' . $row['id'] . '_' . $row['filename'];
		if ( ! is_file( $path ) ) {
			return false;
		}
		@unlink( $path );
		$this->DB->delete( self::table_files, [ 'id' => $id ] );

		return true;
	}

	public function upload_file( $post, $files ) {
		$default = [
			'student_id'  => 0,
			'teacher_id'  => 0,
			'uploaded_by' => 'student',
		];
		$post    = set_args( $post, $default );

		$post['student_id'] = (int) $post['student_id'];
		$post['teacher_id'] = (int) $post['teacher_id'];

		if ( $post['student_id'] < 1 ) {
			return 'Student missing';
		}
		if ( $post['teacher_id'] < 1 ) {
			return 'Teacher missing';
		}

		if ( ! isset( $files['file']['error'] ) ) {
			return 'File upload file error';
		}
		if ( $files['file']['error'] > 1 ) {
			return 'File upload file error';
		}
		$file = $files['file'];

		$last_id = (int) $this->DB->select_cols( 'id' )
		                          ->order( 'id', 'DESC' )
		                          ->get_var( self::table_files ) + 1;
		$folder  = $last_id <= 100 ? 1 : intval( $last_id / 100 );

		$data = [
			'student_id'  => $post['student_id'],
			'teacher_id'  => $post['teacher_id'],
			'filename'    => $file['name'],
			'uploaded_by' => $post['teacher_id'],
			'folder'      => $folder,
		];

		$this->DB->insert( self::table_files, $data );
		$insert_id = $this->DB->insert_id;

		$new_path = $this->Path . $folder . '/';
		@mkdir( $new_path );

		$new_path .= $insert_id . '_' . $file['name'];

		@move_uploaded_file( $file['tmp_name'], $new_path );

		if ( ! is_file( $new_path ) ) {
			$this->DB->delete( self::table_files, [ 'id' => $insert_id ] );

			return 'File upload error';
		}

		return 'OK';
	}

	public function get_students_list() {
		$query = 'SELECT `id`,`name`,`class_name` FROM `students_view` ORDER BY `name`';
		$rows = $this->DB->get_rows( $query );

		return $rows;
	}
}