<?php

namespace TimeTable;

class Shifts {

	protected $Table = "time_table_shifts";
	protected $TableTimeTable = "time_table";
	protected $DB;

	function __construct() {
		global $db;
		$this->DB = $db;
	}

	/**
	 * @author Shakeel Ahmed Siddiqi <shakeel@shakeel.pk>
	 *
	 * @param string $args
	 *
	 * @return array
	 *
	 * Get shifts
	 */
	function get_shifts( $args = "" ) {
		$default = [
			"id"       => "",
			"page"     => "1",
			"pagesize" => "10",
			"orderby"  => "`sort`",
			"order"    => "ASC",

			"teacher_id" => "0",
			"mysql_cols" => "*"
		];
		$args    = set_args( $args, $default );
		extract( $args );

		$Q = "SELECT {$mysql_cols} FROM `{$this->Table}` WHERE 1";

		if ( ! empty( $id ) ) {
			$id = \Str::make_array( $id );
			$Q  .= " AND `id` IN (" . implode( ",", $id ) . ")";
		}

		if ( ! empty( $teacher_id ) ) {
			$teacher_id = \Str::make_array( $teacher_id );
			$Q          .= " AND `teacher_id` IN ('" . implode( "','", $teacher_id ) . "')";
		}

		$Q .= " ORDER BY $orderby $order";

		$rows = $this->DB->get_rows2( $Q, $page, $pagesize );

		return $rows;
	}

	/**
	 * @author Shakeel Ahmed Siddiqi <shakeel@shakeel.pk>
	 *
	 * @param string $args
	 *
	 * @return string
	 *
	 * Saving shift
	 */
	function save_shift( $args = "" ) {
		$default = [
			"id"         => "",
			"shift_name" => "",
			"start_time" => "",
			"end_time"   => "",
		];
		$args    = set_args( $args, $default );

		if ( empty( $args["shift_name"] ) ) {
			return __( "Shift name missing." );
		}
		if ( empty( $args["start_time"] ) || empty( $args["end_time"] ) ) {
			return __( "Shift times are missing." );
		}

		$Q = $this->DB->prepare( "SELECT COUNT(*) FROM `{$this->Table}` WHERE `shift_name`=%s AND `id`<>'{$args["id"]}'", \Str::xtrim( $args["shift_name"] ) );
		if ( $this->DB->get_var( $Q ) > 0 ) {
			return __( "Please type different shift name" );
		}

		if ( \Dates::time_to_sec( $args["end_time"] ) < \Dates::time_to_sec( $args["start_time"] ) ) {
			return __( "Start time should be earlier then end time" );
		}

		if ( empty( $args["id"] ) ) {
			$args["sort"] = 100;
			$response     = $this->DB->insert( $this->Table, $args );
		} else {
			$response = $this->DB->update( $this->Table, $args, [ "id" => $args["id"] ] );
		}

		if ( $response === false ) {
			return $this->DB->last_error;
		}

		return "OK";
	}

	/**
	 * @author Shakeel Ahmed Siddiqi <shakeel@shakeel.pk>
	 *
	 * @param $id
	 *
	 * @return string
	 *
	 * Delete shifts.
	 */
	function delete_shift( $id ) {
		$response = $this->DB->delete( $this->Table, [ "id" => $id ] );

		if ( $response === false ) {
			return $this->DB->last_error;
		}

		return "OK";
	}

	function save_sort_shifts( $ids ) {
		foreach ( $ids as $sort => $id ) {
			$this->DB->update( $this->Table, [ "sort" => $sort ], [ "id" => $id ] );
		}
	}

	public function count_shifts() {
		return $this->DB->get_var( "SELECT COUNT(*) FROM `{$this->Table}`" );
	}

	public function get_shifts_by_teacher($teacher_id) {
		$Q = "SELECT s.* FROM `{$this->Table}` s, `{$this->TableTimeTable}` t WHERE
			t.`teacher_id`='{$teacher_id}' AND t.`shift_id`=s.`id` GROUP BY s.`id`";

		$rows = $this->DB->get_rows($Q);
		return $rows;
	}
}