<?php

class Translation {

	var $Table = "translations";
	const Table = "translations";
	var $TableLangs = "langs";
	const TableLangs = "langs";
	var $DB;

	## https://en.wikipedia.org/wiki/List_of_language_names
	var $Langs = [
		"us" => [ 'label' => 'English US', 'flag' => 'us.png', 'is_rtl' => false ],
		"ur" => [
			'label'  => "اردو",
			'flag'   => 'pk.png',
			'is_rtl' => true,
		],
		"ar" => [
			'label'  => "العربية",
			'flag'   => 'sa.png',
			'is_rtl' => true,
		],
		"du" => [
			'label'  => "Dutch",
			'flag'   => 'nl.png',
			'is_rtl' => false,
		],
		"fr" => [
			'label'  => "French",
			'flag'   => 'fr.png',
			'is_rtl' => false,
		],
		"es" => [
			'label'  => "Spanish",
			'flag'   => 'es.png',
			'is_rtl' => false,
		],
		"ua" => [
			'label'  => "Ukranian",
			'flag'   => 'ua.png',
			'is_rtl' => false,
		],
		"de" => [
			'label'  => "German",
			'flag'   => 'de.png',
			'is_rtl' => false,
		],
		"el" => [
			'label'  => "Greek",
			'flag'   => 'eu.png',
			'is_rtl' => false,
		],
		"pt" => [
			'label'  => "Portuguese",
			'flag'   => 'pt.png',
			'is_rtl' => false,
		],
		"dk" => [
			'label'  => "Danish",
			'flag'   => 'dk.png',
			'is_rtl' => false,
		],
		"ru" => [
			'label'  => "Russian",
			'flag'   => 'ru.png',
			'is_rtl' => false,
		],
		"tr" => [
			'label'  => "Turkish",
			'flag'   => 'tr.png',
			'is_rtl' => false,
		],
		"ms" => [
			'label'  => "Malay",
			'flag'   => 'my.png',
			'is_rtl' => false,
		],

//Malay
	];

	var $FilesPath;
	var $DefaultLanguage = 'us';

	function __construct() {
		$this->FilesPath = ASSETS_PATH . 'translations/source_files/';

		global $db;
		$this->DB = $db;
	}

	/**
	 * @param string $lang
	 *
	 * @return string
	 *
	 * This returns path of .po file
	 */
	public function get_translation_file( $lang = '' ) {
		$lang = (string) $lang;
		if ( $lang == '' ) {
			$lang = $this->get_language();
		}

		$path = ROOT_PATH . 'contents/langs/' . $lang . '.po';
		if ( ! is_file( $path ) ) {
			$path = $this->FilesPath . $lang . '.po';
		}

		return $path;
	}

	public function is_rtl() {
		$lang = $this->get_language();

		return $this->Langs[ $lang ]['is_rtl'];
	}

	/**
	 * @param string $include_default
	 *
	 * @return array
	 *
	 * This method will return array of all available languages...
	 * $include_default=1 > will return only default language
	 * $include_default=0 > will return all languages except default language
	 * Blank $include_default will return all languages
	 */
	public function get_languages( $include_default = '' ) {
		$langs        = [];
		$current_lang = $this->get_language();


		if ( $include_default === 1 || $include_default === '' ) {
			foreach ( $this->Langs as $lang => $lang_array ) {
				$lang_array['img']       = DOMAIN_URL . 'assets/img/flags/' . $lang . '.png';
				$lang_array['lang_file'] = $this->get_translation_file( $lang );
				if ( $lang == $current_lang ) {
					$langs[ $lang ] = $lang_array;
				}
			}
		}
		if ( $include_default === 0 || $include_default === '' ) {
			foreach ( $this->Langs as $lang => $lang_array ) {
				$lang_array['img']       = DOMAIN_URL . 'assets/img/flags/' . $lang . '.png';
				$lang_array['lang_file'] = $this->get_translation_file( $lang );
				if ( $lang <> $current_lang ) {
					$langs[ $lang ] = $lang_array;
				}
			}
		}

		return $langs;
	}

	public function set_language( $lang ) {
		$langs = array_keys( $this->get_languages() );

		if ( ! in_array( $lang, $langs ) ) {
			return false;
		}

		$_SESSION['lang'] = $lang;
		if ( \Admins\UserInfo::is_loggedin() ) {
			global $db;
			$db->set_option( 'lang_' . \Admins\UserInfo::get_my_id(), $lang );
		}

		return true;
	}

	public function get_language() {
		if ( isset( $_SESSION['lang'] ) ) {
			return $_SESSION['lang'];
		}

		if ( ! \Admins\UserInfo::is_loggedin() ) {
			return $this->DefaultLanguage;
		}

		global $db;
		$user_lang = (string) $db->get_option( 'lang_' . \Admins\UserInfo::get_my_id() );
		if ( $user_lang == '' ) {
			$_SESSION['lang'] = $this->DefaultLanguage;

			return $this->DefaultLanguage;
		} else {
			$_SESSION['lang'] = $user_lang;

			return $user_lang;
		}
	}

	public function get( $string ) {
		$string = (string) $string;

		$lang = isset( $_SESSION["lang"] ) ? $_SESSION["lang"] : "en";
		$Q    = "SELECT `msgval` FROM `$this->Table` WHERE `msgstr`='" . $this->DB->_escape( $string ) . "' AND `lang`='$lang'";

		$msgval = $this->DB->get_var( $Q );

		self::write_translation_file();

		if ( empty( $msgval ) ) {
			$data = [
				"msgstr" => $string,
				"msgval" => $string,
				"lang"   => $lang,
			];

			//$this->DB->insert( $this->Table, $data );

			return $string;
		} else {
			return $msgval;
		}
	}

	/**
	 * @return string
	 *
	 * Get default translations.
	 */
	function get_default_language() {
		return $this->DefaultLanguage;
		//		$Q = "SELECT `code` FROM `$this->TableLangs` WHERE `active`='1' AND `default`='1'";
		//
		//		return $this->DB->get_var( $Q );
	}

	public function get_translations( $args = "" ) {
		$default = [
			"lang"     => "",
			"orderby"  => "msgstr",
			"order"    => "ASC",
			"page"     => "1",
			"pagesize" => "20",
			"search"   => "",
		];
		$args    = set_args( $args, $default );
		extract( $args );

		if ( empty( $lang ) ) {
			$lang = $this->get_default_language();
		}

		$Q = "SELECT * FROM `$this->Table` WHERE 1";
		$Q .= " AND `lang`='$lang'";

		if ( trim( $search ) <> "" ) {
			$search = $this->DB->_escape( $search );
			$Q      .= " AND (`msgstr` LIKE '%{$search}%' OR `msgval` LIKE '%{$search}%')";
		}

		$Q .= " ORDER BY `$orderby` $order";

		$rows = $this->DB->get_rows2( $Q, $page, $pagesize );

		return $rows;
	}

	function update_translation( $args = "" ) {
		$default = [
			"id"     => "",
			"msgval" => "",
		];
		$args    = set_args( $args, $default );

		if ( empty( $args["msgval"] ) ) {
			return "New value missing.";
		}
		$response = $this->DB->update( $this->Table, [ "msgval" => $args["msgval"] ], [ "id" => $args["id"] ] );
		if ( $response === false ) {
			return $this->DB->last_error;
		}

		self::write_translation_file();

		return "OK";
	}

	public static function translations_table_array() {
		if ( ! file_exists( ROOT_PATH . 'contents/t.ini' ) ) {
			return [];
		}

		$content = file_get_contents( ROOT_PATH . 'contents/t.ini' );

		if ( $content <> '' ) {
			$content = json_decode( $content, true );
		}

		if ( ! is_array( $content ) ) {
			return [];
		}

		return $content;
	}

	public static function write_translation_file() {
		$file = ROOT_PATH . 'contents/t.ini';

		global $db;
		$Q    = 'SELECT * FROM `' . self::Table . '`';
		$rows = $db->get_rows( $Q );

		$array = [];
		foreach ( $rows as $row ) {
			$array[ $row['msgstr'] ] = $row['msgval'];
		}

		@file_put_contents( $file, json_encode( $array, JSON_PRETTY_PRINT ) );
	}
}