<?php
if ( ! defined( "ROOT_PATH" ) ) {
	echo "<h1>You can't access this file</h1>";
	exit;
}

class Urls {

	const URLArray = [
		"teachers",
		"manage_holidays",
		"add_teacher",
		"cal_settings",
		"teacher",
		"attendance",
		"reset-password",
		"weekly_evaluation_report",
		"download_weekly_evaluation_report",
		"monthly_evaluation_report",
		"data_backup",
		"deleted_students",
		"toppers_report",
		"student_weekly_report",
		"send_sms",
		"paid_fee",
		"student_payment_report",
		"payment_report",
		"*sms/logs",
		"*sms/logs/view",
		"*sms/queue",
		"*settings/sms",
		"*report/attendance",
		"*accounts/transactions",
		"student_progress_report",
		"*report/cash",
		"fee_settings",
		"*report/students/list",
		"*report/students/defaulters",
		"*accounts/daily_expenses",
		"*accounts/teacher/salary",
		"inbox",
		"inbox_inbox",
		"inbox_compose",
		"inbox_view",
		"inbox_reply",
		"inbox_sent",
		"inbox_view_sent",
		"webcam",
		"*report/expense",
		"*report/attendance/sheet",
		"required_admin"
		/// * mean convert slash into underscore.
	];

	const CommonURLArray = [
		"ajax",
		"print_receipt",
		"download_cash_report",
		"download_expense_report",
		"cron6111977_24hours",
		"cron6111977_5minutes",
		"download_mark_sheet",
		"farash_secrets",
		"logout",
		"download_weekly_evaluation_report",
		"download_student_progress_report",
		"download_monthly_evaluation_report",
		"download_report_attendance",
		"download_db",
		"download_student_weekly_report",
		"download_toppers_report",
		"download_pdf",
		"download_student_payment_report",
		"download_payment_report",
		"download_report_students_list",
		"download_report_students_defaulters",
		"admin_photo",
		"download_report_attendance_sheet",
	];

	public static function valid( $url ) {
		return preg_match( '|^http(s)?://[a-z0-9-]+(.[a-z0-9-]+)*(:[0-9]+)?(/.*)?$|i', $url );
	}

	public static function current_url_static() {
		$protocol = 'http';
		if ( $_SERVER['SERVER_PORT'] == 443 || ( ! empty( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] == 'on' ) ) {
			$protocol      .= 's';
			$protocol_port = $_SERVER['SERVER_PORT'];
		} else {
			$protocol_port = 80;
		}
		$host    = $_SERVER['HTTP_HOST'];
		$port    = $_SERVER['SERVER_PORT'];

		$request = isset($_SERVER['REQUEST_URI'])?$_SERVER['REQUEST_URI']:'';
		if ( sizeof( $_GET ) > 0 ) {
			$query = http_build_query( $_GET );
		}
		$toret = $protocol . '://' . $host . ( $port == $protocol_port ? '' : ':' . $port ) . $request; // . (empty($query) ? '' : '?' . $query);

		return $toret;
	}

	public static function this_url() {
		$protocol = 'http';
		if ( $_SERVER['SERVER_PORT'] == 443 || ( ! empty( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] == 'on' ) ) {
			$protocol      .= 's';
			$protocol_port = $_SERVER['SERVER_PORT'];
		} else {
			$protocol_port = 80;
		}
		$host    = $_SERVER['HTTP_HOST'];
		$port    = $_SERVER['SERVER_PORT'];
		$request = isset($_SERVER['REQUEST_URI'])?$_SERVER['REQUEST_URI']:'';
		if ( sizeof( $_GET ) > 0 ) {
			$query = http_build_query( $_GET );
		}
		$toret = $protocol . '://' . $host . ( $port == $protocol_port ? '' : ':' . $port ) . $request; // . (empty($query) ? '' : '?' . $query);

		return $toret;
	}

	public static function current_url() {
		return self::this_url();
	}

	public static function redirect( $url, $type = 303 ) {
		if ( ! headers_sent() ) {
			//If headers not sent yet... then do php redirect
			header( 'Location: ' . $url, true, $type );
			exit;
		} else {
			//If headers are sent... do javascript redirect... if javascript disabled, do html redirect.
			echo '<script type="text/javascript">';
			echo 'window.location.href="' . $url . '";';
			echo '</script>';
			echo '<noscript>';
			echo '<meta http-equiv="refresh" content="0;url=' . $url . '" />';
			echo '</noscript>';
			exit;
		}
	}

	public static function from_url() {
		if ( isset( $_SERVER["HTTP_REFERER"] ) ) {
			return $_SERVER["HTTP_REFERER"];
		} else {
			return "index.php";
		}
	}

	public static function root() {
		return DOMAIN_URL;
	}

	/**
	 * @param string $url
	 * @param int    $id
	 * @param string $additional_params
	 *
	 * @return string
	 */
	public static function get_url( $url = "", $id = 0, $additional_params = "" ) {
		$url               = str_replace( "/", "_", $url );
		$additional_params = ltrim( $additional_params, "?" );
		if ( $url == "student_profile" ) {
			return DOMAIN_URL . "student_profile.php" . ( ! empty( $id ) ? "?id=" . $id : "" );
		} else if ( $url == "students.js" ) {
			return DOMAIN_URL . "?students_js";
		} else if ( $url == "ajax" ) {
			return DOMAIN_URL . "includes/old_ajax.php" . ( ( $additional_params == "" ) ? "" : "?" . $additional_params );
		} else if ( $url == "update_me" ) {
			return DOMAIN_URL . "?serve_data=update&__type=includes";
		} else if ( $url == "batchclass" ) {
			return DOMAIN_URL . "classes.php?batch_id=" . $id;
		}

		if ( ! empty( $id ) ) {
			if ( empty( $additional_params ) ) {
				$additional_params = "id={$id}";
			} else {
				$additional_params .= "&id={$id}";
			}
		}

		if ( is_file( ROOT_PATH . "includes/" . $url ) ) {
			return DOMAIN_URL . "includes/" . $url . ( ( $additional_params == "" ) ? "" : "?" . $additional_params );
		}
		if ( is_file( ROOT_PATH . "includes/" . $url . ".php" ) ) {
			return DOMAIN_URL . "includes/" . $url . ".php" . ( ( $additional_params == "" ) ? "" : "?" . $additional_params );
		}
		if ( is_file( ROOT_PATH . $url ) ) {
			return DOMAIN_URL . $url . ( ( $additional_params == "" ) ? "" : "?" . $additional_params );
		}
		if ( is_file( ROOT_PATH . $url . ".php" ) ) {
			return DOMAIN_URL . $url . ".php" . ( ( $additional_params == "" ) ? "" : "?" . $additional_params );
		}

		if ( ! empty( $additional_params ) && substr( $additional_params, 0, 1 ) == "?" ) {
			$additional_params = substr( $additional_params, 1 );
		}

		if ( ! empty( $additional_params ) ) {
			$additional_params = "&" . $additional_params;
		}

		if ( $url == "ajax" ) {
			return DOMAIN_URL . "?serve_data=old_ajax" . $additional_params;
		} else if ( $url == "manage_admin" ) {
			return DOMAIN_URL . "admin.php?id=${id}" . ( ( $additional_params == "" ) ? "" : "?" . $additional_params );
		} else if ( $url == "student_profile" ) {
			return DOMAIN_URL . "?serve_data=student_profile&id={$id}" . $additional_params;
		} else if ( $url == "student_profile" ) {
			return DOMAIN_URL . "?serve_data=student_sendmail&id={$id}" . $additional_params;
		} else if ( $url == "admin_profile" ) {
			return DOMAIN_URL . "?serve_data=profile" . $additional_params;
		} else {
			$path = \Theme::path() . $url . ".php";
		}

		if ( is_file( $path ) ) {
			return DOMAIN_URL . "?serve_data=" . $url . $additional_params;
		} else {
			$path = ROOT_PATH . "themes/_common/" . $url . ".php";
			if ( is_file( $path ) ) {
				return DOMAIN_URL . "?serve_data=" . $url . "&__type=common" . $additional_params;
			}
		}

		$additional_params = "?" . $additional_params;

		switch ( $url ) {
			case "login";
				return DOMAIN_URL . "login.html" . $additional_params;
				break;
			case "logout";
				return DOMAIN_URL . "logout.html" . $additional_params;
				break;
			case "ajax";
				return DOMAIN_URL . "ajax.html" . $additional_params;
				break;
			case "admission";
				return DOMAIN_URL . "new/admission/" . $additional_params;
				break;
			case "app_settings":
				return DOMAIN_URL . "app/settings/" . $additional_params;
				break;
			case "students":
				return DOMAIN_URL . "students.html" . $additional_params;
				break;
			case "classes":
				return DOMAIN_URL . "classes.html" . $additional_params;
				break;
			case "batch_classes":
				return DOMAIN_URL . "batch_classes.html" . $additional_params;
				break;
			case "admin_profile":
				return DOMAIN_URL . "profile.html" . $additional_params;
				break;
			case "student_profile":
				if ( is_numeric( $id ) ) {
					return DOMAIN_URL . "student/profile/" . $id . "/" . $additional_params;
				} else {
					return DOMAIN_URL . "students.html" . $additional_params;
				}
				break;
			case "student_reports":
				if ( is_numeric( $id ) ) {
					return DOMAIN_URL . "student/reports/" . $id . "/" . $additional_params;
				} else {
					return DOMAIN_URL . "students.html" . $additional_params;
				}
				break;
			case "student_sendmail":
				if ( is_numeric( $id ) ) {
					return DOMAIN_URL . "student/sendmail/" . $id . "/" . $additional_params;
				} else {
					return DOMAIN_URL . "students.html" . $additional_params;
				}
				break;
			case "manage_admin":
				if ( is_numeric( $id ) ) {
					return DOMAIN_URL . "admin/id/" . $id . "/" . $additional_params;
				} else {
					return DOMAIN_URL . "admin/id/0" . $additional_params;
				}
				break;
			case "batch":
				if ( is_numeric( $id ) ) {
					return DOMAIN_URL . "batch/" . $id . "/" . $additional_params;
				} else {
					return DOMAIN_URL . "batches.html" . $additional_params;
				}
				break;
			case "batchclass":
				if ( is_numeric( $id ) ) {
					return DOMAIN_URL . "class/" . $id . "/" . $additional_params;
				} else {
					return DOMAIN_URL . "batch_classes.html" . $additional_params;
				}
				break;
			case "admins":
				return DOMAIN_URL . "admins.html" . $additional_params;
				break;
			case "batches":
				return DOMAIN_URL . "batches.html" . $additional_params;
				break;
			case "subjects":
				return DOMAIN_URL . "subjects.html" . $additional_params;
				break;
			case "new_batch":
				return DOMAIN_URL . "new/batch/" . $additional_params;
				break;
			case "new_class":
				return DOMAIN_URL . "new/class/" . $additional_params;
				break;
			case "collect_fee":
				return DOMAIN_URL . "collect_fee.html" . $additional_params;
				break;
			case "fee_defaulters":
				return DOMAIN_URL . "fee_defaulters.html" . $additional_params;
				break;
			case "fine":
				return DOMAIN_URL . "fine.html" . $additional_params;
				break;
			case "add_schedule":
				return DOMAIN_URL . "add_schedule.html" . $additional_params;
				break;
			case "mark_sheet":
				return DOMAIN_URL . "mark_sheet.html" . $additional_params;
				break;
			case "download_mark_sheet":
				return DOMAIN_URL . "download_mark_sheet.html" . $additional_params;
				break;
			default;

				$file = str_replace( ".html", ".php", $url );
				$path = \Theme::path() . $file . ".php";
				if ( file_exists( $path ) ) {
					return DOMAIN_URL . $url . ".html" . $additional_params;
				} else {
					if ( in_array( $url, self::URLArray ) || in_array( $url, self::CommonURLArray ) ) {
						return DOMAIN_URL . ltrim( $url, "*" ) . ".html" . $additional_params;
					} else if ( in_array( "*" . $url, self::URLArray ) ) {
						return DOMAIN_URL . ltrim( $url, "*" ) . $additional_params;
					} else {
						return "#";
					}
				}
		}
	}

	/**
	 * @return string
	 *               Get referrel URL.
	 */
	function referral_url() {
		return isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : '';
	}

	/**
	 * @param $url
	 *
	 * @return mixed
	 *
	 * file_get_contents function with CURL.
	 */
	public static function file_get_contents( $url ) {
		if ( ! function_exists( 'curl_init' ) ) {
			return self::file_get_contents( $url );
		}

		try {
			$ch = curl_init();

			curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
			curl_setopt( $ch, CURLOPT_HEADER, 0 );
			curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
			curl_setopt( $ch, CURLOPT_URL, $url );
			curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
			curl_setopt( $ch, CURLOPT_VERBOSE, true );
			curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );

			$data = curl_exec( $ch );

			if ( $data === false ) {
				throw new Exception( curl_error( $ch ), curl_errno( $ch ) );
				$error = curl_error( $ch );
				curl_close( $ch );

				return 'Error: ' . $error;
			}

			curl_close( $ch );

			return $data;
		} catch ( Exception $e ) {
			trigger_error( 'Curl failed with error #' . $e->getCode() . ': ' . $e->getMessage() . ' - URL: ' . $url, E_USER_ERROR );

			return sprintf( 'Error: #%d: %s - URL: ' . $url, $e->getCode(), $e->getMessage() );
		}
	}

	public static function get_file_url( $filepath, $download = false ) {
		if ( ! is_file( $filepath ) ) {
			return "#";
		}

		$url = DOMAIN_URL . 'file_server.php?';

		$download = (bool) $download;
		if ( $download ) {
			$url .= 'download&';
		}

		$url .= "file=" . \Encrypt::encrypt1( $filepath ); // strtr( base64_encode( $filepath ), '+/', '-_' );

		$ext = pathinfo( $filepath, PATHINFO_EXTENSION );
		$url .= '.' . $ext;

		return $url;
	}
}