<?php



namespace DB;



class Select {



	protected $DB;



	protected $select_from;

	protected $select_cols = '*';

	protected $where = [];

	protected $where_operator = 'AND';

	protected $order_by = [];

	protected $order = [];

	protected $group_by = '';

	protected $join = '';



	public $last_query;



	protected $force_operator = null;



	public function __construct() {

		global $DB;

		$this->DB = $DB;

	}



	/**

	 * Clear all options.

	 */

	public function clear() {

		$this->select_cols    = '*';

		$this->select_from    = '';

		$this->where          = [];

		$this->where_operator = 'AND';

		$this->order          = [];

		$this->order_by       = [];

		$this->group_by       = '';

		$this->join           = '';

	}



	/**

	 * @param $table_name

	 *

	 * Select from table.

	 */

	function select_from( $table_name ) {

		$this->select_from = $this->DB->prepare( '%t', $table_name );



		return $this;

	}



	/**

	 * @param string $cols

	 *

	 * This method is optional, By default * will be selected.

	 */

	function select_cols( $cols = '*' ) {

		$this->select_cols = $cols;



		return $this;

	}



	/**

	 * @param $where_condition

	 *

	 * Set where operator.

	 */

	function where( $where_condition ) {

		global $DB;



		if ( ! is_null( $this->force_operator ) ) {

			$op = $this->force_operator;

		} else {

			$op = $this->where_operator;

		}



		$args = func_get_args();



		if ( func_num_args() == 2 && is_string( $args[0] ) ) {

			$newargs[0] = [ $args[0] => $args[1] ];

			$args       = $newargs;

		}



		foreach ( $args as $where_condition ) {

			if ( is_string( $where_condition ) ) {

				$this->where[] = $op . ' ' . $where_condition;

			} else if ( is_array( $where_condition ) ) {

				foreach ( $where_condition as $column => $value ) {

					$column          = trim( (string) $column );

					$column_operator = '=';

					if ( substr( $column, - 2 ) == '!=' ) {

						$column          = '`' . substr( $column, 0, - 2 ) . '`';

						$column_operator = '!=';

					} else if ( substr( $column, - 2 ) == '<>' ) {

						$column          = '`' . substr( $column, 0, - 2 ) . '`';

						$column_operator = '<>';

					} else if ( substr( $column, - 1 ) == '>' ) {

						$column          = '`' . substr( $column, 0, - 1 ) . '`';

						$column_operator = '>';

					} else if ( substr( $column, - 1 ) == '<' ) {

						$column          = '`' . substr( $column, 0, - 1 ) . '`';

						$column_operator = '<';

					}



					if ( is_int( $value ) ) {

						$this->where[] = $DB->prepare( '%w %w %w %d', $op, $column, $column_operator, $value );

					} else if ( is_float( $value ) ) {

						$this->where[] = $DB->prepare( '%w %w %w %f', $op, $column, $column_operator, $value );

					} else if ( is_array( $value ) ) {

						//$value         = json_encode( $value );

						$this->where[] = $DB->prepare( '%w %w IN (\'', $op, $column ) . implode( "','", $value ) . "')";

					} else if ( is_object( $value ) ) {

						$value         = json_encode( $value );

						$this->where[] = $DB->prepare( '%w %w %w %s', $op, $column, $column_operator, $value );

					} else if ( is_bool( $value ) ) {

						$value         = $value ? 'TRUE' : 'FALSE';

						$this->where[] = $DB->prepare( '%w %w %w %w', $op, $column, $column_operator, $value );

					} else if ( is_null( $value ) ) {

						$value         = 'NULL';

						$this->where[] = $DB->prepare( '%w %w %w %w', $op, $column, $column_operator, $value );

					} else if ( \DateTimeBindia::is_datetime( $value ) ) {

						$value         = \DateTimeBindia::mysql_datetime( $value );

						$this->where[] = $DB->prepare( '%w %w %w %s', $op, $column, $column_operator, $value );

					} else if ( \DateTimeBindia::is_date( $value ) ) {

						$value         = \DateTimeBindia::mysql_date( $value );

						$this->where[] = $DB->prepare( '%w %w %w %s', $op, $column, $column_operator, $value );

					} else {

						if ( substr( $value, 0, 1 ) == '%' && substr( $value, - 1 ) == '%' ) {

							$this->where[] = $DB->prepare( '%w %w LIKE %s', $op, $column, '%' . substr( $value, 1, - 1 ) . '%' );

						} else if ( substr( $value, 0, 1 ) == '%' ) {

							$this->where[] = $DB->prepare( '%w %w LIKE %s', $op, $column, '%' . substr( $value, 1 ) );

						} else if ( substr( $value, - 1 ) == '%' ) {

							$this->where[] = $DB->prepare( '%w %w LIKE %s', $op, $column, substr( $value, 0, - 1 ) . '%' );

						} else {

							$this->where[] = $DB->prepare( '%w %w %w %s', $op, $column, $column_operator, $value );

						}

					}



				}

			}

		}



		$this->force_operator = null;

		return $this;

	}



	/**

	 * @param $where_condition

	 *

	 * Set where condition with AND operator.

	 */

	function where_and( $where_condition ) {

		$this->force_operator = 'AND';

		$this->where( $where_condition );



		return $this;

	}



	/**

	 * @param $where_condition

	 *

	 * Set where condition with OR operator.

	 */

	function where_or( $where_condition ) {

		$this->force_operator = 'OR';

		$this->where( $where_condition );



		return $this;

	}



	/**

	 * @param string $operator

	 *

	 * Set default operator for where clause.

	 */

	function where_operator( $operator = 'AND' ) {

		$this->where_operator = strtoupper( trim( $operator ) );



		return $this;

	}



	/**

	 * @param string $output_type

	 *

	 * @return mixed

	 *

	 * Get single row from MySQL table

	 */

	public function get_row( $output_type = ARRAY_A ) {

		# You can select OBJECT_K for object output



		$query = $this->generate_query();



		$row = $this->DB->get_row( $query, $output_type );



		$this->clear();



		return $row;

	}



	/**

	 * @return mixed

	 *

	 * Return single value from MySQL

	 */

	public function get_var() {

		$query = $this->generate_query();



		$var = $this->DB->get_var( $query );



		$this->clear();



		return $var;

	}



	/**

	 * @param string $output_type

	 *

	 * @return mixed

	 *

	 * Get databases row according to conditions.

	 */

	public function get_rows( $output_type = ARRAY_A ) {

		# You can select OBJECT_K for object output



		$query = $this->generate_query();



		$rows = $this->DB->get_rows( $query, $output_type );

		$this->clear();



		return $rows;

	}



	/**

	 * @param int    $page

	 * @param int    $page_size

	 * @param int    $range

	 * @param string $output_type

	 *

	 * @return mixed

	 *

	 * Get table result with pagination.

	 */

	public function get_rows2( $page = 1, $page_size = 10, $range = 3, $output_type = ARRAY_A ) {

		# You can select OBJECT_K for object output



		$query = $this->generate_query();



		$rows = $this->DB->get_rows2( $query, $page, $page_size, $range, $output_type );



		$this->clear();



		return $rows;

	}



	/**

	 * Generate query according to where clause etc.

	 */

	private function generate_query() {

		$query = 'SELECT ' . $this->select_cols . ' FROM ' . $this->select_from;

		if ( ! empty( $this->join ) ) {

			$query .= ' JOIN ' . $this->join[0] . ' ON ' . $this->join[1];

		}

		$query         .= ' WHERE 1';

		$with_operator = true;



		if ( ! empty( $this->where ) ) {

			foreach ( $this->where as $key => $query_part ) {

				if ( substr( $query_part, 0, 3 ) == 'OR ' ) {

					$op         = 'OR';

					$query_part = substr( $query_part, 3 );

				} else if ( substr( $query_part, 0, 4 ) == 'AND ' ) {

					$op         = 'AND';

					$query_part = substr( $query_part, 4 );

				} else {

					$op = $this->where_operator;

				}



				if ( $key == 0 ) {

					$op = 'AND';

				}



				if ( $query_part == '(' ) {

					$query         .= ' ' . $op . ' (';

					$with_operator = false;

				} else if ( $query_part == ')' ) {

					$query .= ')';

				} else if ( $with_operator ) {

					$query .= ' ' . $op . ' ' . $query_part;

				} else {

					if ( substr( $query_part, 0, 3 ) == 'OR ' ) {

						$query_part = substr( $query_part, 3 );

					} else if ( substr( $query_part, 0, 4 ) == 'AND ' ) {

						$query_part = substr( $query_part, 4 );

					}



					$query         .= $query_part;

					$with_operator = true;

				}

			}

		}



		$this->group_by = trim( (string) $this->group_by );

		if ( $this->group_by <> '' ) {

			$query .= ' GROUP BY ' . $this->group_by;

		}



		if ( ! empty( $this->order_by ) ) {

			foreach ( $this->order_by as $key => $order_by ) {

				if ( $key == 0 ) {

					$query .= ' ORDER BY';

				}

				if ( $key > 0 ) {

					$query .= ',';

				}

				$query .= ' ' . $order_by . ' ' . $this->order[ $key ];

			}

		}



		$this->last_query = $query;



		return $query;

	}



	/**

	 * Set start group in query ( in query

	 */

	public function start_group() {

		$this->where[] = '(';



		return $this;

	}



	/**

	 * Set end group in query ) in query.

	 */

	public function end_group() {

		$this->where[] = ')';



		return $this;

	}



	/**

	 * @param        $order_by

	 * @param string $order

	 *

	 * Set order by column and order

	 */

	public function order( $order_by, $order = 'asc' ) {

		$this->order_by[] = $order_by;

		$this->order[]    = strtoupper( $order );



		return $this;

	}



	/**

	 * @param $table_name

	 *

	 * @return mixed

	 * This method will return whole MySQL table.

	 */

	public function get_table( $table_name ) {

		$query = 'SELECT ' . $this->select_cols . ' FROM `' . $table_name . '`';



		$rows = $this->DB->get_rows( $query );

		$this->clear();



		return $rows;

	}



	/**

	 * @return mixed

	 *

	 * Count result after

	 */

	public function count_result() {

		$this->select_cols = 'COUNT(*)';

		$query             = $this->generate_query();



		$this->last_query = $query;

		$count            = $this->DB->get_var( $query );

		$this->clear();



		return $count;

	}



	/**

	 * @param $column

	 *

	 * Set group by column name

	 */

	public function group_by( $column ) {

		$this->group_by = $column;



		return $this;

	}



	/**

	 * @param $table_name

	 * @param $join_condition

	 *

	 * Set join table with condition.

	 */

	public function join( $table_name, $join_condition ) {

		$this->join = [ $table_name, $join_condition ];



		return $this;

	}

}