<?php



use Brick\PhoneNumber\PhoneNumber;

use Brick\PhoneNumber\PhoneNumberParseException;



function my_url() {

	return "http" . ( ( ! empty( $_SERVER['HTTPS'] ) ) ? "s" : "" ) . "://" . $_SERVER['SERVER_NAME'] . ( isset( $_SERVER['SERVER_PORT'] ) ? ':' . $_SERVER['SERVER_PORT'] : '' ) . $_SERVER['REQUEST_URI'];

}



function redirect( $url, $type = 303 ) {

	if ( ! headers_sent() ) {

		//If headers not sent yet... then do php redirect

		header( 'Location: ' . $url, true, $type );

		exit();

	} else {

		//If headers are sent... do javascript redirect... if javascript disabled, do html redirect.

		echo '<script type="text/javascript">';

		echo 'window.location.href="' . $url . '";';

		echo '</script>';

		echo '<noscript>';

		echo '<meta http-equiv="refresh" content="0;url=' . $url . '" />';

		echo '</noscript>';

		exit();

	}

}



function filename( $full = false ) {

	$bt   = debug_backtrace();

	$file = $bt[0]["file"];

	if ( ! $full ) {

		return basename( $file );

	} else {

		return $file;

	}

}



function xtrim( $string ) {

	$string = trim( $string, " \t\n\r\0\x0Bÿ" );



	return $string;

}



function is_email( $email ) {

	return filter_var( $email, FILTER_VALIDATE_EMAIL );

}



function get_row( $query ) {

	global $DB;



	return $DB->get_row( $query );

}



function get_rows( $query ) {

	global $DB;

	$rows = $DB->get_rows( $query );



	return $rows;

}



function get_var( $query ) {

	global $DB;



	return $DB->get_var( $query );

}



function get_rows2( $Q, $page = 1, $pageSize = 10, $range = 3, $encode = false ) {

	global $DB;

	$rows = $DB->get_rows2( $Q, $page, $pageSize, $range, ARRAY_A );



	return $rows;

}



function insert( $table_name, $data ) {

	global $DB;

	$response = $DB->insert( $table_name, $data );

	if ( ! $response ) {

		$status = $DB->last_error;

		$new_id = null;

	} else {

		$status = "OK";

		$new_id = $DB->insert_id;

	}



	return array(

		"status" => $status,

		"id"     => $new_id,

	);

}



function update( $table_name, $data, $condition = array() ) {

	global $DB;

	$response = $DB->update( $table_name, $data, $condition );

	if ( $response === false ) {

		$status        = $DB->last_error;

		$affected_rows = null;

	} else {

		$status        = "OK";

		$affected_rows = $DB->rows_affected;

	}



	return array(

		"status"        => $status,

		"affected_rows" => $affected_rows,

	);

}



function set_args( $args, $default ) {

	if ( ! is_array( $args ) ) {

		$args = stripslashes( $args );

		parse_str( $args, $args );

	}

	if ( ! is_array( $args ) ) {

		return array();

	}

	if ( ! is_array( $default ) ) {

		return $default;

	}

	foreach ( $default as $k => $v ) {

		if ( ! isset( $args[ $k ] ) ) {

			$args[ $k ] = $v;

		}

	}

	if ( get_magic_quotes_gpc() ) {

		$args = array_map( "stripslashes", $args );

	}



	return $args;

}



function localhost() {

	return ( $_SERVER["REMOTE_ADDR"] == "::1" || $_SERVER["REMOTE_ADDR"] == "127.0.0.1" || $_SERVER["HTTP_HOST"] == "localhost" || $_SERVER["HTTP_HOST"] == "127.0.0.1" || $_SERVER["HTTP_HOST"] == "shakeel.com" || $_SERVER["HTTP_HOST"] == "fekara.pk" );

}



function myip() {

	if ( getenv( 'HTTP_CLIENT_IP' ) ) {

		$ip = getenv( 'HTTP_CLIENT_IP' );

	} elseif ( getenv( 'HTTP_X_FORWARDED_FOR' ) ) {

		$ip = getenv( 'HTTP_X_FORWARDED_FOR' );

	} elseif ( getenv( 'HTTP_X_FORWARDED' ) ) {

		$ip = getenv( 'HTTP_X_FORWARDED' );

	} elseif ( getenv( 'HTTP_FORWARDED_FOR' ) ) {

		$ip = getenv( 'HTTP_FORWARDED_FOR' );

	} elseif ( getenv( 'HTTP_FORWARDED' ) ) {

		$ip = getenv( 'HTTP_FORWARDED' );

	} else {

		$ip = $_SERVER['REMOTE_ADDR'];

	}



	return $ip;

}



function show_array( $ar, $return = false ) {

	if ( is_null( $ar ) ) {

		$ar = "NULL";

	} elseif ( is_bool( $ar ) && $ar ) {

		$ar = "TRUE";

	} elseif ( is_bool( $ar ) && ! $ar ) {

		$ar = "FALSE";

	}



	if ( $return ) {

		return print_r( $ar, true );

	} else {

		echo "<pre>\n";

		print_r( $ar );

		echo "</pre>\n";

	}

}



function debug( $var ) {

	if ( is_array( $var ) || is_object( $var ) ) {

		$str = print_r( $var, true ) . "\n";

	} elseif ( is_null( $var ) ) {

		$str = "NULL\n";

	} elseif ( is_bool( $var ) ) {

		if ( $var ) {

			$str = "TRUE\n";

		} else {

			$str = "FALSE\n";

		}

	} else {

		$str = $var . "\n";

	}



	if ( ! localhost() ) {

		$bt   = debug_backtrace();

		$file = $bt[0]["file"];

		$line = $bt[0]["line"];



		$str .= "<br><br>debug function in file<br><b>" . $file;

		$str .= "</b><br>Line # <b>" . $line . "</b>";



		$headers = "From: Fekara Debugger <debug@fekara.com>\r\n" . 'Reply-To: shakeel@shakeel.pk' . "\r\n" . 'Content-type: text/html; charset=iso-8859-1' . "\r\n" . 'MIME-Version: 1.0' . "\r\n" . 'X-Mailer: PHP/' . phpversion();



		@mail( "shakeel@shakeel.pk", System . ": Debug", $str, $headers );

	} else {

		file_put_contents( ROOT_PATH . "logs.txt", $str, FILE_APPEND );

	}

}



function my_date( $date = "", $format = "" ) {

	if ( $format == "" ) {

		$format = "d-M-Y";

	}

	if ( $date == "" ) {

		$date = date( "d-M-Y" );

	}



	return date( $format, strtotime( $date ) );

}



function my_datetime( $datetime = "", $format = "" ) {

	if ( $format == "" ) {

		$format = "d-M-Y H:i";

	}

	if ( $datetime == "" ) {

		$datetime = date( "d-M-Y H:i" );

	}



	return date( $format, strtotime( $datetime ) );

}



function is_logged_in() {

	$token = \Josantonius\Cookie\Cookie::get( 'fekaraLoginToken' );

	if ( $token !== false ) {

		$token = (string) $token;

		$token = json_decode( base64_decode( $token ), true );

		if ( $token[1] == 'admins' ) {

			global $DB;

			$_SESSION['fekara_system_a'] = $DB->get_row( "SELECT * FROM `admins` WHERE `id`=" . $token[0] );

		}

	}



	return ( isset( $_SESSION['fekara_system_a'] ) || isset( $_SESSION['fekara_system_i'] ) );

}



/**

 * @return string

 *

 * Get current user's ID. If both Admin/Non Admins are logged in then Admin's ID will return.

 */

function get_my_id(): int {

	if ( ! is_logged_in() ) {

		return 0;

	}



	if ( is_admin() ) {

		return (int) $_SESSION['fekara_system_a']['id'];

	} else {

		return (int) $_SESSION['fekara_system_i']['id'];

	}

}



/**

 * @return bool

 *

 * Check if current user is an administrator?

 */

function is_admin() {

	return ( isset( $_SESSION['fekara_system_a']['id'] ) );

}



function get_admin( $field = "*", $admin_id = 0 ) {

	if ( ! is_admin() ) {

		if ( $field == "*" ) {

			return array();

		} else {

			return "";

		}

	}

	if ( empty( $admin_id ) ) {

		if ( $field == "*" ) {

			return $_SESSION['fekara_system_a'];

		} else {

			return @$_SESSION['fekara_system_a'][ $field ];

		}

	} else {

		$Q   = "SELECT * FROM `admins` WHERE `id`='$admin_id'";

		$row = get_row( $Q );

		if ( $field == "*" ) {

			return $row;

		} else {

			return @$row[ $field ];

		}

	}

}



function get_app( $field = "*", $app_id = 0 ) {

	if ( $app_id > 0 ) {

		$Q   = "SELECT * FROM `apps` WHERE `id`='$app_id'";

		$row = get_row( $Q );

		if ( $field == "*" ) {

			return $row;

		} else {

			return @$row[ $field ];

		}

	}

	if ( ! isset( $_SESSION['fekara_system_i'] ) ) {

		if ( $field == "*" ) {

			return array();

		} else {

			return "";

		}

	}

	if ( empty( $admin_id ) ) {

		if ( $field == "*" ) {

			return $_SESSION['fekara_system_i'];

		} else {

			return @$_SESSION['fekara_system_i'][ $field ];

		}

	} else {

		$Q   = "SELECT * FROM `apps` WHERE `id`='$app_id'";

		$row = get_row( $Q );

		if ( $field == "*" ) {

			return $row;

		} else {

			return @$row[ $field ];

		}

	}

}



function make_pagination_html( $page, $pages, $start, $end ) {

	$html = "<ul class=\"pagination pagination-sm\" style=\"margin: 0\">";



	if ( $page > 1 ) {

		$html .= "<li><a data-page='1' href=\"#\">«</a></li>\n";

		$html .= "<li><a data-page='" . ( $page - 1 ) . "' href=\"#\">‹</a></li>\n";

	}

	if ( $start > 1 ) {

		$html .= "<li><a data-page='1' href=\"#\">1</a></li>\n";

	}



	for ( $x = $start; $x <= $end; $x++ ) {

		if ( $page == $x ) {

			$class = "active";

		} else {

			$class = "";

		}

		$html .= "<li class='$class'><a data-page='$x' href=\"#\">$x</a></li>\n";

	}



	if ( $page < $pages ) {

		$html .= "<li><a data-page='" . ( $page + 1 ) . "' href=\"#\">›</a></li>\n";

		$html .= "<li><a data-page='{$pages}' href=\"#\">»</a></li>\n";

	}

	if ( $end < $pages ) {



	}



	$html .= "</ul>";



	return $html;

}



function get_countries() {

	$Q = "SELECT * FROM `countries` ORDER BY `name`";

	global $DB;



	return $DB->get_rows( $Q );

}



function send_mail( $to, $subject, $message, $attach_files = "", $from_email = "" ) {

	if ( localhost() ) {

		debug( "To: $to, Subject: $subject\nMessage: $message" );

	}

	//	require_once( ROOT_PATH . "library/phpmailer/class.phpmailer.php" );

	//	require_once( ROOT_PATH . "library/phpmailer/class.smtp.php" );



	require_once ROOT_PATH . 'vendor/autoload.php';



	if ( ! is_email( $from_email ) ) {

		$from_email = 'do-not-reply@fekara.com';

	}



	//$subject = "❤ ".$subject;

	global $db;



	//	$transport = (new \Swift_SmtpTransport('smtp.example.org', 25))

	//		->setUsername('your username')

	//		->setPassword('your password')

	//	;



	// Create the Transport

	$transport = ( new \Swift_SmtpTransport() );





	// Create the Mailer using your created Transport

	$mailer = new \Swift_Mailer( $transport );



	$message_obj = ( new \Swift_Message( $subject ) )->setFrom( [ $from_email => 'feKara' ] )

	                                                 ->setTo( $to )

	                                                 ->setBody( $message, 'text/html', 'utf-8' )

	                                                 ->addPart( $message, 'text/plain', 'utf-8' );



	$attach_files = make_array( $attach_files );

	foreach ( $attach_files as $file ) {

		if ( is_file( $file ) ) {

			$message_obj->attach( \Swift_Attachment::fromPath( $file ) );

		}

	}



	// Send the message

	try {

		$mailer->send( $message_obj );

	} catch ( \Swift_TransportException $e ) {

		return $e->getMessage();

	}



	return 'OK';



	//	$mailer           = new PHPMailer;

	//	$mailer->Encoding = '8bit';

	//	//$mailer->CharSet = 'iso-8859-1';

	//	$mailer->CharSet = 'UTF-8';

	//



	//	$mailer->setFrom( $from_email, "feKara" );

	//	$mailer->Sender   = 'do-not-reply@fekara.com';

	//	$mailer->Subject  = $subject;

	//	$mailer->Body     = $message;

	//	$mailer->WordWrap = 0;

	//	//$mailer->Mailer = 'mail';

	//	$mailer->ContentType = 'text/html';

	//	global $db;

	//	$message         .= $db->get_option( 'email_signature' );

	//	$mailer->MsgHTML = $message;

	//

	//	$to = make_array( $to );

	//	foreach ( $to as $t ) {

	//		$mailer->addAddress( $t );

	//	}

	//

	//	global $_smtp;

	//	if ( $_smtp ) {

	//		$mailer->SMTPDebug = 2;

	//	}



	/*$mailer->isSMTP();

	$mailer->SMTPAuth = true;

	//$mailer->SMTPSecure = 'ssl';

	//$mailer->Host = '127.0.0.1:465'; //'109.123.99.87';

	$mailer->Host = '127.0.0.1:587'; //'109.123.99.87';

	$mailer->Username = 'do-not-reply@app.fekara.com';

	$mailer->Password = '9wu3RkSF3cSm';*/



	//$mailer->SMTPSecure = 'tls';

	/*$mailer->SMTPOptions = array(

		'ssl' => array(

			'verify_peer' => false,

			'verify_peer_name' => false,

			'allow_self_signed' => true

		)

	);*/



	$attach_files = make_array( $attach_files );

	foreach ( $attach_files as $file ) {

		if ( is_file( $file ) ) {

			$mailer->addAttachment( $file );

		}

	}



	try {

		$a = $mailer->send();

		if ( $a === false ) {

			return "Email can't send";

		}

	} catch ( phpmailerException $e ) {

		return $e->errorMessage();

	} catch ( Exception $e ) {

		return ( "Error: " . $e->getMessage() ); //Boring error messages from anything else!

	}



	$data["message"] = $message;

	if ( is_array( $to ) ) {

		$to = json_encode( $to );

	}

	$data["to"]        = $to;

	$data["sent_time"] = mysql_datetime();

	$data["subject"]   = $subject;

	insert( "email_logs", $data );



	return "OK";

}



/*function send_mail2($to, $subject, $message_text, $attach_files="", $from_email="") {

    if (localhost()) {

        debug("To: $to, Subject: $subject\nMessage: $message");

    }

    $to = make_array($to);

    include_once (ROOT_PATH."library/swiftmailer/swift_required.php");



    if (!is_email($from_email)) $from_email = 'do-not-reply@fekara.com';



    $message = Swift_Message::newInstance();



    // Give the message a subject

    $message->setSubject($subject);



    // Set the From address with an associative array

    $message->setFrom(array($from_email => 'FeKara'));



    // Set the To addresses with an associative array

    $message->setTo($to);



    // Give it a body

    $message->setBody($message_text);



    // And optionally an alternative body

    $message->addPart($message_text, 'text/html');



    // Optionally add any attachments

    //$message->attach(Swift_Attachment::fromPath('my-document.pdf'));



    // Create the Transport

    $transport = Swift_SmtpTransport::newInstance('fekara.com', 465, 'ssl')

        ->setUsername('do-not-reply@fekara.com')

        ->setPassword('woTq7HqVU;Wi');



    // Create the Mailer using your created Transport

    $mailer = Swift_Mailer::newInstance($transport);



    // Send the message

    $result = $mailer->send($message);

    var_dump($result);

}*/

function mysql_date( $date = '', $format = "Y-m-d" ) {

	if ( xtrim( $date ) == "" ) {

		return date( $format );

	} else {

		return date( $format, strtotime( $date ) );

	}

}



function mysql_datetime( $date = '', $format = "Y-m-d H:i:s" ) {

	if ( xtrim( $date ) == "" ) {

		return date( $format );

	} else {

		return date( $format, strtotime( $date ) );

	}

}



/**

 * @param $array

 *

 * Show json data..

 */

function json( $array ): void {

	header( 'Content-Type: application/json' );



	if ( ! is_array( $array ) ) {

		$array = [];

	}



	echo json_encode( $array );

	exit();

}



function is_json( $string ) {

	if ( is_numeric( $string ) ) {

		return false;

	}

	if ( is_bool( $string ) ) {

		return false;

	}

	if ( is_null( $string ) ) {

		return false;

	}

	if ( ! is_string( $string ) ) {

		return false;

	}

	@json_decode( $string );



	return ( json_last_error() == JSON_ERROR_NONE );

}



function make_array( $arr, $sep = ",", $considerLineBreaks = false ) {

	if ( is_string( $arr ) && trim( $arr ) == "" ) {

		return array();

	}

	if ( is_array( $arr ) ) {

		return $arr;

	}

	if ( is_json( $arr ) ) {

		$arr = json_decode( $arr, true );



		return $arr;

	}

	if ( $considerLineBreaks ) {

		$arr = str_replace( "\n", ",", $arr );

	}

	$arr = explode( $sep, $arr );

	$arr = array_map( 'trim', $arr );

	$arr = array_filter( $arr );



	return $arr;

}



function get_email_template( $template_name, $process_array = array() ) {

	$path = ROOT_PATH . "library/email_templates/{$template_name}.txt";

	if ( ! is_file( $path ) ) {

		return array();

	}



	$content = file( $path );

	$subject = $content[0];

	unset( $content[0] );

	$content = implode( "", $content );



	foreach ( $process_array as $key => $val ) {

		if ( is_array( $val ) ) {

			continue;

		}

		$content = str_ireplace( "{{" . $key . "}}", $val, $content );

	}



	if ( is_file( ROOT_PATH . "library/email_templates/email_signatures.txt" ) ) {

		$signatures = file_get_contents( ROOT_PATH . "library/email_templates/email_signatures.txt" );

		$content    .= $signatures;

	}



	return array( "subject" => $subject, "content" => $content );

}



function make_password( $length = 8, $possible = "2346789bcdfghjkmnpqrtvwxyzBCDFGHJKLMNPQRTVWXYZ!@#$%^&*()-><{}[]=+?" ) {

	// start with a blank password

	$password = "";



	// define possible characters - any character in this string can be

	// picked for use in the password, so if you want to put vowels back in

	// or add special characters such as exclamation marks, this is where

	// you should do it



	// we refer to the length of $possible a few times, so let's grab it now

	$maxlength = strlen( $possible );



	// check for length overflow and truncate if necessary

	if ( $length > $maxlength ) {

		$length = $maxlength;

	}



	// set up a counter for how many characters are in the password so far

	$i = 0;



	// add random characters to $password until $length is reached

	while ( $i < $length ) {



		// pick a random character from the possible ones

		$char = substr( $possible, mt_rand( 0, $maxlength - 1 ), 1 );



		// have we already used this character in $password?

		if ( ! strstr( $password, $char ) ) {

			// no, so it's OK to add it onto the end of whatever we've already got...

			$password .= $char;

			// ... and increase the counter by one

			$i++;

		}



	}



	// done!

	return $password;



}



function esc_attr( $string ) {

	$string = cleanUTF8( $string );

	$string = htmlspecialchars( $string, ENT_COMPAT, 'UTF-8' );



	return $string;

}



function cleanUTF8( $str, $force_php = false ) {

	// UTF-8 validity is checked since PHP 4.3.5

	// This is an optimization: if the string is already valid UTF-8, no

	// need to do PHP stuff. 99% of the time, this will be the case.

	// The regexp matches the XML char production, as well as well as excluding

	// non-SGML codepoints U+007F to U+009F

	if ( ! is_string( $str ) ) {

		return $str;

	}

	if ( preg_match( '/^[\x{9}\x{A}\x{D}\x{20}-\x{7E}\x{A0}-\x{D7FF}\x{E000}-\x{FFFD}\x{10000}-\x{10FFFF}]*$/Du', $str ) ) {

		return $str;

	}



	$mState = 0; // cached expected number of octets after the current octet

	// until the beginning of the next UTF8 character sequence

	$mUcs4  = 0; // cached Unicode character

	$mBytes = 1; // cached expected number of octets in the current sequence



	// original code involved an $out that was an array of Unicode

	// codepoints.  Instead of having to convert back into UTF-8, we've

	// decided to directly append valid UTF-8 characters onto a string

	// $out once they're done.  $char accumulates raw bytes, while $mUcs4

	// turns into the Unicode code point, so there's some redundancy.



	$out  = '';

	$char = '';



	$len = strlen( $str );

	for ( $i = 0; $i < $len; $i++ ) {

		$in   = ord( $str{$i} );

		$char .= $str[ $i ]; // append byte to char

		if ( 0 == $mState ) {

			// When mState is zero we expect either a US-ASCII character

			// or a multi-octet sequence.

			if ( 0 == ( 0x80 & ( $in ) ) ) {

				// US-ASCII, pass straight through.

				if ( ( $in <= 31 || $in == 127 ) && ! ( $in == 9 || $in == 13 || $in == 10 ) // save \r\t\n

				) {

					// control characters, remove

				} else {

					$out .= $char;

				}

				// reset

				$char   = '';

				$mBytes = 1;

			} elseif ( 0xC0 == ( 0xE0 & ( $in ) ) ) {

				// First octet of 2 octet sequence

				$mUcs4  = ( $in );

				$mUcs4  = ( $mUcs4 & 0x1F ) << 6;

				$mState = 1;

				$mBytes = 2;

			} elseif ( 0xE0 == ( 0xF0 & ( $in ) ) ) {

				// First octet of 3 octet sequence

				$mUcs4  = ( $in );

				$mUcs4  = ( $mUcs4 & 0x0F ) << 12;

				$mState = 2;

				$mBytes = 3;

			} elseif ( 0xF0 == ( 0xF8 & ( $in ) ) ) {

				// First octet of 4 octet sequence

				$mUcs4  = ( $in );

				$mUcs4  = ( $mUcs4 & 0x07 ) << 18;

				$mState = 3;

				$mBytes = 4;

			} elseif ( 0xF8 == ( 0xFC & ( $in ) ) ) {

				// First octet of 5 octet sequence.

				//

				// This is illegal because the encoded codepoint must be

				// either:

				// (a) not the shortest form or

				// (b) outside the Unicode range of 0-0x10FFFF.

				// Rather than trying to resynchronize, we will carry on

				// until the end of the sequence and let the later error

				// handling code catch it.

				$mUcs4  = ( $in );

				$mUcs4  = ( $mUcs4 & 0x03 ) << 24;

				$mState = 4;

				$mBytes = 5;

			} elseif ( 0xFC == ( 0xFE & ( $in ) ) ) {

				// First octet of 6 octet sequence, see comments for 5

				// octet sequence.

				$mUcs4  = ( $in );

				$mUcs4  = ( $mUcs4 & 1 ) << 30;

				$mState = 5;

				$mBytes = 6;

			} else {

				// Current octet is neither in the US-ASCII range nor a

				// legal first octet of a multi-octet sequence.

				$mState = 0;

				$mUcs4  = 0;

				$mBytes = 1;

				$char   = '';

			}

		} else {

			// When mState is non-zero, we expect a continuation of the

			// multi-octet sequence

			if ( 0x80 == ( 0xC0 & ( $in ) ) ) {

				// Legal continuation.

				$shift = ( $mState - 1 ) * 6;

				$tmp   = $in;

				$tmp   = ( $tmp & 0x0000003F ) << $shift;

				$mUcs4 |= $tmp;



				if ( 0 == --$mState ) {

					// End of the multi-octet sequence. mUcs4 now contains

					// the final Unicode codepoint to be output



					// Check for illegal sequences and codepoints.



					// From Unicode 3.1, non-shortest form is illegal

					if ( ( ( 2 == $mBytes ) && ( $mUcs4 < 0x0080 ) ) || ( ( 3 == $mBytes ) && ( $mUcs4 < 0x0800 ) ) || ( ( 4 == $mBytes ) && ( $mUcs4 < 0x10000 ) ) || ( 4 < $mBytes ) || // From Unicode 3.2, surrogate characters = illegal

					     ( ( $mUcs4 & 0xFFFFF800 ) == 0xD800 ) || // Codepoints outside the Unicode range are illegal

					     ( $mUcs4 > 0x10FFFF ) ) {



					} elseif ( 0xFEFF != $mUcs4 && // omit BOM

					           // check for valid Char unicode codepoints

					           ( 0x9 == $mUcs4 || 0xA == $mUcs4 || 0xD == $mUcs4 || ( 0x20 <= $mUcs4 && 0x7E >= $mUcs4 ) || // 7F-9F is not strictly prohibited by XML,

					             // but it is non-SGML, and thus we don't allow it

					             ( 0xA0 <= $mUcs4 && 0xD7FF >= $mUcs4 ) || ( 0x10000 <= $mUcs4 && 0x10FFFF >= $mUcs4 ) ) ) {

						$out .= $char;

					}

					// initialize UTF8 cache (reset)

					$mState = 0;

					$mUcs4  = 0;

					$mBytes = 1;

					$char   = '';

				}

			} else {

				// ((0xC0 & (*in) != 0x80) && (mState != 0))

				// Incomplete multi-octet sequence.

				// used to result in complete fail, but we'll reset

				$mState = 0;

				$mUcs4  = 0;

				$mBytes = 1;

				$char   = '';

			}

		}

	}



	return $out;

}



function human_time_diff( $from, $to = '', $show_time = true, $convert_pk_time = false ) {

	if ( empty( $to ) ) {

		$to = my_datetime();

	}



	if ( $convert_pk_time ) {

		$from = strtotime( $from . ' +5 hours' );

		$to   = strtotime( $to . ' +5 hours' );

	} else {

		$from = strtotime( $from );

		$to   = strtotime( $to );

	}

	$ago_after = "ago";

	if ( $to < $from ) {

		$ago_after = "later";

		$diff      = (int) abs( $to - $from );

	} else {

		$diff = (int) ( $to - $from );

	}



	if ( $diff <= 3600 ) {

		$mins = round( $diff / 60 );

		if ( $mins <= 1 ) {

			$mins = 1;

		}

		/* translators: min=minute */

		if ( $show_time ) {

			$since = $mins . " mins $ago_after";

		} else {

			$since = $mins . " mins";

		}

	} else if ( ( $diff <= 86400 ) && ( $diff > 3600 ) ) {

		$hours = round( $diff / 3600 );

		if ( $hours <= 1 ) {

			$hours = 1;

		}

		if ( $show_time ) {

			$since = $hours . " hour(s) $ago_after";

		} else {

			$since = $hours . " hours";

		}

	} elseif ( $diff >= 86400 ) {

		$days = round( $diff / 86400 );

		if ( $days <= 1 ) {

			$days = 1;

		}

		if ( $show_time ) {

			$since = $days . " days $ago_after";

		} else {

			$since = $days . " days";

		}

	} else {

		if ( $show_time ) {

			$since = Date( 'd-M-Y', strtotime( $from ) );

		} else {

			$since = Date( 'd-M-Y', strtotime( $from ) );

		}

	}



	return $since;

}



function string_to_color( $str ) {

	$code = dechex( crc32( $str ) );

	$code = substr( $code, 0, 6 );



	return $code;

}



if ( ! function_exists( 'is_404' ) ) {

	function is_404( $url ) {

		$handle = curl_init( $url );

		curl_setopt( $handle, CURLOPT_RETURNTRANSFER, true );

		$response = curl_exec( $handle );

		$httpCode = curl_getinfo( $handle, CURLINFO_HTTP_CODE );

		curl_close( $handle );

		if ( $httpCode == 404 ) {

			return true;

		} else {

			return false;

		}

	}

}



if ( ! function_exists( 'get_headers' ) ) {

	function get_headers( $url, $format = 0 ) {

		$url = parse_url( $url );

		$end = "\r\n\r\n";

		$fp  = fsockopen( $url['host'], ( empty( $url['port'] ) ? 80 : $url['port'] ), $errno, $errstr, 30 );

		if ( $fp ) {

			$out = "GET / HTTP/1.1\r\n";

			$out .= "Host: " . $url['host'] . "\r\n";

			$out .= "Connection: Close\r\n\r\n";

			$var = '';

			fwrite( $fp, $out );

			while ( ! feof( $fp ) ) {

				$var .= fgets( $fp, 1280 );

				if ( strpos( $var, $end ) ) {

					break;

				}

			}

			fclose( $fp );



			$var = preg_replace( "/\r\n\r\n.*\$/", '', $var );

			$var = explode( "\r\n", $var );

			if ( $format ) {

				foreach ( $var as $i ) {

					if ( preg_match( '/^([a-zA-Z -]+): +(.*)$/', $i, $parts ) ) {

						$v[ $parts[1] ] = $parts[2];

					}

				}



				return $v;

			} else {

				return $var;

			}

		}

	}

}

function get_pagination_string( $start, $end, $selected, $total_page = 0, $click_function = "ChangePage" ) {

	$output_string = "";



	if ( $start > 1 ) {

		$output_string .= "<button title='Goto first page' data-placement='right' style='margin: 0;' onclick='{$click_function}(1)' class='btn btn-xs purple-plum'>1</button>";

	}

	if ( $start > 2 ) {

		$output_string .= " .. ";

	}

	$output_string .= "<div class='btn-group btn-group-solid'>";

	for ( $x = $start; $x <= $end; $x++ ) {

		if ( $x == $selected ) {

			$class = "active";

		} else {

			$class = "purple-plum";

		}

		$output_string .= "<button style='margin: 0;' onclick='{$click_function}($x)' class='btn btn-xs $class'>$x</button> ";

	}

	$output_string .= "</div>";

	if ( $total_page > 0 and $end < $total_page - 1 ) {

		$output_string .= " .. ";

	}

	if ( $total_page > 0 and $end < $total_page ) {

		$output_string .= "<button data-placement='left' title='Goto last page' style='margin: 0;' onclick='{$click_function}($total_page)' class='btn btn-xs purple-plum'>$total_page</button>";

	}



	$output_string .= '<input onchange="' . $click_function . '(this.value)" value="' . $selected . '" min="1" max="' . $total_page . '" type="number" style="padding:5px;width:100px;margin-left: 5px;;" placeholder="Page" title="Type page number">';



	return $output_string;

}



function curl_get_file_contents( $URL ) {

	$ch = curl_init();

	curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );

	curl_setopt( $ch, CURLOPT_URL, $URL );

	curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );

	curl_setopt( $ch, CURLOPT_VERBOSE, true );

	curl_setopt( $ch, CURLOPT_FAILONERROR, true );

	$contents = curl_exec( $ch );



	if ( $contents ) {

		curl_close( $ch );



		return $contents;

	} else {

		if ( curl_errno( $ch ) ) {

			$_SESSION["curl_error"] = curl_error( $ch );

		}



		curl_close( $ch );



		return false;

	}

}



function get_http_response_code( $url ) {

	$headers = get_headers( $url );



	return substr( $headers[0], 9, 3 );

}



function encode_key( $array ) {

	if ( is_array( $array ) ) {

		$array = json_encode( $array );

	}



	$array = openssl_encrypt( $array, "AES-128-ECB", "DJd$%Fd)%f6659*##8713" );



	return $array;

}



function decode_key( $encrypted_string ) {

	$decoded = openssl_decrypt( $encrypted_string, "AES-128-ECB", "DJd$%Fd)%f6659*##8713" );



	if ( is_json( $decoded ) ) {

		$decoded = json_decode( $decoded, true );

	}



	return $decoded;

}



function array_paganation( $array, $page = 1, $pagesize = 10 ) {

	$page = $page < 1 ? 1 : $page;



	// start position in the $display_array

	// +1 is to account for total values.

	$start  = ( $page - 1 ) * ( $pagesize + 1 );

	$offset = $pagesize + 1;



	$outArray = array_slice( $array, $start, $offset );



	var_dump( $outArray );

}



/*function get_pagination_string( $start, $end, $selected, $total_page = 0, $click_function = "ChangePage" ) {

	$btn_class     = "btn-xs";

	$output_string = "";



	if ( $selected > 1 ) {

		$p             = $selected - 1;

		$output_string .= "<button title='Goto previous' data-placement='right' style='margin: 0;' onclick='{$click_function}({$p})' class='btn $btn_class purple-plum'><i class='fa fa-chevron-left'></i></button>";

	}

	if ( $start > 1 ) {

		$output_string .= "<button title='Goto first page' data-placement='right' style='margin: 0;' onclick='{$click_function}(1)' class='btn $btn_class purple-plum'>1</button> .. ";

	}

	$output_string .= "<div class='btn-group btn-group-solid'>";

	for ( $x = $start; $x <= $end; $x ++ ) {

		if ( $x == $selected ) {

			$class = "active";

		} else {

			$class = "purple-plum";

		}

		$output_string .= "<button style='margin: 0;' onclick='{$click_function}($x)' class='btn $btn_class $class'>$x</button> ";

	}



	$output_string .= "</div>";

	if ( $total_page > 0 && $end < $total_page ) {

		$output_string .= " .. <button data-placement='left' title='Goto last page' style='margin: 0;' onclick='{$click_function}($total_page)' class='btn $btn_class purple-plum'>$total_page</button>";

	}

	if ( $selected < $total_page ) {

		$p             = $selected + 1;

		$output_string .= "<button title='Goto next' data-placement='right' style='margin: 0;' onclick='{$click_function}({$p})' class='btn $btn_class purple-plum'><i class='fa fa-chevron-right'></i></button>";

	}

	$output_string .= " ";



	return $output_string;

}*/



/**

 * @param $phone

 *

 * @return bool

 *

 * Check if given string is a valid phone number.

 */

function is_phone_number( $phone ) {

	$phone = trim( (string) $phone );



	if ( substr( $phone, 0, 2 ) == '00' ) {

		$phone = substr_replace( $phone, '+', 0, 2 );

	}



	try {

		$number = PhoneNumber::parse( $phone );



		return $number->isValidNumber();

	} catch ( PhoneNumberParseException $e ) {

		return false;

	}

}